------------------------------------------------------------------------------------------------
-- Starcraft 2 Model Objects version 0.1
-- by NiNtoxicated (madyavic@gmail.com)
-- by Leruster (leruster@gmail.com)
-- 3ds max objects for use with StarCraft II model files (.M3)
-- Created for 3ds max 2010, but should be backwards compatible to 3ds Max 9 SP2
--
-- Objects:
-- sc2material - Material plugin for setting material and layer properties
-- sc2bitmap - Bitmap plugin with extra map options
-- sc2attachment - Helper plugin to add attachments to a scene
-- sc2boundsphere -- Bounding sphere plugin for custom bounding sphere
-- sc2pemitter -- Particle emitter plugin
-- sc2light -- Light source
-- sc2ribbon -- ribbon effect helper
-- sc2tight -- tight hit test object
-- sc2fuzzy -- fuzzy hit test object
--
-- User Interfaces:
-- sc2animUI - Animation interface for viewing and editing animations
-- sc2boneFlags - bone flags management interface
--
------------------------------------------------------------------------------------------------
--
-- Version History:
-- 0.1 - December 23rd 2012
-- Added tight and fuzzy hit tests
-- Added bone flags support
-- Added Bone Tools interface to manage bone flags
-- Added ribbon effect support
-- Changes in Particle Emitters. Some values have new names, places. Some are properly described.
-- Changes in layer system (mainly due to particle materials compatibility)
-- Added layer managment when importing objects
-- Scene check and Add bones buttons are now available from the M3-Export menu
-- Modified lights according to Blizzard's pdf
-- m3 scripts contain foundation for forces, sRibbons and projections
-- Fixed bug when trying to import .m3 file with volume materials
-- Bone count after import is now as it is in original file (necessary for proper ribbon and particles animations import)
-- Particle Copies are temporarily taken out

-- 0.08 - July 3rd 2012
-- Added sc2light, StarCraft II Light source
-- Added volume materials support
-- Changes in layers system and minor in standard material

-- 0.07 - May 13th 2012
-- Added Displace Material type to the StarCraft II Material;

-- 0.06 - March 5th 2011
-- Added sc2pemitter particle emitter support
-- Added sc2attachment volume support
-- Added sc2boundsphere custom bound sphere support
-- Added sc2bitmap animation support
-- Improved sc2attachment wireframe display
-- Improved sc2bitmap support
-- Miscellaneous code fixes and improvements
--
-- 0.05c - August 15th 2010
-- Fixed animation object bug
--
-- 0.05b - July 22nd 2010
-- Attachments now retain previous settings
-- Added two new attachment ID presets
--
-- 0.05 - July 15th 2010
-- Added sc2bitmap custom map support
-- Added sc2bitmap brightness multiplier
-- Added to sc2bitmap more UV options
--
-- 0.04b - June 29th 2010
-- Fixed Start frame minusing by 1 bug in animation UI
--
-- 0.04 - June 28th 2010
-- Added attachment custom object
-- Updated material, now supports Terrain (Null) textures
-- Updated decal entry in material, now supports decals properly
-- Updated bitmap to support map channels
-- Fixed same range animation UI error
--
-- 0.03b - June 12th 2010
-- Fixed Animation data holder not being updated correctly
--
-- 0.03 - June 9th 2010
-- Updated Animation UI heavily to include editing capabilities
-- Added callback support for Animation UI
-- Added view button to sc2bitmap materials
--
-- 0.02 - May 17th 2010
-- sc2bitmap definition updated
-- Integrated animation UI
--
-- 0,01 - May 15th 2010
-- Initial objects script version
-- sc2material defined
-- sc2bitmap defined
--
------------------------------------------------------------------------------------------------
-- *****************
--  MISC GLOBALS
-- *****************

global SC2PLUGIN_VERS = 0.07

-- ***********************
--  GLOBAL FUNCTIONS
-- ***********************
fn sc2uiOpenFile ftypes &fname =
(
	local ret 
	if (fname != undefined) then
	(
		ret = getOpenFileName types:ftypes filename:fname
	)
	else
	(
		ret = getOpenFileName types:ftypes
	)
	if ret != undefined then fname = ret
)

fn sc2_toStrArray aray =
(
	local aClass = classOf aray as string
	if (aClass == "Array") then
	(
		local aReturn = #()
		for i = 1 to aray.count do
		(
			local aStr = aray[i] as string
			append aReturn aStr
		)
		
		return aReturn
	)
	else
	(
		return undefined
	)
)

-- ***************
--  M3 OBJECTS
-- *************** 
plugin Material sc2material
name:"StarCraft II Material" 
classID:#(0x1ccf0cb1, 0x1e0c2826)
extends:Standard replaceUI:true version:1
( 
	local teamColourIDlist = #("Custom", "White", "Red", "Blue", "Teal", "Purple", "Yellow", "Orange", "Green", "Light Pink", "Violet", "Light Grey", "Dark Green", "Brown", "Light Green", "Dark Grey", "Pink")
	local teamColourDiffList = #([255,255,255], [180,20,30], [0,66,255], [28,167,234], [84,0,129], [235,225,41], [254,138,14], [22,128,0], [204,166,252], [31,1,201], [82,84,148], [16,98,70], [78,42,4], [150,255,145], [35,35,35], [229,91,176])
	local teamColourDiffListStr = sc2_toStrArray teamColourDiffList
	local teamColourEmissList = #()
	local mattypeui, matsettui, flagsui, layersui, layers, dissettui, dislayersui, dislayers, volsett, volsettui, vollayers, vollayersui
	local prevRollouts = #(matsettui, layersui, flagsui)
	local isCreated = false
	
	fn TeamColourSetupUI =
	(
		local tc = this.TeamColourDiff as point3 -- convert from colour to p3
		tc = tc as string -- convert to string for comparison
		local tcFind = findItem teamColourDiffListStr tc
		if (tcFind > 0) then
		(
			this.TeamColourID = tcFind + 1
		)
		else
		(
			this.TeamColourID = 1
		)
	)
	
	parameters matsett rollout:matsettui
	(
		blendMode type:#integer default:1 ui:ddlBlendMode
		priority type:#integer default:0 ui:spnPriority
		cutoutThresh type:#integer default:0 ui:spnCutoutThresh
		specVal type:#float default:0 ui:spnSpec
		specMult type:#float default:1 ui:spnSpecMult
		emisMult type:#float default:1 ui:spnEmisMult
		layerBlendType type:#integer default:3 ui:ddlLayerBlend
		emissiveBlendType type:#integer default:3 ui:ddlEmissiveBlend
		specF type:#float default:0.0 --ui:spnspecF
		specType type:#integer defalut:0 --ui:spnspecType
		emisMode type:#integer default:2 ui:ddlEmissive2Blend
		
		on specVal set val do delegate.specularLevel = val
	)
	
	parameters dissett rollout:dissettui
	(
		priorityDis type:#integer default:0 ui:spnPriorityDis
		globStr type:#float default:0.0 ui:spnglobStr animatable:true
	)
	
	parameters volsett rollout:volsettui
	(
		volDensity type:#float default:0.15 ui:spnvolDensity animatable:true
		--volFloat1 type:#float default:1 ui:spnvolFloat1
		--volFloat2 type:#float default:1 ui:spnvolFloat2
		--volInt1 type:#integer default:0 ui:spnvolInt1
		--volInt2 type:#integer default:32 ui:spnvolInt2

	)
	
	parameters flags rollout:flagsui
	(
		OF1 type:#boolean --ui:chkOF1
		OF2 type:#boolean --ui:chkOF2
		OF3 type:#boolean --ui:chkOF3
		OF4 type:#boolean --ui:chkOF4
		
		Unfogged type:#boolean ui:chkUnfogged
		TwoSided type:#boolean ui:chkTwoSided
		Unshaded type:#boolean ui:chkUnshaded
		NoHitTest type:#boolean ui:chkNoHitTest
		DepthPrepass type:#boolean ui:chkDepthPrepass
		UseTerrainHDR type:#boolean ui:chkTerrainHDR
		SplatUVfix type:#boolean ui:chkSplatUV
		SoftBlending type:#boolean ui:chkSoftBlend
		NoShadowsCast type:#boolean ui:chkNoShadowsCast
		NoShadowsReceived type:#boolean ui:chkNoShadowsRec
		ForParticles type:#boolean ui:chkForParticles
		
		on TwoSided set val do delegate.twoSided = val
	)
	
	parameters layers rollout:layersui
	(
		-- for team colour preview
		teamColourDiff type:#color default:[255,255,255] ui:clpTeamColourDiff
		teamColourEmiss type:#color default:[255,255,255] ui:clpTeamColourEmiss
		teamColourID type:#integer default:1 ui:ddlTeamColourID
		
		compMap type:#textureMap
		diffuseMap type:#textureMap ui:mbDiffuse subAnim:true
		decalMap type:#textureMap ui:mbDecal subAnim:true
		specularMap type:#textureMap ui:mbSpec subAnim:true
		selfIllumMap type:#textureMap ui:mbEmis subAnim:true
		emisMap2 type:#textureMap ui:mbEmis2 subAnim:true
		envioMap type:#textureMap ui:mbEnvio subAnim:true
		envioMaskMap type:#textureMap ui:mbEnvioMask subAnim:true
		alphaMaskMap type:#textureMap ui:mbAlphaMask subAnim:true
		bumpMap type:#textureMap ui:mbNorm subAnim:true
		heightMap type:#textureMap ui:mbHeight subAnim:true
		
		
		-- Set Team Colours
		on teamColourID set val do
		(
			if (val != 1) then
			(
				local tcInd = val - 1
				teamColourDiff = teamColourDiffList[tcInd]
			)
		)
		on teamColourDiff set val do
		(
			delegate.diffuse = val
			this.TeamColourSetupUI()
		)
		
		on teamColourEmiss set val do
		(
			-- not working yet
			-- delegate.selfIllumColor = val
		)
		
		-- Set Maps
		on diffuseMap set val do 
		(
			compMap.mapList[1] = val
		)
		
		on decalMap set val do
		(
			local dmap = val
			if (dmap != undefined) then 
			(
				local dmapClass = classOf dmap as string
				if (dmapClass == "sc2bitmap") then
				(
					dmap.isDecal = true
					dmap.preMultAlpha = false
				)
			)
			compMap.mapList[2] = dmap
		)
		
		on compMap set val do
		(
			delegate.diffuseMap = val
		)
		
		on specularMap set val do delegate.specularMap = val
		on selfIllumMap set val do delegate.selfIllumMap = val
		on bumpMap set val do delegate.bumpMap = val
		--on envioMap set val do delegate.ReflectionMap = val
	)
	
	parameters dislayers rollout:dislayersui
	(
		normDisMap type:#textureMap ui:mbDNorm subAnim:true
		strenghtDisMap type:#textureMap ui:mbDstrenght subAnim:true
	)
	
	parameters vollayers rollout:vollayersui
	(
		vDiffMap type:#textureMap ui:mbVDiff subAnim:true
		vUnkn01 type:#textureMap subAnim:true --ui:mbVUnkn01 
		vUnkn02 type:#textureMap subAnim:true --ui:mbVUnkn02 
	)
	
	parameters mattype rollout:mattypeui
	(
		materialType type:#integer default:2 ui:ddlMaterialType
		
		
		on materialType set val do
		(
			
			if (this.isCreated == true) then
			(
				for i in 1 to prevRollouts.count do
				(
						removeRollout this.prevRollouts[i]
				)
				
				case val of
				(
					1: 
					(
						--removeRollout this.matsettui
						for i in 1 to prevRollouts.count do
						(
								removeRollout this.prevRollouts[i]
						)	
						
						-- blank maps
						this.diffuseMap 		= undefined
						this.decalMap 			= undefined
						this.specularMap 		= undefined
						this.selfIllumMap 		= undefined
						this.emisMap2			= undefined
						this.envioMap 			= undefined
						this.envioMaskMap	= undefined
						this.alphaMaskMap	= undefined
						this.bumpMap			= undefined
						this.heightMap			= undefined
						--this.bumpDisMap		= undefined
						--this.heightDisMap		= undefined
						
						prevRollouts = #()
						
						
					)
					2: 
					(
						if (prevRollouts == #()) then
							(
								addRollout this.matsettui
								addRollout this.flagsui
								addRollout this.layersui
							)
							else
							(
								for i in 1 to prevRollouts.count do
								(
									removeRollout this.prevRollouts[i]
								)
								
								addRollout this.matsettui
								addRollout this.flagsui
								addRollout this.layersui
								
								prevRollouts = #()
								prevRollouts[1] = this.matsettui
								prevRollouts[2] = this.flagsui
								prevRollouts[3] = this.layersui
							)
					)
					
					3:
					(
						for i in 1 to prevRollouts.count do
						(
								removeRollout this.prevRollouts[i]
						)	
							
						addRollout this.dissettui
						addRollout this.flagsui
						addRollout this.dislayersui
						
						prevRollouts = #()
						prevRollouts[1] = this.dissettui
						prevRollouts[2] = this.flagsui
						prevRollouts[3] = this.dislayersui
					)
					4:
					(
						for i in 1 to prevRollouts.count do
						(
								removeRollout this.prevRollouts[i]
						)
							
						addRollout this.volsettui
						addRollout this.vollayersui
						
						prevRollouts = #()
						prevRollouts[1] = this.volsettui
						prevRollouts[2] = this.vollayersui
					)
				)
			)
		)
	)
	
	rollout mattypeui "Material Type" category:1
	(
		dropdownlist ddlMaterialType "Material Type:" items:#("Terrain (Null)", "Standard", "Displace", "Volume") width:120
		
		on mattypeui open do
		(
			-- necessary for proper add/remove rollout in plugin
			if this.isCreated == false then 
			(
				this.isCreated = true
			)
		)
	)
	
	rollout matsettui "Standard Material Settings" category:2
	(
		group "Blending"
		(
			label lbl1 "Blend Mode:" across:2 align:#left offset:[0,3] tooltip:"Your mom"
			dropdownlist ddlBlendMode "" items:#("Opaque", "Alpha Blend", "Add", "Alpha Add",  "Mod", "Mod 2x") align:#left width:100 offset:[-60,0] tooltip:"Blend mode.\nSpecifies how the entire model with this material will be rendered on top of other elements."
			label lbl2 "Layer Blend:" across:2 align:#left offset:[0,3]
			dropdownlist ddlLayerBlend "" items:#("Mod", "Mod 2x", "Add", "Blend", "Team Colour Emissive Add", "Team Colour Diffuse Add") selection:3 align:#left width:100 offset:[-60,0]
			label lbl3 "Emissive Blend:" across:2 align:#left offset:[0,3]
			dropdownlist ddlEmissiveBlend "" items:#("Mod", "Mod 2x", "Add", "Blend", "Team Colour Emissive Add", "Team Colour Diffuse Add") selection:3 align:#left width:100 offset:[-60,0]
			label lbl4 "Emissive 2 Blend:" across:2 align:#left offset:[0,3]
			dropdownlist ddlEmissive2Blend "" items:#("None", "Screen", "Add") selection:0 align:#left width:100 offset:[-60,0]
		)

		group "Properties"
		(
			label lbl5 "Priority:" height:16 across:4 align:#left
			spinner spnPriority "" width:60 align:#left offset:[-24,0] type:#integer
			label lbl6 "Specularity:" height:16 align:#left offset:[20,0]
			spinner spnSpec "" width:60 align:#left offset:[13,0] type:#float
			label lbl7 "Cutout:" height:16 across:4 align:#left offset:[0,0]
			spinner spnCutoutThresh "" width:60 align:#left offset:[-24,0] type:#float
			label lbl8 "Emiss. Mult.:" height:16 align:#left offset:[20,0]
			spinner spnEmisMult "" width:60 align:#left offset:[13,0] type:#float
			label lbl9 "Spec. Mult.:" height:16 align:#right offset:[81,0] across:2
			spinner spnSpecMult "" width:60 align:#right offset:[0,0] type:#float
			
			
			/*spinner spnPriority "Priority:" width:80 align:#left range:[0,1e9,0] type:#integer across:2
			spinner spnCutoutThresh "Cutout Threshold: " fieldwidth:40 range:[0,1e9,0] type:#integer scale:1
			spinner spnSpec "Specularity: " fieldwidth:45 range:[0,1e9,0] scale:1 align:#left across:2
			spinner spnEmisMult "HDR Emiss. Mult.: " fieldwidth:45 range:[0,1e9,1] scale:1 align:#right
			--spinner spnspecF "SpecularF:" fieldwidth:45 range:[0,10,0] scale:1 align:#left across:2
			spinner spnSpecMult "HDR Spec. Mult.: " fieldwidth:45 range:[0,1e9,1] scale:1 align:#right 
			--spinner spnspecType "SpecType:" fieldwidth:40 range:[0,10,0] type:#integer*/
		)
		
		on matsettui open do
		(
			case materialType of
			(
				1: removeRollout this.matsettui
				3: removeRollout this.matsettui
				4: removeRollout this.matsettui
			)
		)
	)
	
	rollout dissettui "Displace Material Settings" category:2
	(
		group "Properties"
		(
			spinner spnPriorityDis "Priority: " fieldwidth:40 range:[0,1e9,0] type:#integer align:#left across:2
			spinner spnglobStr "Global strength" default:0.0 scale:0.1 width:85 align:#left tooltip:"Global Strength.\nGlobal strength of the displace effect. Scale factor of the Strength Map parameters."
		)
		
		on dissettui open do
		(
			case materialType of
			(
				1: removeRollout this.dissettui
				2: removeRollout this.dissettui
				4: removeRollout this.dissettui
			)
		)
	)
	
	rollout volsettui "Volume Material Settings" category:2
	(
		group "Properties"
		(
			spinner spnvolDensity "Density: " fieldwidth:40 range:[0,1,0.15] type:#float align:#left across:3
			--spinner spnvolInt1 "Integer1:" fieldwidth:40 range:[-1e9,1e9,0] type:#integer align:#left
			--spinner spnvolFloat1 "Float1: " fieldwidth:40 range:[-1e9,1e9,0] type:#integer align:#left
			--spinner spnvolFloat2 "Float2: " fieldwidth:40 range:[-1e9,1e9,0] type:#integer align:#left
			--spinner spnvolInt2 "Integer2:" fieldwidth:40 range:[-1e9,1e9,32] type:#integer align:#left across:3
		)
		
		on volsettui open do
		(
			case materialType of
			(
				1: removeRollout this.volsettui
				2: removeRollout this.volsettui
				3: removeRollout this.volsettui
			)
		)
	)
	
	rollout flagsui "Material Flags" category:4
	(
		/*group "Object Flags"
		(
			checkbox chkOF1 "OF1" align:#left across:2
			checkbox chkOF2 "OF2"
			checkbox chkOF3 "OF3" align:#left across:2			
			checkbox chkOF4 "OF4" align:#left
		)*/
		
		group "Flags"
		(
			checkbox chkUnfogged "Unfogged" align:#left across:3
			checkbox chkTwoSided "Two Sided"
			checkbox chkUnshaded "Unshaded"
			
			checkbox chkNoHitTest "No Hit Test" align:#left across:3
			checkbox chkDepthPrepass "Depth Pre-pass"
			checkbox chkTerrainHDr "Use Terrain HDR" 
			
			checkbox chkSplatUV "Splat UV Fix" align:#left across:3
			checkbox chkSoftBlend "Soft Blending"			
			checkbox chkNoShadowsCast "No Shadows Cast"
			
			checkbox chkNoShadowsRec "No Shadows Received"
			checkbox chkForParticles "ForParticles"
		)
		
		on flagsui open do
		(
			case materialType of
			(
				1: removeRollout this.flagsui
				4: removeRollout this.flagsui
			)			
		)
	)
	
	rollout dislayersui "Displace Map Layers" category:3
	(
		group "Displace Map Layers"
		(
			label lblDNorm "Normal Map: " across:2 align:#left
			mapbutton mbDNorm "None" tooltip:"Select Normal Map" offset:[-45,0] width:175
			label lblDStrength "Strength Map: " across:2 align:#left
			mapbutton mbDStrenght "None" tooltip:"Select Strenght Map" offset:[-45,0] width:175
		)
		
		on dislayersui open do
		(
			case materialType of
			(
				1: removeRollout this.dislayersui
				2: removeRollout this.dislayersui
				4: removeRollout this.dislayersui
			)
		)
	)
	
	rollout vollayersui "Volume Map Layers" category:3
	(
		group "Volume Map Layers"
		(
			label lblVDiff "Color Map: " across:2 align:#left
			mapbutton mbVDiff "None" tooltip:"Select Difuse Map" offset:[-45,0] width:175
			--label lblVUnkn01 "Unknown01 Map: " across:2 align:#left
			--mapbutton mbVUnkn01 "None" tooltip:"Select Unknown01 Map" offset:[-45,0] width:175
			--label lblVUnkn02 "Unknown02 Map: " across:2 align:#left
			--mapbutton mbVUnkn02 "None" tooltip:"Select Unknown02 Map" offset:[-45,0] width:175
		)
		
		on vollayersui open do
		(
			case materialType of
			(
				1: removeRollout this.vollayersui
				2: removeRollout this.vollayersui
				3: removeRollout this.vollayersui
			)
		)
	)
	
	rollout layersui "Standard Material Layers" category:3
	(
		group "Team Colour Preview"
		(
			dropdownlist ddlTeamColourID "Team Colour:" items:teamColourIDlist align:#left width:85 across:3
			colorPicker clpTeamColourDiff "Diffuse:" color:[255,255,255] align:#right height:30
			colorPicker clpTeamColourEmiss "Emissive:" color:[255,255,255] align:#left height:30 enabled:false visible:false
		)
		
		group "Map Layers"
		(
			label lblDiffuse "Diffuse Map: " across:2 align:#left
			mapbutton mbDiffuse "None" tooltip:"Select Diffuse Map" offset:[-45,0] width:175
			label lblDecal "Decal Map: " across:2 align:#left
			mapbutton mbDecal "None" tooltip:"Select Decal Map" offset:[-45,0] width:175
			label lblSpec "Specular Map: " across:2 align:#left
			mapbutton mbSpec "None" tooltip:"Select Specular Map" offset:[-45,0] width:175
			label lblEmis "Emissive Map: " across:2 align:#left
			mapbutton mbEmis "None" tooltip:"Select Emissive Map" offset:[-45,0] width:175
			label lblEmis2 "Emissive Map 2: " across:2 align:#left
			mapbutton mbEmis2 "None" tooltip:"Select Emissive Map" offset:[-45,0] width:175
			label lblEnvio "Envio Map: " across:2 align:#left
			mapbutton mbEnvio "None" tooltip:"Select Envio Map" offset:[-45,0] width:175
			label lblEnvioMask "Envio Mask Map: " across:2 align:#left
			mapbutton mbEnvioMask "None" tooltip:"Select Envio Mask Map" offset:[-45,0] width:175
			label lblAlphaMask "Alpha Mask Map: " across:2 align:#left
			mapbutton mbAlphaMask "None" tooltip:"Select Alpha Mask Map" offset:[-45,0] width:175
			label lblNorm "Normal Map: " across:2 align:#left
			mapbutton mbNorm "None" tooltip:"Select Normal Map" offset:[-45,0] width:175
			label lblHeight "Height Map: " across:2 align:#left
			mapbutton mbHeight "None" tooltip:"Select Height Map" offset:[-45,0] width:175
		)
		
		on layersui open do
		(
			this.TeamColourSetupUI()
			case materialType of
			(
				1: removeRollout this.layersui
				3: removeRollout this.layersui
				4: removeRollout this.layersui
			)
		)
	)
		
	on create do
	(
		-- setup initial material
		delegate.bumpMapAmount = 100
		
		-- initialise composite
		compMap = compositeTextureMap()
		compMap.mapList.count = 2
		
	)
	
	on update do
	(
		if version == 1 then
		(
			this.compMap = compositeTextureMap()
			this.compMap.mapList.count = 2
			if (delegate.diffuseMap != undefined) then
			(
				local classMap = classOf delegate.diffuseMap as string
				if (classMap != "CompositeTexturemap") then 
				(
					this.compMap.mapList[1] = diffuseMap
					this.compMap.mapList[2] = decalMap
					delegate.diffuseMap = compMap
				)
			)
			this.isCreated = false
			this.materialType = 2
		)
	)
)

plugin textureMap sc2bitmap
name:"StarCraft II Layer"
classID:#(0x48f387a2, 0x6950179f)
autoPromoteDelegateProps:true
extends:Bitmaptexture replaceUI:true version:5
remap:#(#("TexWrap"),#("TexWrap2"))
(
	local isCreated = false
	local mappingSourceTypeList = #("Default","UV layer 2","UV layer 3","UV layer 4","UNK 4","UNK 5","Animated Particle UV","UNK 7","UNK 8","UNK 9","UNK 10")
	
	fn assign_bitmap =
	(
		local bmap = SelectBitmap()
		if (bmap != undefined) then
		(
			-- update bitmap
			delegate.bitmap = bmap
		)
		else
		(
			-- else leave it as it was if no custom map detected
			if (this.scCustomMap.count < 1) then
			(
				bmap = this.scbitmap
			)
		)
		return bmap
	)
	
	fn alphaSetup =
	(
		-- setup bitmap alpha
		if (this.TeamColour == true) then
		(
			delegate.alphasource = 0
		)
		else
		(
			delegate.alphasource = 2
		)
	)
	
	parameters map rollout:mapui
	(
		scbitmap type:#bitmap
		scCustomMap type:#string default:""
		isDecal type:#boolean default:false
		
		on scbitmap set val do
		(
			if (scbitmap != undefined) then
			(
				local scbitmapClass = classOf scbitmap as string
				if (scbitmapClass == "BitMap") then
				(
					if (scbitmap.filename.count > 0) then
					(
						delegate.bitmap = scbitmap
						if (isDecal == true) then delegate.preMultAlpha = false
					)
					else
					(
						delegate.bitmap.filename = ""
					)
				)
			)
		)
		
		useColour type:#boolean default:false ui:chkuseColour
		layerColour type:#point4 default:[0.0,0.0,0.0,0.0] animatable:true ui:pckLayerColour
		FalloffStrength type:#float default:4.0 ui:spnFalloffStrength animatable:true
		FalloffCutout type:#float default:1.0 ui:spnFalloffCutout animatable:true
		FalloffStart type:#float default:0.0 ui:spnFalloffStart animatable:true
		FalloffUnk type:#float default:1.0 animatable:true --ui:spnFalloffUnk
		falloffUse type:#boolean default:false ui:chkFalloffUse
		falloffAlpha type:#boolean default:false ui:chkFalloffAlpha
		FalloffF3 type:#boolean default:false --ui:chkFalloffF3 
		FalloffF4 type:#boolean default:false --ui:chkFalloffF4
	)--end params
	
	parameters mapsett rollout:mapsettui
	(
		-- General Settings
		Bright_Mult type:#float default:1.0 ui:spnBright_Mult animatable:true
		MidtoneOffset type:#float default:0.0 ui:spnMidtoneOffset animatable:true range:[-1e9,1e9,0]
		inverseMap	type:#boolean default:false ui:chkInverseMap animatable:false
		
		mappingSource type:#integer default:1 ui:ddlmappingSource
		-- Coordinate Settings
		
		U_Offset type:#float default:0.0 ui:spnU_Offset animatable:true
		V_Offset type:#float default:0.0 ui:spnV_Offset animatable:true
		U_Tiling type:#float default:1.0 ui:spnU_Tiling animatable:true
		V_Tiling type:#float default:1.0 ui:spnV_Tiling animatable:true
		
		U_Tile type:#boolean default:true ui:chkU_Tile
		V_Tile type:#boolean default:true ui:chkV_Tile
		
		unkMapInt type:#integer default:-1
		
		uLF1 type:#boolean default:false
		uLF2 type:#boolean default:false
		uLF3 type:#boolean default:false
		
		on mappingSource set val do
		(
			delegate.coords.mapChannel = val
		)
		on U_Offset set val do
		(
			delegate.coords.U_Offset = val
		)
		on V_Offset set val do 
		(
			delegate.coords.V_Offset = val
		)
		on U_Tiling set val do
		(
			delegate.coords.U_Tiling = val
		)
		on V_Tiling set val do
		(
			delegate.coords.V_Tiling = val
		)
		on U_Tile set val do delegate.coords.U_Tile = val
		on V_Tile set val do delegate.coords.V_Tile = val

	)
	
	parameters alphasett rollout:alphasettui
	(
		-- Alpha Flags
		TeamColour type:#boolean ui:chkTeamColour
		TexAlphaOnly type:#boolean ui:chkTexAlphaOnly
		ShadeBasedOnAlpha type:#boolean ui:chkShadeBasedOnAlpha
		
		on TeamColour set val do
		(
			alphaSetup()
		)
		on TexAlphaOnly set val do
		(
			alphaSetup()
		)
		on ShadeBasedOnAlpha set val do
		(
			alphaSetup()
		)
	)


	rollout mapui "Layer Color properties"
	(
		local mapSet
		
		group "Map"
		(
			label	lblBitmap "Bitmap:" align:#left across:2
			button btnBitmap "None" offset:[-55,0] width:260 text:"None"
			--button btnClearImage "Clear" align:#right enabled:true offset:[75,0] across:2
			button btnViewImage "View Image" align:#right enabled:false
		)
		
		rollout mapPick "Map Type"
		(
			group "Bitmap Type"
			(
				radiobuttons mapType labels:#("3ds Max Bitmap", "Custom Bitmap")
			)
			
			label lblNote "Note: Custom maps won't render in max!" align:#center width:125 height:32 offset:[0,5]
			
			button bOK "OK" across:2 width:66 height:20
			button bCancel "Cancel" width:66 height:20
			
			on bOK pressed do
			(
				case mapType.state of
				(
					1:	mapui.mapSet = #maxmap
					2: mapui.mapSet = #custom
				)
				
				Destroydialog mapPick
			)
			on bCancel pressed do
			(
				mapui.mapSet = undefined
				Destroydialog mapPick
			)
		)
		
		on mapui open do
		(
			if (scCustomMap.count > 0) then
			(
				btnBitmap.text = scCustomMap
				btnViewImage.enabled = false
			)
			else
			(
				if (scbitmap != undefined) then
				(
					btnBitmap.text = scbitmap.filename
					btnViewImage.enabled = true
				)
				else btnBitmap.text = "None"
			)
		)
		on btnBitmap pressed do
		(
			local newPos = mouse.screenpos
			newPos.x -= 85
			newPos.y -= 95
			if (createDialog mapPick modal:true pos:newPos width:173 height:138) then
			(
				case mapui.mapSet of
				(
					#maxmap:
					(
						local map = assign_bitmap()
						if (map != undefined) then
						(
							this.scbitmap = map
							this.scCustomMap = ""
							btnBitmap.text = this.scbitmap.filename
							btnViewImage.enabled = true
						)
					)
					#custom: 
					(
						sc2uiOpenFile "OGV Movie Files (*.ogv)|*.ogv|All Files|*.*|" &scCustomMap
						if (scCustomMap.count > 0) then 
						(
							if (this.scbitmap != undefined) then 
							(
								local tBitMap 
								--tBitMap.filename = ""
								--this.scbitmap = tBitMap
								delegate.filename = ""
							)
							btnBitmap.text = scCustomMap
							btnViewImage.enabled = false
						)
					)
				)
			)
		)
		on btnViewImage pressed do
		(
			if (scbitmap != undefined) then
			(
				display scbitmap
			)
		)
		on mapui reload do
		(
			if (scCustomMap.count > 0) then
			(
				btnBitmap.text = scCustomMap
				btnViewImage.enabled = false
			)
			else
			(
				if (scbitmap != undefined) then
				(
					btnBitmap.text = scbitmap.filename
					btnViewImage.enabled = true
				)
				else btnBitmap.text = "None"
			)
		)
		
		group "Color"
		(
			
			checkbox chkuseColour "Use Color:" checked:false align:#left offset:[0,1] across:2
			colorPicker pcklayerColour "" align:#left color:(layerColour as color) height:18 fieldWidth:49 alpha:true enabled:true offset:[-80,0] animatable:true
			on pcklayerColour changed NewColour do
			(
				format "Przed: %\n" layerColour
				layerColour = NewColour
				format "Po: %\n" layerColour
			)
			
		)
		
		group "Falloff"
		(
			checkbox chkFalloffUse "Use Falloff" offset:[1,0]

			label lbl6 "Strength:" height:16 across:2 align:#left offset:[0,5]
			spinner spnFalloffStrength "" width:60 align:#left offset:[-99,5]
			label lbl7 "Start:" across:2 align:#left
			spinner spnFalloffStart "" width:60 range:[-1,1,0] align:#left offset:[-99,0]
			label lbl5 "Cutout:" height:16 across:2 align:#left
			spinner spnFalloffCutout "" width:60 range:[-1e9,1e9,0] offset:[-99,0] align:#left
			
			--spinner spnFalloffUnk "Unk" width:60 range:[-1e9,1e9,0] align:#left --offset:[-99,0]
			checkbox chkFalloffAlpha "use on Alpha" offset:[1,5] across:3 height:18
			--checkbox chkFalloffF3 "usecolor?" offset:[1,6]
			--checkbox chkFalloffF4 "unknown" offset:[1,6]
			
			/*on chkFalloffUse changed theState do
			(
				lbl6.enabled = lbl5.enabled = lbl7.enabled = spnFalloffStrength.enabled = spnFalloffLevel.enabled = spnFalloffStart.enabled = chkFalloffAlpha.enabled = theState
			)*/
			
		)
	)--end rollout
	
	rollout mapsettui "Map Settings"
	(
		/*group "Unknown Labels"
		(
			checkbox chkuLF1 "uLF1" checked:false align:#left
			checkbox chkuLF2 "uLF1" checked:false align:#left
			checkbox chkuLF3 "uLF1" checked:false align:#left
		)*/
		group "General Properties"
		(
			label lbl10 "Brightness Multiplier:" across:4 align:#left
			spinner spnBright_Mult "" default:1.0 scale:0.1 width:45 align:#left offset:[25,0]
			label lbl11 "Midtone offset:" align:#left offset:[25,0]
			spinner spnMidtoneOffset "" default:0.0 scale:0.1 width:45 align:#left range:[-1e9,1e9,0] offset:[25,0]
			checkbox chkInverseMap "Inverse Colors" default:false align:#left
		)
		
		group "UV Coordinates"
		(
			label lbl1 "Mapping source: " align:#left across:2
			dropdownlist ddlmappingSource "" items:mappingSourceTypeList width:130 offset:[-66,-2]
			
			label lblOffset "Offset" align:#left offset:[25, 0] across:3
			label lblTiling "Tiling" align:#left offset:[5,0]
			label lblTile "Tile" align:#left offset:[-37,0]
			
			label lblU "U:" across:4 align:#left
			spinner spnU_Offset "" type:#float default:1.0 scale:0.1 width:60 align:#left offset:[-60, 0] range:[-1e9, 1e9, 0]
			spinner spnU_Tiling "" type:#float default:1.0 scale:0.1 width:60 align:#left offset:[-60, 0] range:[-1e9, 1e9, 0]
			checkbox chkU_Tile "" checked:true align:#left offset:[-60, 0]
			
			label lblV "V:" across:4 align:#left width:10
			spinner spnV_Offset "" type:#float default:1.0 scale:0.1 width:60 align:#left offset:[-60, 0] range:[-1e9, 1e9, 0]
			spinner spnV_Tiling "" type:#float default:1.0 scale:0.1 width:60 align:#left offset:[-60, 0] range:[-1e9, 1e9, 0]
			checkbox chkV_Tile "" checked:true align:#left offset:[-60, 0]
		)
		
		fn set_decal =
		(
			if (this.isDecal == true) then
			(
				if (this.isCreated != true) then
				(
					this.mappingSource = 2
					--this.TexWrap = false
					this.TeamColour = true
					this.isCreated = true
				)
			)
		)
		
		on mapsettui open do
		(
			set_decal()
		)
		on mapsettui reload do
		(
			set_decal()
		)
	)
	
	rollout alphasettui "Alpha Settings"
	(
		group "Alpha Flags"
		(
			checkbox chkTeamColour "Alpha as Team Colour" align:#left across:2 checked:false
			checkbox chkTexAlphaOnly "Render alpha only" align:#right checked:false 
			checkbox chkShadeBasedOnAlpha "Alpha-based shading" align:#left checked:false 
		)	
		
		on chkTexAlphaOnly changed state do
		(
			if (state == false) then
			(
				chkTeamColour.enabled = true
			)
			else
			(
				chkTeamColour.checked = false
				chkTeamColour.enabled = false
			)
		)
		on mapsettui open do
		(
			if (chkTexAlphaOnly.checked == true) then
			(
				chkTeamColour.checked = false
				chkTeamColour.enabled = false
			)
		)
		on mapsettui reload do
		(
			if (chkTexAlphaOnly.checked == true) then
			(
				chkTeamColour.checked = false
				chkTeamColour.enabled = false
			)			
		)		
	)
	
	on create do
	(
		alphaSetup()
		
		if (this.isDecal == true) then
		(
			this.mapingSource = 2
			--this.TexWrap = false
			this.TeamColour = true
		)
	)
	
	on update do
	(
		if (version < 4) then
		(
			this.isCreated = false
			this.isDecal = false
			this.mapingSource = 1
		)
		
		if (version < 5) then
		(
			if (delegate.coords.U_Tile == true) then this.U_Tile = true else this.U_Tile = false
			if (delegate.coords.V_Tile == true) then this.V_Tile = true else this.V_Tile = false
			
			if (delegate.bitmap != undefined) then
			(
				this.scbitmap = delegate.bitmap
			)
		)
	)
)

global sc2att_size = 0.05
global sc2att_rad = 0.2
global sc2att_id = ""
global sc2att_colour = green

plugin Helper sc2attachment
name:"Attachment"
classID:#(0x2e993b39, 0x43824ff0)
category:"Starcraft 2 Objects"
extends:Dummy
version:4
(
	local AttachIDlist = #("Custom", "Ref_Center", "Ref_Damage", "Ref_Hardpoint", "Ref_Head", "Ref_Hit", "Ref_Origin", "Ref_Overhead", "Ref_Shield", "Ref_Target", "Ref_Weapon", "Ref_Weapon Left", "Ref_Weapon Right")
	local meshObj, meshPyr, meshSph, lastSize, meshNode, lastRad 
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	parameters main rollout:mainui
	(
		boxSize type:#float ui:spnBoxSize
		
		on boxSize set val do
		(
			sc2att_size = val
		)
	)
	
	parameters attachsett rollout:attachsettui
	(
		attachID type:#integer ui:ddlAttachID
		attachName type:#string ui:edtAttachID
		attachRadius type:#float ui:spnAttachRad
		showVolume type:#boolean ui:cbnShowVolume
		
		on attachID set val do
		(
			if (val > 1) then
			(
				attachName = AttachIDlist[val]
			)
		)
		on attachName set val do
		(
			if (val != "" and val != undefined) then
			(
				local attInd = findItem AttachIDlist val
				if (attInd < 2) then 
				(
					attachID = 1 
				)
				else 
				(
					attachID = attInd
				)
				sc2att_id = val
			)
			else
			(
				attachID = 1
			)
		)
		on attachRadius set val do
		(
			sc2att_rad = val
		)
	)
	
	rollout mainui "Helper Parameters"
	(
		spinner spnBoxSize "Size:" range:[0, 1e9, sc2att_size] type:#float scale:0.01
	)
	
	rollout attachsettui "Attachment Parameters"
	(
		group "Select Attachment ID"
		(
			dropdownlist ddlAttachID items:AttachIDlist
		)
		edittext edtAttachID "Attachment ID:" labelOnTop:true
		group "Attachment Volume"
		(
			spinner spnAttachRad "Radius" enabled:false align:#left type:#float scale:0.01 range:[0, 1e9, sc2att_rad]
			checkButton cbnShowVolume "Show Volume" 
		)
		
		fn spnRadOnOff val =
		(
			if ((matchPattern val pattern:"Ref_Target*") or (matchPattern val pattern:"Ref_Shield*")) then
			(
				spnAttachRad.enabled = true
				cbnShowVolume.enabled = true
			)
			else
			(
				spnAttachRad.enabled = false
				cbnShowVolume.enabled = false
				showVolume = false
			)
		)
		
		on ddlAttachID selected i do
		(
			if (i > 1) then
			(
				edtAttachID.text = ddlAttachID.selected
			)
			-- need to call the function to activate spinner if target/shield selected from dropdownlist
			edtAttachID.entered ddlAttachID.selected
		)
		on edtAttachID entered val do
		(
			local attInd = findItem AttachIDlist val
			if (attInd < 2) then ddlAttachID.selection = 1 else ddlAttachID.selection = attInd
			spnRadOnOff val
		)
		on attachsettui open do
		(
			if (edtAttachID.text != "" and edtAttachID.text != undefined) then
			(
				local attInd = findItem AttachIDlist edtAttachID.text
				if (attInd < 2) then ddlAttachID.selection = 1 else ddlAttachID.selection = attInd
			)
			edtAttachID.entered edtAttachID.text
		)
	)
	
	on getDisplayMesh do
	(
		if (showVolume == true) then
		(
			if (meshSph == undefined) then
			(
				meshSph = createInstance sphere radius:attachRadius mapCoords:false
				lastRad = attachRadius
			)
		)
		else
		(
			if (meshPyr == undefined) then
			(
				meshPyr = createInstance pyramid depth:boxSize width:boxSize height:(boxSize * 2) mapCoords:false
				lastSize = boxSize
			)
		)
		
		if ((boxSize != lastSize) and (this.showVolume == false)) then
		(
			meshPyr.depth = meshPyr.width = boxSize
			meshPyr.height = boxSize * 2
			lastSize = boxSize
		)
		
		if ((attachRadius != lastRad) and (this.showVolume == true)) then
		(
			meshSph.radius = attachRadius
			lastRad = attachRadius
		)
		
		if (this.showVolume == true) then
		(
			return meshSph.mesh
		)
		else
		(
			return meshPyr.mesh
		)
	)
	
	on create do
	(
		boxSize = sc2att_size
		attachName = sc2att_id
		attachRadius = sc2att_rad
	)

	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2att_colour
				--delegate.boxsize = [boxSize, boxSize, boxSize]
			)
			else #stop
		)
	) 
)

global sc2par2_color  = (color 255 0 0)

plugin Helper sc2pemitter2
name: "Pemitter"
classID:#(0x4bc621e5, 0x4ab7f1d9)
category:"Starcraft 2 Objects"
extends:Dummy
replaceUI:true

(
	--local particleTypeList = #("Point", "Plane", "Sphere", "Box", "Cylinder (1)", "Disc", "Spline", "Planar Billboard", "Planar", "Cylinder (2)", "Starshaped")
	--local emissionAreaTypeList = #("Cylinder","Box","Sphere","Plane","Point")
	local emissionAreaTypeList = #("Point", "Plane", "Sphere", "Box", "Cylinder (1)", "Disc", "Spline", "Planar Billboard", "Planar", "Cylinder (2)", "Starshaped")
	local emissionTypeList = #("Directional","Cylindrycal","2","3","4","5","6","7")
	--local particleTypeList = #("Square billboards","Speed scaled","Rectangular billboards","Velocity aligned")
	local particleTypeList = #("0 - Square Billboards","1 - Speed Scaled","2 - Square Billboards","3 - Square Billboards","4 - Square Billboards","5 - Square Billboards","6 - Rectangular Billboards","7 - Square velocity aligned")
	local meshObj, meshNode
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	fn intToBool integ =
	(
		boolean = undefined
		if (integ==1) then (boolean = true) else (boolean = false)
		return boolean
	)
	
	parameters particleEmission rollout:particleEmissionui
	(
		pFSimulateOnInit type:#boolean animatable:false ui:chkpFSimulateOnInit
		emissionSpeed1 type:#float animatable:true ui:spnemissionSpeed1
		emissionSpeed2 type:#float animatable:true ui:spnemissionSpeed2
		randomizeWithEmissionSpeed2 type:#integer animatable:false ui:spnrandomizeWithEmissionSpeed2
		emissionAngleX type:#float animatable:true ui:spnemissionAngleX
		emissionAngleY type:#float animatable:true ui:spnemissionAngleY
		emissionSpreadX type:#float animatable:true ui:spnemissionSpreadX
		emissionSpreadY type:#float animatable:true ui:spnemissionSpreadY
		lifespan1 type:#float animatable:true ui:spnlifespan1
		lifespan2 type:#float animatable:true ui:spnlifespan2
		randomizeWithLifespan2 type:#integer animatable:false ui:spnrandomizeWithLifespan2
		--usedModel
		--copyIndices
		--indexPlusHighestIndex type:#integer animatable:false ui:spnindexPlusHighestIndex
		maxParticles type:#integer animatable:false ui:spnmaxParticles
		emissionRate type:#float animatable:true ui:spnemissionRate
		emissionAreaType type:#integer animatable:false ui:ddlemissionAreaType default:5
		emissionAreaSizeX type:#float animatable:true ui:spnemissionAreaSizeX
		emissionAreaSizeY type:#float animatable:true ui:spnemissionAreaSizeY
		emissionAreaSizeZ type:#float animatable:true ui:spnemissionAreaSizeZ
		emissionAreaRadius type:#float animatable:true ui:spnemissionAreaRadius
		spreadUnk type:#float animatable:true ui:spnspreadUnk
		emissionType type:#integer animatable:false ui:ddlemissionType default:3
		partEmit type:#integer animatable:true ui:spnpartEmit
		lengthWidthRatio type:#float animatable:false ui:spnlengthWidthRatio
		pFInheritParentVel type:#boolean animatable:false ui:chkpFInheritParentVel
	)
	
	parameters particleBehaviour rollout:particleBehaviourui
	(
		pFCollideTerrain type:#boolean animatable:false ui:chkpFCollideTerrain
		pFCollideObjects type:#boolean animatable:false ui:chkpFCollideObjects
		zAcceleration type:#float animatable:false ui:spnzAcceleration
		unknownFloat1a type:#float animatable:false ui:spnunknownFloat1a
		unknownFloat1b type:#float animatable:false ui:spnunknownFloat1b
		unknownFloat1c type:#float animatable:false ui:spnunknownFloat1c
		unknownFloat1d type:#float animatable:false ui:spnunknownFloat1d
		
		rotationValuesX type:#float animatable:false ui:spnrotationValuesX
		rotationValuesY type:#float animatable:false ui:spnrotationValuesY
		rotationValuesZ type:#float animatable:false ui:spnrotationValuesZ
		
		slowdown type:#float animatable:false ui:spnslowdown
		unknownFloat2a type:#float animatable:false --ui:spnunknownFloat2a
		unknownFloat2b type:#float animatable:false --ui:spnunknownFloat2b
		unknownFloat2c type:#float animatable:false --ui:spnunknownFloat2c
		unknown4 type:#float animatable:false --ui:spnunknown4
		
		forceFlags type:#integer animatable:false --ui:spnforceFlags
		forceFlagsCopy type:#integer animatable:false --ui:spnforceFlagsCopy
		unknownFloat3a type:#float animatable:false --ui:spnunknownFloat3a
		unknownFloat3b type:#float animatable:false --ui:spnunknownFloat3b
		unknownFloat3c type:#float animatable:false-- ui:spnunknownFloat3c
		unknownFloat3d type:#float animatable:false --ui:spnunknownFloat3d
		
		
		
		randomizeWithRotationValues2 type:#integer animatable:false ui:spnrandomizeWithRotationValues2
		rotationValues2X type:#float animatable:true ui:spnrotationValues2X
		rotationValues2y type:#float animatable:true ui:spnrotationValues2y
		rotationValues2Z type:#float animatable:true ui:spnrotationValues2Z
		pFSmoothRotation type:#boolean animatable:false ui:chkpFSmoothRotation
		pFBezSmoothRotation type:#boolean animatable:false ui:chkpFBezSmoothRotation
		
		unknown7 type:#integer animatable:false --ui:spnunknown7
		
	
		unknownFloat4 type:#float animatable:false --ui:spnunknownFloat4
		unknownFloat5 type:#float animatable:false --ui:spnunknownFloat5
		unknown9 type:#integer animatable:false --ui:spnunknown9
	)
	
	parameters particleTrails rollout:particleTrailsUI 
	(
		trailingEnabled type:#integer animatable:false ui:spntrailingEnabled
		tailUnk1X type:#float animatable:true ui:spntailUnk1X
		tailUnk1Y type:#float animatable:true ui:spntailUnk1Y
		tailUnk1Z type:#float animatable:true ui:spntailUnk1Z
		pFClampTailParts type:#boolean animatable:false ui:chkpFClampTailParts
		pFSpawnTrailingParts type:#boolean animatable:false ui:chkpFSpawnTrailingParts
		pFFixLengthTailParts type:#boolean animatable:false ui:chkpFFixLengthTailParts
	)
	
	parameters particleLook rollout:particleLookUI
	(
		particleMaterial type:#material ui:mtlParticleMaterial
		
		particleSizesX type:#float animatable:false ui:spnparticleSizesX
		particleSizesY type:#float animatable:false ui:spnparticleSizesY
		particleSizesZ type:#float animatable:false ui:spnparticleSizesZ
		
		initialColor1 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckinitialColor1
		finalColor1 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckfinalColor1
		unknownColor1 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckunknownColor1
		
		randomizeWithParticleSizes2 type:#integer animatable:false ui:spnrandomizeWithParticleSizes2
		particleSizes2X type:#float animatable:true ui:spnparticleSizes2X
		particleSizes2Y type:#float animatable:true ui:spnparticleSizes2Y
		particleSizes2Z type:#float animatable:true ui:spnparticleSizes2Z
		pFSmoothSize type:#boolean animatable:false ui:chkpFSmoothSize
		pFBezSmoothSize type:#boolean animatable:false ui:chkpFBezSmoothSize
		
		randomizeWithColor2 type:#integer animatable:false ui:spnrandomizeWithColor2
		initialColor2 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckinitialColor2
		finalColor2 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckfinalColor2
		unknownColor2 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckunknownColor2
		pFSmoothColor type:#boolean animatable:false ui:chkpFSmoothColor
		pFBezSmoothColor type:#boolean animatable:false ui:chkpFBezSmoothColor
		
		phase1StartImageIndex type:#integer animatable:false ui:spnphase1StartImageIndex
		phase1EndImageIndex type:#integer animatable:false ui:spnphase1EndImageIndex
		phase2EndImageIndex type:#integer animatable:false ui:spnphase2EndImageIndex
		phase2StartImageIndex type:#integer animatable:false ui:spnphase2StartImageIndex
		relativePhase1Length type:#float animatable:false ui:spnrelativePhase1Length
		numberOfColumns type:#integer animatable:false ui:spnnumberOfColumns
		numberOfRows type:#integer animatable:false ui:spnnumberOfRows
		columnWidth type:#float animatable:false ui:spncolumnWidth
		rowHeight type:#float animatable:false ui:spnrowHeight
		pFRandFlipBookStart type:#boolean animatable:false ui:chkpFRandFlipBookStart
		particleType type:#integer animatable:false ui:ddlparticleType
	)
	
	parameters particleFlags rollout:particleFlagsUI
	(
		--FLAGS--
		pFsort type:#boolean animatable:false ui:chkpFsort
		
		pFSpawnOnBounce type:#boolean animatable:false ui:chkpFSpawnOnBounce
		pFUseInnerShape type:#boolean animatable:false ui:chkpFUseInnerShape
		pFInheritEmissionParams type:#boolean animatable:false ui:chkpFInheritEmissionParams
		
		pFSortByZHeight type:#boolean animatable:false ui:chkpFSortByZHeight
		pFReverseIteration type:#boolean animatable:false ui:chkpFReverseIteration
		
		
		
		pFLitParts type:#boolean animatable:false ui:chkpFLitParts
		
		pFMultiplyByGravity type:#boolean animatable:false ui:chkpFMultiplyByGravity
		pFUseVertexAlpha type:#boolean animatable:false ui:chkpFUseVertexAlpha
		pFModelParts type:#boolean animatable:false ui:chkpFModelParts
		pFSwapYZonModelParts type:#boolean animatable:false ui:chkpFSwapYZonModelParts
		pFScaleTimeByParent type:#boolean animatable:false ui:chkpFScaleTimeByParent
		pFUseLocalTime type:#boolean animatable:false ui:chkpFUseLocalTime
		
		pFCopy type:#boolean animatable:false ui:chkpFCopy
		--END OF FLAGS--
	)
	
	
	rollout particleEmissionui "Emission settings"
	(
		fn boneFilter obj =
		(
			classof obj == Bonegeometry
		)
		
		label lblParent "Parent: "  across:2 align:#left width:40 offset:[0,8]
		pickbutton btnPickBone "Pick bone" align:#left width:80 offset:[-10,5] filter:boneFilter autoDisplay:False
		
		on btnPickBone picked obj do
		( 
			if (obj!=undefined) then
			(
				btnPickBone.text = obj.name
				$.parent= obj
				$.pos = obj.pos
				alignToParent $
			)
		)
		
		on particleEmissionui open do 
		(
			for o in selection where o.baseObject == this do
				if (o.parent != undefined) do btnPickBone.text = o.parent.name;
		)
		
		checkbox chkpFSimulateOnInit "Simulate on initialization"
		group "Emitter settings"
		(
			label emitlbl1 "Direction: " align:#left across:2 width:60 offset:[0,0]
			dropdownlist ddlemissionType "" items:emissionTypeList width:92 offset:[-20,-2] type:#integer
			--spinner spnemissionType "spnemissionType"
			label emitlbl2 "Shape: " align:#left width:60 across:2 offset:[12,0]
			dropdownlist ddlemissionAreaType "" items:emissionAreaTypeList width:93 offset:[-20,-2] type:#integer default:5
			--spinner spnemissionAreaType "spnemissionAreaType"
			spinner spnemissionAreaSizeX "Length:" enabled:(if (emissionAreaType==2 or emissionAreaType==3) then true else false)
			spinner spnemissionAreaSizeY "Width:" enabled:(if (emissionAreaType==2 or emissionAreaType==3) then true else false)
			spinner spnemissionAreaSizeZ "Height:" enabled:(if (emissionAreaType==2 or emissionAreaType==3) then true else false)
			spinner spnemissionAreaRadius "Radius: " enabled:(if (emissionAreaType==1 or emissionAreaType==3) then true else false)
			spinner spnlengthWidthRatio "Length/Width:" offset:[0,5]
		)
		
		group "Emission"
		(
			spinner spnemissionRate "Rate:"
			spinner spnemissionSpeed1 "Speed:"
			--checkbutton displaySpeed "Helper" width:70 height:18 align:#right tooltip:"Display speed helper toggle." offset:[1,0]
			
			checkbox spnrandomizeWithEmissionSpeed2 "" across:2 align:#left width:20
			spinner spnemissionSpeed2 "Variation:" align:#right width:162 enabled:(intToBool randomizeWithEmissionSpeed2)
			checkbox chkpFInheritParentVel "Inherit parent velocity"
			spinner spnemissionAngleX "X angle:" width:102 offset:[46,5] across:2 align:#right
			label lblangle1 "deg" offset:[0,5] align:#right
			spinner spnemissionAngleY "Y angle:" width:102 offset:[46,0] across:2 align:#right
			label lblangle2 "deg" align:#right
			spinner spnemissionSpreadX "X spread:" width:110 offset:[46,0] across:2 align:#right
			label lblangle3 "deg" align:#right
			spinner spnemissionSpreadY "Y spread:" width:110 offset:[46,0] across:2 align:#right
			label lblangle4 "deg" align:#right
		)
		
		group "Lifespan"
		(
			spinner spnlifespan1 "Lifespan:" width:102 offset:[60,0] across:2 align:#right
			label lblspan1 "s" align:#right
			checkbox spnrandomizeWithLifespan2 "" across:3 align:#left width:20
			spinner spnlifespan2 "Variation:" align:#right width:137 enabled:(intToBool randomizeWithLifespan2) offset:[38,0]
			label lblspan2 "s" align:#right offset:[2,0]
		)
		
		--spinner spnindexPlusHighestIndex "spnindexPlusHighestIndex" type:#integer
		spinner spnmaxParticles "Max Particles:"
		
		
		spinner spnspreadUnk "Spread Unk:"
		
		spinner spnpartEmit "PartEmit:" type:#integer
		
		on spnrandomizeWithEmissionSpeed2 changed val do
		(
			if (val==true) then
			(
				spnemissionSpeed2.enabled = true
			)
			else
			(
				spnemissionSpeed2.enabled = false
			)
		)
		
		on spnrandomizeWithLifespan2 changed val do
		(
			if (val==true) then
			(
				spnlifespan2.enabled = true
			)
			else
			(
				spnlifespan2.enabled = false
			)
		)
		
		/*on ddlemissionAreaType selected val do
		(
			case val of
			(
				1:
				(
					spnemissionAreaSizeX.enabled = false
					spnemissionAreaSizeY.enabled =  false
					spnemissionAreaSizeZ.enabled = true
					spnemissionAreaRadius.enabled = true
				)
				2:
				(
					spnemissionAreaSizeX.enabled = true
					spnemissionAreaSizeY.enabled = true
					spnemissionAreaSizeZ.enabled = true
					spnemissionAreaRadius.enabled =  false
				)
				3:
				(
					spnemissionAreaSizeX.enabled =  false
					spnemissionAreaSizeY.enabled =  false
					spnemissionAreaSizeZ.enabled =  false
					spnemissionAreaRadius.enabled = true
				)
				4:
				(
					spnemissionAreaSizeX.enabled = true
					spnemissionAreaSizeY.enabled = true
					spnemissionAreaSizeZ.enabled =  false
					spnemissionAreaRadius.enabled =  false
				)
				5:
				(
					spnemissionAreaSizeX.enabled =  false
					spnemissionAreaSizeY.enabled =  false
					spnemissionAreaSizeZ.enabled =  false
					spnemissionAreaRadius.enabled =  false
				)
			)	
		)*/
		
	)
	
	rollout particleBehaviourUI "Particle Behaviour"
	(
		checkbox chkpFCollideTerrain "Collide with terrain"
		checkbox chkpFCollideObjects "Collide with objects"
		spinner spnzAcceleration "Z acceleration:"
		spinner spnslowdown "Drag:"
		spinner spnunknownFloat1a "Float1a"
		spinner spnunknownFloat1b "Float1b"
		spinner spnunknownFloat1c "Float1c"
		spinner spnunknownFloat1d "Float1d"
		
		group "Rotation"
		(
			spinner spnrotationValuesX "X:"
			spinner spnrotationValuesY "Y:"
			spinner spnrotationValuesZ "Z:"
			checkbox spnrandomizeWithRotationValues2 "Variation"
			spinner spnrotationValues2X "X variation:" enabled:(intToBool randomizeWithRotationValues2)
			spinner spnrotationValues2y "Y variation:" enabled:(intToBool randomizeWithRotationValues2)
			spinner spnrotationValues2Z "Z variation:" enabled:(intToBool randomizeWithRotationValues2)
			
			checkbox chkpFSmoothRotation "Smooth rotation" offset:[0,5]
			checkbox chkpFBezSmoothRotation "Smooth Bezier rotation"
			
		)
		
		
		--spinner spnunknownFloat2a "spnunknownFloat2a"
		--spinner spnunknownFloat2b "spnunknownFloat2b"
		--spinner spnunknownFloat2c "spnunknownFloat2c"
		--spinner spnunknown4 "spnunknown4"
		

		--spinner spnforceFlags "spnforceFlags"
		--spinner spnforceFlagsCopy "spnforceFlagsCopy"
		--spinner spnunknownFloat3a "spnunknownFloat3a"
		--spinner spnunknownFloat3b "spnunknownFloat3b"
		--spinner spnunknownFloat3c "spnunknownFloat3c"
		--spinner spnunknownFloat3d "spnunknownFloat3d"
		
		--spinner spnunknown7 "spnunknown7"
		
		
		--spinner spnunknownFloat4 "spnunknownFloat4"
		--spinner spnunknownFloat5 "spnunknownFloat5"
		--spinner spnunknown9 "spnunknown9"
		
		
		on spnrandomizeWithRotationValues2 changed val do
		(
			if (val==true) then
			(
				spnrotationValues2X.enabled = true
				spnrotationValues2Y.enabled = true
				spnrotationValues2Z.enabled = true
			)
			else
			(
				spnrotationValues2X.enabled = false
				spnrotationValues2Y.enabled = false
				spnrotationValues2Z.enabled = false
			)
		)
	)
	
	rollout particleTrailsUI "Trails"
	(
		checkbox spntrailingEnabled "Enable trails"
		spinner spntailUnk1X "X:"
		spinner spntailUnk1Y "Y:"
		spinner spntailUnk1Z "Z:"
		checkbox chkpFClampTailParts "Clamp tail particles"
		checkbox chkpFSpawnTrailingParts "Spawn trailing particles"
		checkbox chkpFFixLengthTailParts "Fix trailing particles length"
	)
	
	rollout particleLookUI "Rendering"
	(
		label parlooklbl1 "Shape:" align:#left across:2 width:70 offset:[10,0]
		dropdownlist ddlparticleType "" items:particleTypeList width:92 offset:[-20,-2] tooltip:"Sets the particle shape if it's not defined by instanced geometry."
		--spinner spnparticleType "Particle type:" particleTypeList
		label parlooklbl2 "Material:" across:2 align:#left offset:[2,0]
		materialButton mtlParticleMaterial "Pick Material" width:93 offset:[5,-3] align:#right
		
		group "Bitmap settings"
		(
			spinner spnnumberOfColumns "Columns:" type:#integer
			spinner spnnumberOfRows "Rows:" type:#integer
			spinner spncolumnWidth "Column width:" offset:[0,5] type:#float
			spinner spnrowHeight "Row height:" type:#float
			label parlooklbl3 "Phase one:" align:#left offset:[0,5] 
			spinner spnphase1StartImageIndex "Start index:" type:#integer
			spinner spnphase1EndImageIndex "End index:" type:#integer
			label parlooklbl4 "Phase two:" align:#left offset:[0,2] 
			spinner spnphase2StartImageIndex "Start index:" type:#integer
			spinner spnphase2EndImageIndex "End index:" type:#integer
			spinner spnrelativePhase1Length "Phase length:" offset:[0,5]
			checkbox chkpFRandFlipBookStart "Random flipbook start"
		)
		
		group "Size"
		(
			spinner spnparticleSizesX "X:"
			spinner spnparticleSizesY "Y:"
			spinner spnparticleSizesZ "Z:"
			checkbox spnrandomizeWithParticleSizes2 "Enable Variation:" align:#left offset:[0,5]
			spinner spnparticleSizes2X "X variation:" enabled:(intToBool randomizeWithParticleSizes2)
			spinner spnparticleSizes2Y "Y variation:" enabled:(intToBool randomizeWithParticleSizes2)
			spinner spnparticleSizes2Z "Z variation:" enabled:(intToBool randomizeWithParticleSizes2)
			
			checkbox chkpFSmoothSize "Smooth size" offset:[0,5]
			checkbox chkpFBezSmoothSize "Smooth Bezier size"
		)
		
		group "Coloring"
		(
			label colorlbl1 "Start color: " across:2 align:#left width:60
			colorPicker pckinitialColor1 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2]
			label colorlbl2 "Middle color: " across:2 align:#left width:60
			colorPicker pckfinalColor1 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2]
			label colorlbl3 "End color: " across:2 align:#left width:60
			colorPicker pckunknownColor1 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2]
			checkbox spnrandomizeWithColor2 "Color variation:" offset:[0,5]
			label colorlbl4 "Start color: " across:2 align:#left width:60 enabled:(intToBool randomizeWithColor2)
			colorPicker pckinitialColor2 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2] enabled:(intToBool randomizeWithColor2)
			label colorlbl5 "Middle color: " across:2 align:#left width:60 enabled:(intToBool randomizeWithColor2)
			colorPicker pckfinalColor2 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2] enabled:(intToBool randomizeWithColor2)
			label colorlbl6 "End color: " across:2 align:#left width:60 enabled:(intToBool randomizeWithColor2)
			colorPicker pckunknownColor2 "" height:18 fieldWidth:49 alpha:true enabled:true animatable:true offset:[0,-2] enabled:(intToBool randomizeWithColor2)
			
			checkbox chkpFSmoothColor "Smooth color" offset:[0,5]
			checkbox chkpFBezSmoothColor "Smooth Bezier color"
		)
		
		on spnrandomizeWithParticleSizes2 changed val do
		(
			if (val==true) then
			(
				spnparticleSizes2X.enabled = true
				spnparticleSizes2Y.enabled = true
				spnparticleSizes2Z.enabled = true
			)
			else
			(
				spnparticleSizes2X.enabled = false
				spnparticleSizes2Y.enabled = false
				spnparticleSizes2Z.enabled = false
			)
		)
		
		on spnrandomizeWithColor2 changed val do
		(
			if (val==true) then
			(
				colorlbl4.enabled = true
				pckinitialColor2.enabled = true
				colorlbl5.enabled = true
				pckfinalColor2.enabled = true
				colorlbl6.enabled = true
				pckunknownColor2.enabled = true
			)
			else
			(
				colorlbl4.enabled = false
				pckinitialColor2.enabled = false
				colorlbl5.enabled = false
				pckfinalColor2.enabled = false
				colorlbl6.enabled = false
				pckunknownColor2.enabled = false
			)
		)
	)
	
	rollout  particleFlagsUI "Flags"
	(
		checkbox chkpFsort "Sort particles"
		
		checkbox chkpFSpawnOnBounce "Spawn particles on bounce"
		checkbox chkpFUseInnerShape "Use internal shape"
		checkbox chkpFInheritEmissionParams "Inherit emis. parameters"
		
		checkbox chkpFSortByZHeight "Sort particles by Z-index"
		checkbox chkpFReverseIteration "Reverse iteration"
		
		
		
		checkbox chkpFLitParts "Lit particles"
		
		checkbox chkpFMultiplyByGravity "Multiply by gravity"
		
		checkbox chkpFUseVertexAlpha "Use vertex alpha"
		checkbox chkpFModelParts "Model particles"
		checkbox chkpFSwapYZonModelParts "Swap Y-Z model particles"
		checkbox chkpFScaleTimeByParent "Scale time by parent"
		checkbox chkpFUseLocalTime "Use local time"
		
		checkbox chkpFCopy "Copy system"
	)
	
	fn createMesh =
	(
		if (meshObj == undefined) then
		(
			meshObj = createInstance pyramid depth:0.5 width:0.5 height:0.8 mapCoords:false 
			lastSize = boxSize
		)
		
		if (boxSize != lastSize) then
		(
			meshObj.depth = meshObj.width = meshObj.height = boxSize 
			lastSize = boxSize
		)
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			createMesh();
		)
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2par2_color
			)
			else #stop
		)
	) 
)

/*global sc2force_color  = (color 255 144 0)

plugin Helper sc2force
name: "Force"
classID:#(0x66b0e9e7, 0x378770eb)
category:"Starcraft 2 Objects"
extends:Dummy
replaceUI:true

(
	local meshObj
	local forceTypeList = #("0","1","2","3")
	
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	parameters viewportDisplay rollout:uiDisplayOptions
	(
		radiusDisplay type:#float ui:spnRadiusDisplay default:0.1
		
		on radiusDisplay set val do
		(
			this.createMesh()
		)
	)
	
	parameters main rollout:mainui
	(
		forceType type:#integer ui:ddlForceType default:1
		
		forceStrenght type:#float ui:spnForceStrenght default:1 animatable:true
		forceRange type:#float ui:spnForceRange default:1 animatable:true
		forceUnk1 type:#float ui:spnForceUnk1 default:1 animatable:true
		forceUnk2 type:#float ui:spnForceUnk2 default:1 animatable:true
	)

	
	rollout uiDisplayOptions "Display Options"
	(
		spinner spnRadiusDisplay "Radius: " range:[0.0,500.0,0.02] scale:0.001
	)
	
	rollout mainui "Main Settings"
	(
		fn boneFilter obj =
		(
			classof obj == Bonegeometry
		)
		
		label lblParent "Parent: "  across:2 align:#left width:40 offset:[0,3]
		pickbutton btnPickBone "Pick bone" align:#left width:80 offset:[-10,0] filter:boneFilter

		on btnPickBone picked obj do
		( 
			if (obj!=undefined) then
			(
				btnPickBone.text = obj.name
				$.parent= obj
				$.pos = obj.pos
				alignToParent $
			)
		)
		
		label lbl1 "Type: " align:#left across:2 width:27 offset:[0,3]
		dropdownlist ddlForceType "" items:forceTypeList width:110 offset:[-35,1]
		
		group "Force Settings"
		(
			spinner spnForceStrenght "Strength"
			spinner spnForceRange "Range"
			spinner spnForceUnk1 "Unknown1"
			spinner spnForceUnk2 "Unknown2"
		)
	)
	
	fn createMesh =
	(
		temp_node = Sphere radius:radiusDisplay segments:5 mapcoords:on
		meshObj = snapshotAsMesh temp_node;
		delete temp_node;
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			createMesh();
		)
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2force_color
			)
			else #stop
		)
	) 
)*/

global sc2ribbon_color  = (color 255 88 199)

plugin Helper sc2ribbon
name: "Ribbon"
classID:#(0xe916974, 0x3962bb59)
category:"Starcraft 2 Objects"
extends:Dummy
replaceUI:true

(
	
	local ribbonTypeList = #("Planar Billboarded","Planar","Cylindrical","Star-shaped")
	local meshObj, defObj, sRibBone
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	fn boneFilter obj =
	(
		classof obj == Bonegeometry
	)
	
	fn intToBool integ =
	(
		boolean = undefined
		if (integ==1) then (boolean = true) else (boolean = false)
		return boolean
	)
	
	parameters viewportDisplay rollout:uiDisplayOptions
	(
		amplitudeViz type:#float ui:spnAmplitudeViz default:0.02
		on amplitudeViz set val do
		(
			this.createMesh()
		)
	)
	
	parameters main rollout:mainui
	(
		ribbonMaterial type:#material ui:mtlRibbonMaterial
		
		on iconSize set val do
		(
			-- sc2bndsph_rad = val
		)
		
		waveLength type:#float ui:spnWaveLength default:1.5 animatable:true
		tipOffsetZ type:#float ui:spnTipOffsetZ default:0.0 animatable:false
		endRadius type:#float ui:spnEndRadius default:0.5 animatable:false
		
		Anim1 type:#float default:1.0 animatable:true --ui:spnAnim1 
		Anim2 type:#float default:1.0 animatable:true --ui:spnAnim2 
		Anim3 type:#float default:1.0 animatable:true --ui:spnAnim3 
		Anim4 type:#float default:1.0 animatable:true --ui:spnAnim4 
		Anim5 type:#float default:1.0 animatable:true --ui:spnAnim5 
		Anim6 type:#float default:1.0 animatable:true --ui:spnAnim6 
		
		BGRAref1 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckBGRAref1
		BGRAref2 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckBGRAref3
		BGRAref3 type:#point4 default:[1.0,1.0,1.0,1.0] animatable:true ui:pckBGRAref3
		
		startRadiusScale type:#float ui:spnStartRadiusScale default:1.0 animatable:true
		centerRadiusScale type:#float ui:spnCenterRadiusScale default:0.5 animatable:true
		endRadiusScale type:#float ui:spnEndRadiusScale default:0.01 animatable:true
		
		ribbonTwist type:#float ui:spnRibbonTwist default:0.0 animatable:true
		
		surfaceNoiseAmplitude type:#float ui:spnSurfaceNoiseAmplitude default:0.0 animatable:false
		surfaceNoiseNumberOfWaves type:#float ui:spnSurfaceNoiseNumberOfWaves default:0.0 animatable:false
		surfaceNoiseFrequency type:#float ui:spnSurfaceNoiseFrequency default:0.0 animatable:false
		surfaceNoiseScale type:#float ui:spnSurfaceNoiseScale default:0.0 animatable:false
		
		ribbonType type:#integer ui:ddlRibbonType default:2
		
		ribbonDivisions type:#float ui:spnRibbonDivisions default:10.0 animatable:false
		ribbonSides type:#integer ui:spnRibbonSides default:5 animatable:false 
		ribbonLength type:#float ui:spnRibbonLength default:1.0 animatable:true
		
		on waveLength set val do
		(
			this.createMesh();
		)
		
		on ribbonDivisions set val do
		(
			this.createMesh();
		)
		
		on ribbonSides set val do
		(
			this.createMesh();
		)
		
		on ribbonLength set val do
		(
			this.createMesh()
		)
		
		on startRadiusScale set val do
		(
			this.createMesh()
 		)
-- 		
 		on endRadiusScale set val do
 		(
 			this.createMesh()
 		)
		
		on ribbonType set val do
		(
			this.createMesh()
		)
	)
	
	/*parameters srib rollout:sribui
	(
		enblSRib ui:chkEnblSRib type:#boolean default:false
		
		sRibFloat1 ui:spnsRibFloat1 type:#float
		sRibFloat2 ui:spnsRibFloat2 type:#float
		sRibFloat3 ui:spnsRibFloat3 type:#float
		sRibFloat4 ui:spnsRibFloat4 type:#float
		sRibFloat5 ui:spnsRibFloat5 type:#float
		sRibFloat6 ui:spnsRibFloat6 type:#float
		sRibFloat7 ui:spnsRibFloat7 type:#float
		sRibFloat8 ui:spnsRibFloat8 type:#float
		sRibFloat9 ui:spnsRibFloat9 type:#float
		sRibFloat10 ui:spnsRibFloat10 type:#float
		
		sRibAnim1 ui:spnsRibAnim1 type:#float
		sRibAnim2 ui:spnsRibAnim2 type:#float
		sRibBool1 ui:spnsRibBool1 type:#integer
		sRibAnim3 ui:spnsRibAnim3 type:#float
		sRibAnim4 ui:spnsRibAnim4 type:#float
		sRibBool2 ui:spnsRibBool2 type:#integer
		sRibAnim5 ui:spnsRibAnim5 type:#float
		sRibAnim6 ui:spnsRibAnim6 type:#float
		sRibBool3 ui:spnsRibBool3 type:#integer
		sRibAnim7 ui:spnsRibAnim8 type:#float
		sRibAnim8 ui:spnsRibAnim8 type:#float
		sRibAnim9 ui:spnsRibAnim9 type:#float
		sRibAnim10	ui:spnsRibAnim10 type:#float
		
		sRibInteger1 ui:spnsRibInteger1 type:#integer
		sRibInteger2 ui:spnsRibInteger2 type:#integer
		sRibInteger3 ui:spnsRibInteger3 type:#integer
		sRibInteger4 ui:spnsRibInteger4 type:#integer
		
	)*/
	
	parameters variations rollout:uiVariations
	(
		directionVariationBool type:#boolean ui:chkDirectionVariationBool default:false animatable:false
		directionVariationAmount type:#float ui:spnDirectionVariationAmount default:0.0 animatable:true
		directionVariationFrequency type:#float ui:spnDirectionVariationFrequency default:0.0 animatable:true
		
		amplitudeVariationBool type:#boolean ui:chkAmplitudeVariationBool default:false animatable:false
		amplitudeVariationAmount type:#float ui:spnAmplitudeVariationAmount default:0.0 animatable:true
		amplitudeVariationFrequency type:#float ui:spnAmplitudeVariationFrequency default:0.0 animatable:true
		
		lengthVariationBool type:#boolean ui:chkLengthVariationBool default:false animatable:false
		lengthVariationAmount type:#float ui:spnLengthVariationAmount default:0.0 animatable:true
		lengthVariationFrequency type:#float ui:spnLengthVariationFrequency default:0.0 animatable:true
		
		radiusVariationBool type:#boolean ui:chkRadiusVariationBool default:false animatable:false
		radiusVariationAmount type:#float ui:spnRadiusVariationAmount default:0.0 animatable:true
		radiusVariationFrequency type:#float ui:spnRadiusVariationFrequency default:0.0 animatable:true
		
		/*on DirectionVariationBool set val do
		(
			if (val==true) then
			(
				spnDirectionVariationAmount.enabled = true
				spnDirectionVariationFrequency.enabled = true
			)
			else
			(
				spnDirectionVariationAmount.enabled = false
				spnDirectionVariationFrequency.enabled = false
			)
		)*/
	)
	
	parameters flags rollout:uiFlags
	(
		rF1 type:#boolean ui:chkRF1 default:false animatable:false
		rF2 type:#boolean ui:chkRF2 default:false animatable:false
		rF3 type:#boolean ui:chkRF3 default:false animatable:false
		rF4 type:#boolean ui:chkRF4 default:false animatable:false
		rF5 type:#boolean ui:chkRF5 default:false animatable:false
		rF6 type:#boolean ui:chkRF6 default:false animatable:false
		rF7 type:#boolean ui:chkRF7 default:false animatable:false
		rF8 type:#boolean ui:chkRF8 default:false animatable:false
		rF9 type:#boolean ui:chkRF9 default:false animatable:false
		rF10 type:#boolean ui:chkRF10 default:false animatable:false
		rF11 type:#boolean ui:chkRF11 default:false animatable:false
		rF12 type:#boolean ui:chkRF12 default:false animatable:false
		rF14 type:#boolean ui:chkRF14 default:false animatable:false
	)
	
	rollout uiDisplayOptions "Display Options"
	(
		spinner spnAmplitudeViz "Amplitude: " range:[0.0,500.0,0.02] scale:0.001 tooltip:"Wave amplitude.\nJust for visualization purposes."
	)
	
	rollout mainui "Main settings"
	(
		label lbl1 "Type: " align:#left across:2 width:27
		dropdownlist ddlRibbonType "" items:ribbonTypeList width:110 offset:[-35,-2]
		label lblParent "Parent: "  across:2 align:#left width:40 offset:[0,8]
		pickbutton btnPickBone "Pick bone" align:#left width:80 offset:[-10,5] filter:boneFilter

		on btnPickBone picked obj do
		( 
			if (obj!=undefined) then
			(
				btnPickBone.text = obj.name
				$.parent= obj
				$.pos = obj.pos
				alignToParent $
			)
		)
		
		group "Ribbon rendering"
		(
			label lbl2 "Material: " across:2 align:#left width:40 offset:[0,8]
			materialButton mtlRibbonMaterial "Pick Material" align:#left width:80 offset:[-10,5]
			label lbl3 "Color #1: " across:2 align:#left width:50
			colorPicker pckBGRAref1 "" align:#left color:(BGRAref1 as color) height:18 fieldWidth:49 alpha:true enabled:true animatable:true
			label lbl4 "Color #2: " across:2 align:#left width:50
			colorPicker pckBGRAref2 "" align:#left color:(BGRAref2 as color) height:18 fieldWidth:49 alpha:true enabled:true animatable:true
			label lbl5 "Color #3: " across:2 align:#left width:50
			colorPicker pckBGRAref3 "" align:#left color:(BGRAref3 as color) height:18 fieldWidth:49 alpha:true enabled:true animatable:true
		)
		
		spinner spnWaveLength "Wave Length: " range:[0.0,500.0,1.5] scale:0.01 offset:[0,10]
		spinner spnTipOffsetZ "Tip Offset"
		
		
		/*
		spinner spnAnim1 "Anim1"
		spinner spnAnim2 "Anim2"
		spinner spnAnim3 "Anim3"
		spinner spnAnim4 "Anim4"
		spinner spnAnim5 "Anim5"
		spinner spnAnim6 "Anim6"
		*/
		
		group "Radius"
		(
			spinner spnStartRadiusScale "Start: " scale:0.01
			spinner spnCenterRadiusScale "Center: " scale:0.01
			spinner spnEndRadiusScale "End: " scale:0.01
			spinner spnEndRadius  "Center Point: " range:[0.0,1.0,0.5] scale:0.01 offset:[0,5]
			spinner spnRibbonTwist "Twist: " scale:0.01 offset:[0,6]
		)
		
		group "Shape"
		(
			spinner spnRibbonDivisions "Divisions: "
			spinner spnRibbonSides "Sides: " type:#integer range:[3,100,5]
			spinner spnRibbonLength "Length: " scale:0.01
			
		)
		
		group "Noise"
		(
			spinner spnSurfaceNoiseAmplitude "Amplitude: "
			spinner spnSurfaceNoiseNumberOfWaves "# of Waves: "
			spinner spnSurfaceNoiseFrequency "Frequency: "
			spinner spnSurfaceNoiseScale "Scale: "
		)
		
		on mainui open do 
		(
			for o in selection where o.baseObject == this do
				if (o.parent != undefined) do btnPickBone.text = o.parent.name
		)
		
	)
	
	/*rollout sribui "Sub-Ribbon"
	(
		checkbox chkEnblSRib "Enable Sub-Ribbon" default:false
		label lblParent "Anchor: "  across:2 align:#left width:40 offset:[0,8]
		pickbutton btnPickSBone "Pick bone" align:#left width:80 offset:[-10,5] filter:boneFilter
		
		spinner spnsRibInteger1 "sRibInteger1" type:#integer
		spinner spnsRibFloat1 "sRibFloat1" type:#float
		spinner spnsRibFloat2 "sRibFloat2" type:#float
		spinner spnsRibFloat3 "sRibFloat3" type:#float
		spinner spnsRibFloat4 "sRibFloat4" type:#float
		spinner spnsRibFloat5 "sRibFloat5" type:#float
		spinner spnsRibInteger2 "sRibInteger2" type:#integer
		spinner spnsRibInteger3 "sRibInteger3" type:#integer
		spinner spnsRibFloat6 "sRibFloat6" type:#float
		spinner spnsRibInteger4 "sRibInteger4" type:#integer
		spinner spnsRibFloat7 "sRibFloat7" type:#float
		spinner spnsRibFloat8 "sRibFloat8" type:#float
		
		spinner spnsRibAnim1 "sRibAnim1" type:#float
		spinner spnsRibAnim2 "sRibAnim2" type:#float
		
		spinner spnsRibBool1 "sRibBool1" type:#integer
		spinner spnsRibAnim3 "sRibAnim3" type:#float
		spinner spnsRibAnim4 "sRibAnim4" type:#float
		
		spinner spnsRibBool2 "sRibBool2" type:#integer
		spinner spnsRibAnim5 "sRibAnim5" type:#float
		spinner spnsRibAnim6 "sRibAnim6" type:#float
		
		spinner spnsRibBool3 "sRibBool3" type:#integer
		spinner spnsRibAnim7 "sRibAnim7" type:#float
		spinner spnsRibAnim8 "sRibAnim8" type:#float
		spinner spnsRibAnim9 "sRibAnim9" type:#float
		spinner spnsRibAnim10	 "sRibAnim10" type:#float
		
		spinner spnsRibFloat9 "sRibFloat9" type:#float
		spinner spnsRibFloat10 "sRibFloat10" type:#float
		
		on btnPickSBone picked obj do
		( 
			if (obj!=undefined) then
			(
				sRibBone = obj
				btnPickSBone.text = obj.name
			)
		)
		
		on sribui open do 
		(
			for o in selection where o.baseObject == this do
				if (sRibBone != undefined) do btnPickSBone.text = sRibBone.name;
		)
	)*/
	
	rollout uiVariations "Variations"
	(
		group "Direction Variation"
		(
			checkbox chkDirectionVariationBool "On"
			spinner spnDirectionVariationAmount "Amount" enabled:false
			spinner spnDirectionVariationFrequency "Frequency" enabled:false
		)
		
		group "Amplitude Variation"
		(
			checkbox chkAmplitudeVariationBool "On"
			spinner spnAmplitudeVariationAmount "Amount" enabled:false
			spinner spnAmplitudeVariationFrequency "Frequency" enabled:false
		)
		
		group "Length Variation"
		(
			checkbox chkLengthVariationBool "On"
			spinner spnLengthVariationAmount "Amount" enabled:false
			spinner spnLengthVariationFrequency "Frequency" enabled:false
		)
			
		group "Radius Variation"
		(
			checkbox chkRadiusVariationBool "On"
			spinner spnRadiusVariationAmount "Amount" enabled:false
			spinner spnRadiusVariationFrequency "Frequency" enabled:false
		)
		
		on chkDirectionVariationBool changed val do
		(
			if (val == true) then
			(
				spnDirectionVariationAmount.enabled = true
				spnDirectionVariationFrequency.enabled = true
			)
			
			else
			(
				spnDirectionVariationAmount.enabled = false
				spnDirectionVariationFrequency.enabled = false
			)
		)
		
		on chkAmplitudeVariationBool changed val do
		(
			if (val == true) then
			(
				spnAmplitudeVariationAmount.enabled = true
				spnAmplitudeVariationFrequency.enabled = true
			)
			
			else
			(
				spnAmplitudeVariationAmount.enabled = false
				spnAmplitudeVariationFrequency.enabled = false
			)
		)
		
		on chkLengthVariationBool changed val do
		(
			if (val == true) then
			(
				spnLengthVariationAmount.enabled = true
				spnLengthVariationFrequency.enabled = true
			)
			
			else
			(
				spnLengthVariationAmount.enabled = false
				spnLengthVariationFrequency.enabled = false
			)
		)
		
		on chkRadiusVariationBool changed val do
		(
			if (val == true) then
			(
				spnRadiusVariationAmount.enabled = true
				spnRadiusVariationFrequency.enabled = true
			)
			
			else
			(
				spnRadiusVariationAmount.enabled = false
				spnRadiusVariationFrequency.enabled = false
			)
		)
	)
	
	rollout uiFlags "Flags"
	(
		checkbox chkRF1 "Collide with terrain"
		checkbox chkRF2 "Collide with objects"
		checkbox chkRF3 "Edge falloff"
		checkbox chkRF4 "Inherit parent velocity"
		checkbox chkRF5 "Smooth size"
		checkbox chkRF6 "Bezier smooth size"
		checkbox chkRF7 "Use vertex alpha"
		checkbox chkRF8 "Scale time by parent"
		checkbox chkRF9 "Force legacy"
		checkbox chkRF10 "Use local time"
		checkbox chkRF11 "Simulate on initialization"
		checkbox chkRF12 "Use length and time"
		checkbox chkRF14 "Align cross-section"
	)
	
	fn createMesh =
	(
		case ribbonType of
		(
			1:
			(
				temp_node = plane length:ribbonLength width:startRadiusScale widthSegs:ribbonSides lengthSegs:ribbonDivisions mapcoords:on
				defObj = SpaceWave wavelength:waveLength amplitude1:amplitudeViz  amplitude2:amplitudeViz
				defObj.rotation = (quat 0.707107 0 0 0.707107)
				temp_node.rotation = (quat 0.707107 0 0 0.707107)
				temp_node.position.z = ribbonLength/2
				bindSpaceWarp temp_node defObj
				meshObj = snapshotAsMesh temp_node
				delete temp_node
				delete defObj
			)
			2:
			(
				temp_node = plane length:ribbonLength width:startRadiusScale widthSegs:ribbonSides lengthSegs:ribbonDivisions mapcoords:on
				defObj = SpaceWave wavelength:waveLength amplitude1:amplitudeViz  amplitude2:amplitudeViz
				defObj.rotation = (quat 0.707107 0 0 0.707107)
				temp_node.rotation = (quat 0.707107 0 0 0.707107)
				temp_node.position.z = ribbonLength/2
				bindSpaceWarp temp_node defObj
				meshObj = snapshotAsMesh temp_node
				delete temp_node
				delete defObj
			)
			3:
			(
				temp_node = cone height:ribbonLength sides:ribbonSides heightsegs:ribbonDivisions radius1:(startRadiusScale/2) radius2:(endRadiusScale/2) mapcoords:on
				defObj = SpaceWave wavelength:waveLength amplitude1:amplitudeViz  amplitude2:amplitudeViz
				defObj.rotation = (quat 0.707107 0 0 0.707107)
				bindSpaceWarp temp_node defObj
				meshObj = snapshotAsMesh temp_node
				delete temp_node
				delete defObj
			)
			4:
			(
				temp_node = cone height:ribbonLength sides:ribbonSides heightsegs:ribbonDivisions radius1:(startRadiusScale/2) radius2:(endRadiusScale/2) mapcoords:on
				defObj = SpaceWave wavelength:waveLength amplitude1:amplitudeViz  amplitude2:amplitudeViz
				defObj.rotation = (quat 0.707107 0 0 0.707107)
				bindSpaceWarp temp_node defObj
				meshObj = snapshotAsMesh temp_node
				delete temp_node
				delete defObj
			)
		)
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			createMesh();
		)
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2ribbon_color
			)
			else #stop
		)
	) 
)

global sc2tight_color = (color 114 116 219)

plugin Helper sc2tight
name: "Tight Test"
classID:#(0x3fdbcbb1, 0x16775dd3)
category:"Starcraft 2 Objects"
extends:dummy
replaceUI:true

(
	
	local tightTypeList = #("Cube","Sphere","Capsule")
	local meshObj
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	parameters main rollout:mainUI
	(
		tightType type:#integer ui:ddlTightType default:2
		tightSize0 type:#float ui:spnTightSize0 default:0.5
		tightSize1 type:#float ui:spnTightSize1 default:0.5
		tightSize2 type:#float ui:spnTightSize2 default:0.5
		
		on tightType set val do
		(
			this.createMesh()
		)
		
		on tightSize0 set val do
		(
			this.createMesh()
		)
		
		on tightSize1 set val do
		(
			if(tightType!=2) then
			(
				this.createMesh()
			)
		)
		
		on tightSize2 set val do
		(
			if (tightType==1) then
			(
				this.createMesh()
			)
		)
	)
	
	rollout mainUI "Main"
	(
		fn boneFilter obj =
		(
			classof obj == Bonegeometry
		)
		
		label lbl1 "Type: " align:#left across:2 width:27
		dropdownlist ddlTightType "" items:tightTypeList width:110 offset:[-35,-2]
		label lblParent "Parent: "  across:2 align:#left width:40 offset:[0,8]
		pickbutton btnPickTightBone "Pick bone" align:#left width:80 offset:[-10,5] filter:boneFilter
		
		on btnPickTightBone picked obj do
		( 
			if (obj!=undefined) then
			(
				btnPickTightBone.text = obj.name
				$.parent= obj
			)
		)
		
		group "Size"
		(
			spinner spnTightSize0 " Size 01: " range:[0.005,1000,0.05] scale:0.01
			spinner spnTightSize1 " Size 02: " range:[0.005,1000,0.05] scale:0.01
			spinner spnTightSize2 " Size 03: " range:[0.005,1000,0.05] scale:0.01
		)
		
		fn naming val =
		(
			case val of
			(
				1:
				(
					spnTightSize0.enabled = true
					spnTightSize0.text = "Width: "
					spnTightSize1.enabled = true
					spnTightSize1.text = "Length: "
					spnTightSize2.enabled = true
					spnTightSize2.text = "Height: "
				)
				2:
				(
					spnTightSize0.enabled = true
					spnTightSize0.text = "Radius: "
					spnTightSize1.enabled = false
					spnTightSize2.enabled = false
				)
				3:
				(
					spnTightSize0.enabled = true
					spnTightSize0.text = "Radius: "
					spnTightSize1.enabled = true
					spnTightSize1.text = "Height: "
					spnTightSize2.enabled = false
				)
			)
		)
		
		on ddlTightType selected val do
		(
			naming val
		)
		
		on mainUI open do
		(
			val = TightType
			naming val
			
			for o in selection where o.baseObject == this do
				if (o.parent != undefined) do btnPickTightBone.text = o.parent.name;
		)
		
		
		group "Reminder"
		(
			label lblWanring "You can have only one tight hit test in the scene.\nIf you place more, only the first one will be exported." align:#left width:140 height:55
		)
	)
	
	fn createMesh =
	(
		case tightType of
		(
			1:
			(
				temp_node = Box length:tightSize0 width:tightSize1 height:tightSize2 mapcoords:on
				temp_node.position.z = (tightSize2/-2)
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
			2:
			(
				temp_node = Sphere radius:tightSize0 mapcoords:on
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
			3:
			(
				temp_node = Capsule height:tightSize1 radius:tightSize0 mapcoords:on
				temp_node.position.z = (tightSize1 + (tightSize0*2))/-2
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
		)
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			createMesh();
		)
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2tight_color
			)
			else #stop
		)
	) 
)

global sc2fuzzy_color = (color 99 193 105)

plugin Helper sc2fuzzy
name: "Fuzzy Test"
classID:#(0x5d5b060b, 0x48d8562b)
category:"Starcraft 2 Objects"
extends:dummy
replaceUI:true

(
	local fuzzyTypeList = #("Cube","Sphere","Capsule")
	local meshObj
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	parameters main rollout:mainUI
	(
		fuzzyType type:#integer ui:ddlFuzzyType default:2
		fuzzySize0 type:#float ui:spnFuzzySize0 default:0.5
		fuzzySize1 type:#float ui:spnFuzzySize1 default:0.5
		fuzzySize2 type:#float ui:spnFuzzySize2 default:0.5
		
		on fuzzyType set val do
		(
			this.createMesh()
		)
		
		on fuzzySize0 set val do
		(
			this.createMesh()
		)
		
		on fuzzySize1 set val do
		(
			if(fuzzyType!=2) then
			(
				this.createMesh()
			)
		)
		
		on fuzzySize2 set val do
		(
			if (fuzzyType==1) then
			(
				this.createMesh()
			)
		)
	)
	
	rollout mainUI "Main"
	(
		fn boneFilter obj =
		(
			classof obj == Bonegeometry
		)
		
		label lbl1 "Type: " align:#left across:2 width:27
		dropdownlist ddlFuzzyType "" items:fuzzyTypeList width:110 offset:[-35,-2]
		label lblParent "Parent: "  across:2 align:#left width:40 offset:[0,8]
		pickbutton btnPickFuzzyBone "Pick bone" align:#left width:80 offset:[-10,5] filter:boneFilter
		
		on btnPickFuzzyBone picked obj do
		( 
			if (obj!=undefined) then
			(
				btnPickFuzzyBone.text = obj.name
				$.parent= obj
			)
		)
		
		
		group "Size"
		(
			spinner spnFuzzySize0 " Size 01: " range:[0.005,1000,0.05] scale:0.01
			spinner spnFuzzySize1 " Size 02: " range:[0.005,1000,0.05] scale:0.01
			spinner spnFuzzySize2 " Size 03: " range:[0.005,1000,0.05] scale:0.01
		)
		
		fn naming val =
		(
			case val of
			(
				1:
				(
					spnFuzzySize0.enabled = true
					spnFuzzySize0.text = "Width: "
					spnFuzzySize1.enabled = true
					spnFuzzySize1.text = "Length: "
					spnFuzzySize2.enabled = true
					spnFuzzySize2.text = "Height: "
				)
				2:
				(
					spnFuzzySize0.enabled = true
					spnFuzzySize0.text = "Radius: "
					spnFuzzySize1.enabled = false
					spnFuzzySize2.enabled = false
				)
				3:
				(
					spnFuzzySize0.enabled = true
					spnFuzzySize0.text = "Radius: "
					spnFuzzySize1.enabled = true
					spnFuzzySize1.text = "Height: "
					spnFuzzySize2.enabled = false
				)
			)
		)
		
		on ddlFuzzyType selected val do
		(
			naming val
		)
		
		on mainUI open do
		(
			val = FuzzyType
			naming val
			
			for o in selection where o.baseObject == this do
				if (o.parent != undefined) do btnPickFuzzyBone.text = o.parent.name;
		)
	)
	
	fn createMesh =
	(
		case fuzzyType of
		(
			1:
			(
				temp_node = Box length:fuzzySize0 width:fuzzySize1 height:fuzzySize2 mapcoords:on
				temp_node.position.z = (fuzzySize2/-2)
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
			2:
			(
				temp_node = Sphere radius:fuzzySize0 mapcoords:on
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
			3:
			(
				temp_node = Capsule height:fuzzySize1 radius:fuzzySize0 mapcoords:on
				temp_node.position.z = (fuzzySize1 + (fuzzySize0*2))/-2
				meshObj = snapshotAsMesh temp_node;
				delete temp_node;
			)
		)
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			createMesh();
		)
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then 
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2fuzzy_color
			)
			else #stop
		)
	) 
	
	/*on update do
	(
	)*/
	
)

-- global sc2bndsph_rad = 0.1
global sc2bndsph_colour = (color 6 134 58) -- dark green

plugin Helper sc2boundsphere
name:"Bound Sphere"
classID:#(0x5bd5a942, 0x5b0e266)
category:"Starcraft 2 Objects"
extends:Dummy
version:1
(
	local meshObj, lastRad, lastScale, meshNode
	
	fn getOwners =
	(
		for o in refs.dependents this where isValidNode o collect o
	)
	
	parameters main rollout:mainui
	(
		bndRadius type:#float ui:spnRadius default:0.0 animatable:true
		
		on bndRadius set val do
		(
			-- sc2bndsph_rad = val
		)
	)
	
	rollout mainui "Sphere Parameters"
	(
		spinner spnRadius "Radius:" range:[0, 1e9, 0.0] type:#float scale:0.01
	)
	
	on getDisplayMesh do
	(
		if (meshObj == undefined) then
		(
			meshObj = createInstance sphere radius:bndRadius mapCoords:false
			lastRad = bndRadius
		)
		
		if (radius != lastRad) then
		(
			meshObj.radius = bndRadius
			lastRad = bndRadius
		)
		
		meshObj.mesh
	)
	
	tool create 
	( 
		on mousePoint click do 
		(
			if click == 1 then
			(
				nodeTM.translation = gridPoint
				meshNode = getOwners()
				meshNode.wirecolor = sc2bndsph_colour
			) else #stop
		)
		on mouseMove click do
		(
			local lrgDist = 0.0

			for i = 1 to 3 do
			(
				if (lrgDist < GridDist[i]) then lrgDist = GridDist[i]
			)

			bndRadius = lrgDist
		)
	) 
)


plugin Light sc2light
name:"SCII Light"
classID:#(0x7183b458, 0x1dffbb73)
category:"Standard"
extends:freeSpot
replaceUI:true

(
	local lightTypeList = #("Point","Spot")
	
	parameters genPar rollout:genParUI
	(
		lightFlag4 type:#boolean ui:chkLightFlag4 default:true
		lightType type:#integer ui:ddllightType default:2
		
		on lightType set val do
		(
			case val of
			(
				1: delegate.type = #omni
				2: delegate.type = #freespot
			)
		)
		
		lightTarDis type:#float ui:spnLightTarDis default:3.93
		
		on lightTarDis set val do
		(
			if (lightType != 1) then
			(
				delegate.target_distance = val
			)
		)
		
		lightFlag1 type:#boolean ui:chkLightFlag1 default:false
		lightFlag2 type:#boolean ui:chkSpecular default:false
		lightFlag3 type:#boolean default:false ui:chkAmbient 
		
		ulPr4 type:#float default:1 animatable:true

	)
	
	parameters lightMain rollout:lightMainUI
	(
		lightColor type:#color default:(color 255 255 255) animatable:true ui:pcklightColor 
		lightIntensity type:#float default:1 animatable:true ui:spnlightIntensity
		specColor type:#color default:(color 255 250 1) animatable:true ui:pckspecColor 
		specIntensity type:#float default:1 animatable:true ui:spnspecIntensity
		
		on lightIntensity set val do
		(
			delegate.multiplier = val
		)
		
		ulPr3 type:#float default:1 ui:spnunklightProp3 animatable:true
		
		on ulPr3 set val do
		(
			delegate.decayRadius = val/2.54
		)
		
	)
	
	parameters spotPar rollout:spotParUI
	(
		lightSpot type:#float default:45 ui:spnlightSpot animatable:true 
		lightRange type:#float default:43 ui:spnlightRange animatable:true
		
		on lightSpot set val do
		(
			if (lightType != 1) then
			(
				delegate.falloff = val
			)
		)
		
		on lightRange set val do
		(
			if (lightType != 1) then
			(
				delegate.hotspot = val
			)
		)
	)
	
	rollout genParUI "General Parameters"
	(
		group "Light Type"
		(
			checkbox chkLightFlag4 "On" across:2
			dropdownlist ddllightType "" items:lightTypeList width:85 align:#left offset:[-18,0]
			spinner spnLightTarDis "Target Distance:   " width:110 align:#right
		)
		
		checkbox chkLightFlag1 "Shadows"
		checkbox chkSpecular "Specular"
		checkbox chkAmbient "Ambient Occlusion"
	)
	
	rollout lightMainUI "Intensity/Color"
	(
		spinner spnlightIntensity "Light: " across:2 width:80 align:#right offset:[20,0]
		colorpicker pcklightColor "" color:lightColor align:#right offset:[0,-2]
		spinner spnspecIntensity "Specular: " across:2 width:100 align:#right offset:[20,0]
		colorpicker pckspecColor "" color:specColor align:#right offset:[0,-2]
		
		group "Decay"
		(
			spinner spnunklightProp3 "Start:" align:#left width:90 range:[0,1e9,0] across:2
		)

	)
	
	rollout spotParUI "Spot Parameters"
	(
		group "Light Cone"
		(
			spinner spnlightRange "HotSpot/Beam:" align:#right range:[0.5,179.5,45.0] width:100
			spinner spnlightSpot "Falloff/Field:" align:#right range:[0.5,179.5,45.0] width:110
		)
	)
	
	on create do
	(
		delegate.attenDecay = 3
		delegate.target_distance = 3.93
	)
)

-- ********************
--  M3 UI ELEMENTS
-- ********************
-- Animation UI dependents
struct sc2anim
(
	-- Default values incase not found
	name = "Stand", seqInd = 0, 
	cstart = 0, cend = 1000, 
	freq = 100, moveSpeed = 0, looping = false, GLseqH = false, GLseqP = false
)

fn sc2_AnimToStrm anim =
(
	local ss = stringStream ""
	format ".name:%\n" anim.name to:ss
	format ".seqInd:%\n" anim.seqInd to:ss
	format ".cstart:%\n" anim.cstart to:ss
	format ".cend:%\n" anim.cend to:ss
	format ".freq:%\n" anim.freq to:ss
	format ".moveSpeed:%\n" anim.moveSpeed to:ss
	format ".looping:%\n" anim.looping to:ss
	return ss
)

fn sc2_Set_Scene_Data obj animdata =
(
	local aheader = stringstream ""
	format ".version:%\n" SC2PLUGIN_VERS to:aheader
	format ".count:%\n" animdata.count to:aheader
	setAppData obj 1 aheader
	for i = 1 to animdata.count do
	(
		local aind = i + 1
		local ad = animdata[i]
		
		local adss = sc2_AnimToStrm ad
		setAppData obj aind adss
	)
)

fn sc2_Get_ssData stringStrm property type:#value =
(
	-- reposition at start of stream
	seek stringStrm 0
	if (skipToString stringStrm property != undefined) then
	(
		local val
		case type of
		(
			#string: val = readLine stringStrm
			#value: val = readValue stringStrm
			#boolean: val = readValue stringStrm as booleanClass
		)

		return val
	)
)

fn sc2_Get_AnimData animss sc2Vers =
(
	local anim = sc2anim()
	
	anim.name 		= sc2_Get_ssData animss ".name:" type:#string
	anim.seqInd	= sc2_Get_ssData animss ".seqInd:"
	anim.cstart 	= sc2_Get_ssData animss ".cstart:"
	anim.cend 		= sc2_Get_ssData animss ".cend:"
	if (sc2vers != undefined) then
	(
		case of
		(
			default:
			(
				anim.freq		= sc2_Get_ssData animss ".freq:"
				anim.moveSpeed = sc2_Get_ssData animss ".moveSpeed:"
				anim.looping	= sc2_Get_ssData animss ".looping:" type:#boolean
				anim.GLseqH	= sc2_Get_ssData animss ".GLseqH:" type:#boolean
				anim.GLseqP	= sc2_Get_ssData animss ".GLseqP:" type:#boolean
			)
		)
	)
	else
	(
		anim.freq			= 100
		anim.moveSpeed 	= 0.0
		anim.looping		= false
		anim.GLseqH		= false
		anim.GLseqP		= false
	)
	
	return anim
)

-- Animation UI
utility sc2animUI "M3 - Sequences"
(
	local animList = #("None")
	local aobj = $_M3_Anim_Data
	local aObjClass = classOf aobj as string
	local adata = #()
	
	Group "Select Animation"
	(
		dropdownlist	dlAnim items:#("None")
	)
	
	Group "Edit"
	(
		edittext		edtAnimName "Animation Name:" labelOnTop:true enabled:false
		spinner			spnSeqInd "Sequence Index:" range:[0,1e9,0] type:#integer width:125 align:#right enabled:false
		label				lblAnimRange "Animation Ranges" align:#left offset:[0,10]
		spinner			spnStart "Time Start:" range:[0,1e9,0] type:#integer enabled:false
		spinner			spnEnd "Time End:" range:[0,1e9,0] type:#integer enabled:false
		spinner			spnFreq "Frequency (%):" range:[0,100,100] type:#integer enabled:false fieldwidth:50
		spinner			spnMoveSpeed "Move Speed:" range:[0,1e6,0] type:#float enabled:false scale:1
		checkbox		chkLooping "Looping" align:#left enabled:false
		checkbox		chkGLseqH "GLseq (hard)" align:#left enabled:false
		checkbox		chkGLseqP "GLseq (prev)" align:#left enabled:false
		label				lblSpacer1 height:5
		button btnAdd "Add" align:#left across:2 width:60
		button btnRemove "Remove" align:#right enabled:false
	)
	
	fn NoneSelected =
	(
		if (adata.count > 0) then 
		(
			local lastFrame = 0
			for i = 1 to adata.count do
			(
				local ad = adata[i]
				if ad.cend > lastFrame then lastFrame = ad.cend
			)
			animationRange = interval 0 lastFrame
			spnEnd.value = lastFrame
		)
		else 
		(
			animationRange = interval 0 1000
			spnEnd.value = 1000
		)
		spnStart.value = 0
		
		-- Reset and disable UI elements
		edtAnimName.text = ""
		edtAnimName.enabled = false
		spnSeqInd.value = 0
		spnSeqInd.enabled = false
		spnStart.enabled = false
		spnEnd.enabled = false
		spnFreq.enabled = false
		spnMoveSpeed.enabled = false
		chkLooping.checked = false
		chkLooping.enabled = false
		chkGLseqH.checked = false
		chkGLseqH.enabled = false
		chkGLseqP.checked = false
		chkGLseqP.enabled = false
		btnRemove.enabled = false
	)
	
	fn Selected sel =
	(
		dlAnim.selection = sel

		if sel == 1 then 
		(
			NoneSelected()
		)
		else
		(
			local ad = adata[sel-1]
			animationRange = interval ad.cstart ad.cend
			sliderTime = ad.cstart
			edtAnimName.text = ad.name
			spnSeqInd.value = ad.seqInd
			spnStart.value = ad.cstart
			spnEnd.value = ad.cend
			spnFreq.value = ad.freq
			spnMoveSpeed.value = ad.moveSpeed
			chkLooping.checked = ad.looping
			--chkGLseqH.checked = ad.GLseqH
			--chkGLseqP.checked = ad.GLseqP
			
			-- Enable UI elements
			edtAnimName.enabled = true
			spnSeqInd.enabled = true
			spnStart.enabled = true
			spnEnd.enabled = true
			spnFreq.enabled = true
			spnMoveSpeed.enabled = true			
			chkLooping.enabled = true
			chkGLseqH.enabled = false
			chkGLseqP.enabled = false
			btnRemove.enabled = true
		)
	)
	
	fn List_Repop  =
	(
		animList = dlAnim.items
		animList = #("None")
		for i = 1 to adata.count do
		(
			ad = adata[i]
			--echo (anim_data[i].name)
			local aname = ad.name + ("["+ad.seqInd as string+"]")
			append animList aname
		)
		
		dlAnim.items = animList
	)
	
	fn List_Pop =
	(
		if (aobj != undefined and isValidNode aobj == true and aObjClass == "Point") then
		(	
			--reset vars
			adata = #()
			
			local acss = (getAppData aobj 1) as stringstream
			if (acss != undefined) then
			(
				local sc2Vers = sc2_Get_ssData acss ".version:"
				local animCount = sc2_Get_ssData acss ".count:"
				if (animCount != 0) then
				(
					adata[animCount] = 0
					for h = 1 to animCount do
					(
						local stcInd = h + 1
						local stcss = (getAppData aobj stcInd) as stringstream
						local anim = sc2_Get_AnimData stcss sc2Vers
						
						-- Update with new version data
						local animss = sc2_AnimToStrm anim
						
						setAppData aobj stcInd animss
						
						adata[h] = anim
					)
				)
				
				-- Update animation storage to current version
				-- (AppData) Update Animation Count
				local adss = stringstream ""
				format ".version:%\n" SC2PLUGIN_VERS to:adss
				format ".count:%\n" adata.count to:adss
				setAppData aobj 1 adss
				
				List_Repop()
			)
		)
	)
	
	fn UpdateEntry val entryid =
	(
		-- sanity check
		local aind = dlAnim.selection - 1
		if (aind > 0) then
		(
			-- grab AD
			local ad = adata[aind]
			objInd = aind + 1 -- obj index for replacing user data
			
			-- Set new value
			case entryid of
			(
				#name: ad.name = val
				#seqInd: ad.seqInd = val
				#cstart: ad.cstart = val
				#cend: ad.cend = val
				#freq: ad.freq = val
				#moveSpeed: ad.moveSpeed = val
				#looping: ad.looping = val
				#GLseqH: ad.GLseqH = val
				#GLseqP: ad.GLseqP = val
			)
			
			if (entryid == #cstart or entryid == #cend) then animationRange = interval ad.cstart ad.cend
			
			local adss = sc2_AnimToStrm ad
			-- Update app data
			setAppData aobj objInd adss
			
			-- Update UI data container
			adata[aind] = ad
			List_Repop() -- Refresh UI
		)
	)
	
	on sc2animUI open do
	(
		List_Pop()
		if (adata.count > 0) then
		(
			local standFound = 0
			for i = 1 to adata.count do
			(
				local ad = adata[i]
				local aname = ad.name
				if ((matchPattern aname pattern:"Stand") == true) then
				(
					standFound = i
				)
			)
			
			local dlsel = 1
			if (standFound > 0) then
			(
				dlsel = standFound
			)
			else
			(
				local rsel = random 1 adata.count
				dlsel = rsel
			)
			dlAnim.selection = dlsel + 1 -- skip "none" in array
			Selected dlAnim.selection
		)
	)
	on dlAnim selected i do
	(
		if (adata.count > 0) then
		(
			if (i == 1) then
			(
				NoneSelected()
			)
			else
			(
				Selected i
			)
		)
	)
	-- UI value updates
	on edtAnimName entered txt do (UpdateEntry txt #name)
	on spnSeqInd changed val do (UpdateEntry val #seqInd)
	on spnStart changed val do 
	(
		local nval
		if (val != spnEnd.value) then
		(
			nval = val
		)
		else
		(
			nval = val - 1
		)		
		UpdateEntry nval #cstart
		spnStart.value = nval
	)
	on spnEnd changed val do 
	(
		local nval
		if (val != spnStart.value) then
		(
			nval = val
		)
		else
		(
			nval = val + 1
		)
		UpdateEntry nval #cend
		spnEnd.value = nval
	)
	on spnFreq changed val do (UpdateEntry val #freq)
	on spnMoveSpeed changed val do (UpdateEntry val #moveSpeed)
	on chkLooping changed val do (UpdateEntry val #looping)
	on chkGLseqH changed val do (UpdateEntry val #GLseqH)
	on chkGLseqP changed val do (UpdateEntry val #GLseqP)
	on btnAdd pressed do
	(
		-- Create new animation data placeholder
		ad = sc2anim()
		if (adata.count > 0) then
		(
			ad.seqInd	= (adata[adata.count].seqInd) + 1
			ad.cstart 	= (adata[adata.count].cend as integer) + 1000
			ad.cend		= (ad.cstart as integer) + 1000
			ad.freq		= 100
			ad.moveSpeed = 0
			ad.looping = false
			ad.GLseqH = false
			ad.GLseqP = false
		)
		
		-- Append to UI animation data collection
		append adata ad
		local aObjClass = classOf aObj as string
		if (isValidNode aobj != true or aObjClass != "Point") then
		(
			-- Initiate Scene Object
			if ($_M3_Anim_Data != undefined) then delete($_M3_Anim_Data)
			-- Create scene object to house animation information
			local adObj = point size:0.001 pos:[0,0,0]
			adObj.name = "_M3_Anim_Data"
			adObj.renderable = off
			adObj.isHidden = true
			adObj.wirecolor = color 8 8 136
			
			aobj = adObj
		)
		-- (AppData) Update Animation Count
		local adss = stringstream ""
		format ".version:%\n" SC2PLUGIN_VERS to:adss
		format ".count:%\n" adata.count to:adss
		setAppData aobj 1 adss
		
		-- (AppData) Add Entry
		local adss = sc2_AnimToStrm ad
		local objInd = adata.count + 1
		setAppData aobj objInd adss
		
		-- update list
		List_Pop()
		Selected (adata.count + 1)
	)
	on btnRemove pressed do
	(
		local ind = dlAnim.selection - 1
		if (ind != 0) then
		(
			adata = deleteItem adata ind 
			sc2_Set_Scene_Data aobj adata -- Fix scene data
			
			-- Reset list and selection
			List_Pop()
			Selected ind
		)
	)
)

fn sc2animUIreset =
(
	-- Reset
	if (sc2animUI != undefined) do closeUtility sc2animUI
)

fn sc2bndSphCheck =
(
	local bndSphs = #()
	
	for i = 1 to objects.count do
	(
		local obj = objects[i]
		objClass = classOf obj.baseObject as string
		
		if (objClass == "sc2boundSphere") then
		(
			append bndSphs obj
		)
	)
	
	if (bndSphs.count > 1) then
	(
		for i = 1 to (bndSphs.count - 1) do
		(
			delete bndSphs[i]
		)
		
		-- check and resolve incremented name
		local bsph = bndSphs[bndSphs.count]
		local bname = bsph.name
		if (bname[bname.count] == "2") then bname[bname.count] = "1"
		bsph.name = bname
	)
)

utility sc2boneFlags "M3 - Bone tools"
(
	local boneArray = #()
	local boneNameArray = #()
	local userProps = #("InheritT","InheritS","InheritR","BillboardBone","2DProjection","Animated","EnableIK","Skinned")
	local userPropsStates = #()
	local curSelection = 1
	
	local SuperObjectClass = undefined
	local SelObjArray = #()
	local SelHelpArray = #()
	
	
	group "Flags"
	(
		button btnListImport "Import/Update bones" width:134 height:20
		button btnSelectBone "Select highligthed bone" width:134 height:20
		listbox boneList items:boneNameArray selection:1 height:10
		
		checkbox chkFlagInheritT "Inherit translation" checked:false enabled:false
		checkbox chkFlagInheritS "Inherit scale" checked:false enabled:false
		checkbox chkFlagInheritR "Inherit rotation" checked:false enabled:false
		checkbox chkBillboardBone "Billboard bone" checked:false enabled:false
		checkbox chk2DProjection "2D Projection" checked:false enabled:false
		checkbox chkAnimated "Animated" checked:false enabled:false
		checkbox chkEnableIK "Enable IK" checked:false enabled:false
		checkbox chkSkinned "Skinned" checked:false enabled:false
	)
	
	fn getBoneFlag workingBone workingProp propIndex =
	(
		case propIndex of
		(
			1: 
			(
				chkFlagInheritT.enabled = true
				chkFlagInheritT.checked = getUserProp workingBone workingProp
			)
			2: 
			(
				chkFlagInheritS.enabled = true
				chkFlagInheritS.checked = getUserProp workingBone workingProp
			)
			3: 
			(
				chkFlagInheritR.enabled = true
				chkFlagInheritR.checked = getUserProp workingBone workingProp
			)
			4: 
			(
				chkBillboardBone.enabled = true
				chkBillboardBone.checked = getUserProp workingBone workingProp
			)
			5: 
			(
				chk2DProjection.enabled = true
				chk2DProjection.checked = getUserProp workingBone workingProp
			)
			6: 
			(
				chkAnimated.enabled = true
				chkAnimated.checked = getUserProp workingBone workingProp
			)
			7: 
			(
				chkEnableIK.enabled = true
				chkEnableIK.checked = getUserProp workingBone workingProp
			)
			8: 
			(
				chkSkinned.enabled = true
				chkSkinned.checked = getUserProp workingBone workingProp
			)
		)
	)
	
	fn CollectBones =
	(
		boneArray = #()
		boneNameArray = #()
		boneArray = for b in geometry where classof b == BoneGeometry collect b
		
		for n in 1 to boneArray.count do 
		(
			boneNameArray[n] = boneArray[n].name as string
			
			for i in 1 to userProps.count do
			(
				local prop = userProps[i]
				
				if ((getUserProp boneArray[n] prop) == undefined) then
				(
					if (i==6 or i==8) then
					(
						setUserProp boneArray[n] prop true
					)
					else
					(
						setUserProp boneArray[n] prop false
					)
				)
				else
				(
					getBoneFlag boneArray[n] prop i
				)
			)
		)
	)
	
	fn startLocation =
	(
		collectBones()
		boneList.items = boneNameArray
		curSelection = boneArray[1]
	)
	
	on btnListImport pressed do
	(	
		startLocation()
	)
	
	on boneList selected val do
	(
		
		local selBone = boneArray[val]
		
		
		if (isDeleted selBone) then
		(
			messageBox ("Selected bone does not exist anymore.\nRefreshing the bone list.\n") title:"Import/Update failed"
			startLocation()
		)
		else
		(
			curSelection = selBone
			
			for i in 1 to userProps.count do
			(
				local prop = userProps[i]
				
				getBoneFlag selBone prop i
			)
		)
	)
	
	on btnSelectBone pressed do
	(
		if (curSelection !=undefined) then
		(
			select curSelection
		)
	)
	
	on chkFlagInheritT changed newState do
	(
		setUserProp curSelection userProps[1] newState
	)
	
	on chkFlagInheritS changed newState do
	(
		setUserProp curSelection userProps[2] newState
	)
	
	on chkFlagInheritR changed newState do
	(
		setUserProp curSelection userProps[3] newState
	)
	
	on chkBillboardBone changed newState do
	(
		setUserProp curSelection userProps[4] newState
	)
	
	on chk2DProjection changed newState do
	(
		setUserProp curSelection userProps[5] newState
	)
	
	on chkAnimated changed newState do
	(
		setUserProp curSelection userProps[6] newState
	)
	
	on chkEnableIK changed newState do
	(
		setUserProp curSelection userProps[7] newState
	)
	
	on chkSkinned changed newState do
	(
		setUserProp curSelection userProps[8] newState
	)
	
	on sc2boneFlags open do
	(
		--addForAll()
	)
)

-- Max Reset causes Animation UI to close
callbacks.removeScripts id:#m3animrs
callbacks.removeScripts id:#m3attach
callbacks.removeScripts id:#m3bndsph
callbacks.addScript #systemPostReset "sc2animUIreset()" id:#m3animrs
callbacks.addScript #systemPostReset "sc2att_size = 0.08; sc2par_size = 0.08; sc2att_id = \"\"; " id:#m3attach
callbacks.addScript #filePostOpen "sc2animUIreset()" id:#m3animrs
callbacks.addScript #filePostOpen "sc2att_size = 0.08; sc2par_size = 0.08; sc2att_id = \"\"" id:#m3attach
callbacks.addScript #nodeCreated "sc2bndSphCheck()" id:#m3bndsph

--meditMaterials[1] = sc2material ()
--meditMaterials[1].diffuseMap = sc2bitmap ()