------------------------------------------------------------------------------------------------
-- Starcraft 2 Model (3ds->M3) Exporter version 0.25
-- by NiNtoxicated (madyavic@gmail.com)
-- by Leruster (leruster@gmail.com)
-- Exports M3 models from 3ds max via maxscript
-- Created for 3ds max 2010, but should be backwards compatible (3ds Max 9 SP2 onward)
-- Based on reworked code from my World of Warcraft M2 importer/exporter
--
-- Currently exports:
-- - Geometry
-- - Materials (Standard, Displace, Volume and Terrain)
-- - Skin/Bones
-- - Animations
-- - Attachments
-- - Cameras
-- - Lights
-- - Ribbons
-- - Particles
-- - Tight and Fuzzy hit tests
--
-- Setting up an exportable model:
-- 1. Create 'editable_mesh' type meshes
-- 2. Apply materials of type Starcraft 2, Standard or Multimaterial to them
-- 3. Export!
--
-- To export Cameras properly you need to create targeted camera. Middle point for DOF functions
-- is deifned by the target position.
-- Environment ranges define DOF falloff start and end.
-- Clip planes for the StarCraft to handle are defined by the Clipping Planes values.
-- 
-- Big thanks to: 
-- Blue Isle Studios (http://blueislestudios.com/) - Big thanks to the support provided by these guys. Check them out!
-- Teal (starcraft.incgamers.com) - PHP M3 parser
-- Volcore (http://volcore.limbicsoft.com/)  - Helped figure out vertex flags
-- Witchsong (http://code.google.com/p/libm3/) - Awesome M3 library and for help on sequence data
-- Sixen (http://www.sc2mapster.com/) - Providing a good central resource for SC2 tools
-- der_Ton (http://www.doom3world.org/) - Great work on MD5 format which is similar to M3
-- MrMoonKr - Providing a toUpper function to fix 3ds max incompatibility issues
-- Skizot - For testing and providing suggestions to improve the script, very big thanks
-- Phygit - Providing bug fixes and development information to do with the M3 format
-- ufoZ - One of the original people to reverse engineer the M2 format and provide a good maxscript importer
-- 			from which my importer/exporters were originally based
-- Jakub Jeziorski
-- Florian Kberle (https://github.com/flo/m3addon/) - for creating python scripts, that can convert m3 files to xml and back based on a xml description of the m3 file format.
--
-- Check out http://code.google.com/p/libm3/ for more indepth detail about the M3 format
-- Head to http://www.sc2mapster.com/ for more mod tools and Starcraft 2 content
--
-- If you use any of this code in your own scripts, please credit where it is due.
--
-- Things to do:
-- 1. Clean up messy code
-- 2. Implement 3dsMax GUI properly
-- 3. Add good animation support
-- 4. Improve code functionality
-- 5. Improve processing times
-- 6. Add more feature support
-- 7. Improve error catching and error support
-- 8. Finish particles, forces and sRibbons
--
------------------------------------------------------------------------------------------------
--
-- Version History:
-- 0.26 - December 23rd 2012
-- Added tight and fuzzy hit tests
-- Added bone flags support
-- Added Bone Tools interface to manage bone flags
-- Added ribbon effect support
-- Changes in Particle Emitters. Some values have new names, places. Some are properly described.
-- Changes in layer system (mainly due to particle materials compatibility)
-- Added layer managment when importing objects
-- Scene check and Add bones buttons are now available from the M3-Export menu
-- Modified lights according to Blizzard's pdf
-- m3 scripts contain foundation for forces, sRibbons and projections
-- Fixed bug when trying to import .m3 file with volume materials
-- Bone count after import is now as it is in original file (necessary for proper ribbon and particles animations import)
-- Particle Copies are temporarily taken out

-- 0.25 - July 3rd 2012
-- Added Volume Material support
-- Added Lights support
-- Improved material layers system
-- Made minor changes in Standard Material
--
-- 0.24 - May 13th 2012
-- Added Displace Material support
--
-- 0.23 - April 1st 2012
-- Added Portrait Camera support
--
-- 0.22 - March 5th 2011
-- Added particle emitter support
-- Added custom bounding sphere support
-- Added attachment volume support
-- Added bitmap animation support
-- Improved vertex normal exporting slightly
-- Global animations no longer exported as they aren't currently supported
-- Many miscellaneous fixes and code improvements
--
-- 0.21b - November 22 2010
-- Fixed a bitmap bug that affected cloaking
--
-- 0.21 - August 15th 2010
-- Animation data storage object no longer exported as bone
--
-- 0.20 - August 5th 2010
-- Normals/Smoothing groups are now exported properly
-- Emissive maps should now work properly
--
-- 0.19b - July 24th 2010
-- Improved bounding sphere calculation to include all meshes in the scene
-- Improved the way process time is output
-- Added sc2_objects.ms error catching
-- Fixed vertex weighting bug
--
-- 0.19 - July 15th 2010
-- Fixed zero weights issue
-- Added custom map support
-- Removed redundant 'Use Internal Texture Path' option
-- Fixed crash when hidden objects were used as bones
-- Added scene UI options for frozen and hidden objects
--
-- 0.18 - June 28th 2010
-- Added exporting multiple meshes as submeshes support
-- Added Starcraft 2 attachment object support
-- Added decal support
-- Added terrain (null) material type support
-- Object export code heavily reworked, all objects now export as bones
-- Export no longer requires all bone objects added to skin modifiers
-- Sequences are now exported based on all animation ranges
--
-- 0.17 - June 14th 2010
-- Fixed rotation sometimes not exporting properly
-- Added some basic message boxes for key events
--
-- 0.16b - June 12th 2010
-- Fixed incorrect skin bone assignment
--
-- 0.16 - June 9th 2010
-- Alpha Mask cutout threshold now works correctly
-- Bone animation is now supported
-- Updated bone chain processing
-- Added custom vertex normals support
-- Many code fixes and updates
--
-- 0.14 - May 28th 2010
-- Added Alpha Mask map support
-- Added bone support
-- Added baseframe and bindpose support
-- Added attachment support
-- Vastly improved mesh processing 
--
-- 0.09 - May 17th 2010
-- Incompatibility error with old 3ds max versions fixed (thanks to MrMoonKr)
-- Improved material handling
-- Improved code stability
--
-- 0.08 - May 12th 2010
-- Added support for sc2 material and bitmap plugin objects
-- Fixed error with matid's being out of range crashing exporter
-- Improved material code
-- Added more error catching
--
-- 0.07 - May 5th 2010
-- Added some additional UI elements
-- Fixed texture path processing
--
-- 0.06 - May 1st 2010
-- Basic UI added
-- Some basic error catching added
--
-- 0.04 - April 30th 2010
-- Adapted to new MD34 format
-- Basic geometry exporting implemented
-- Bounding sphere exporting added
--
-- 0.01 - April 12th 2010
-- Initial version of M3 export script
--
------------------------------------------------------------------------------------------------

-- ********************
--  SCRIPT DECLARES
-- ********************
-- globals
global head, mwrite, awrite, bstream, cbones, m3model, m3scene
global scrStart, scrEnd -- For script timeStamps
global twrite		= #()

-- Switches
--global g_UseInternalPath = true -- use internal path
global g_flipuvy = true -- Flips uv.y coords

-- Misc. variables
global g_animObj = "_M3_Anim_Data"
global g_mapchannel = 1
global g_bindposeFrame = 0
global g_baseFrame = 1
global g_pemittersCount = 0
global g_exportAnimation = true
global g_exportHidden = true
global g_exportFrozen = false

global g_chkExpHelpers = true
global g_chkExpMaterials = true
global g_chkExpMesh = true
global g_chkExpBones = true
global g_chkExpErrors = true

global g_useFPS = 1000
global g_doCustomVertNormals = true -- redundant now
global g_doPEmitters = false

-- Temp. write file
global texPath = "Assets/Textures/"

-- Function Declares
fn M3E_WriteRef bitStream data reftype flags:0 = ()
fn M3E_WriteArray bitStream data flags:#uint32 reftype:#null = ()
fn M3E_WriteTight bitStream d = ()

-- ***********
--  HELPERS
-- ***********
-- Liberally stolen from my M2 script
fn echo msg =
(
	format "%\n" (msg) to:listener
)

-- math function for comparing two quaternions
fn qdot q1 q2 = return ((q1.x*q2.x) + (q1.y*q2.y) + (q1.z*q2.z) + (q1.w*q2.w))

-- courtesy of MrMoonKr
-- Converts lower case string to uppercase without needing max 2008
fn M3E_toUpper str =
(
   if str == undefined do return undefined

   upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
   lower = "abcdefghijklmnopqrstuvwxyz"

   outstr = copy str

   for i = 1 to outstr.count do
   (
      j = findString lower outstr[i]
      if j != undefined do outstr[i] = upper[j]
   )
   return outstr
)

fn M3E_ArrayToPoint pntArray =
(
	local pnt
	local pntArrayCount = pntArray.count
	case pntArrayCount of
	(
		2:
		(
			pnt = [0, 0]
			
			pnt.x = pntArray[1]
			pnt.y = pntArray[2]
		)
		3:
		(
			-- initialise array
			pnt = [0, 0, 0]
			
			pnt.x = pntArray[1]
			pnt.y = pntArray[2]
			pnt.z = pntArray[3]
		)
		4:
		(
			pnt = [0, 0, 0, 0]
			
			pnt.x = pntArray[1]
			pnt.y = pntArray[2]
			pnt.z = pntArray[3]
			pnt.w = pntArray[4]
		)
	)
	
	return pnt
)

fn M3E_ColourToPoint4 col =
(
	local p4 = [0, 0, 0, 0]
	p4.x = col.blue
	p4.y = col.green
	p4.z = col.red
	p4.w = col.alpha
	
	return p4
)

fn M3E_BoolToInt bool =
(
	if (bool == true) then return 1 else return 0
)
	
fn LongToString num=
(
	local str = ""
	for i = 1 to 4 do
	(
		str += bit.intAsChar (bit.and num 0xff)
		-- num = bit.shift num -8
		num /= 256
	)
	str
)

fn M3E_Convert_Time t =
(
	t * (1000 / g_useFPS)	
)

fn M3E_isInstance obj =
(
	InstanceMgr.GetInstances obj &instances
	
	local objInd = findItem instances obj
	if (objInd < instances.count) then return true else return false
)

fn M3E_Reset_Globals =
(
	head = mwrite = bstream = m3model = awrite = m3scene = undefined
	twrite = #()
	gc()
)

-- Writing Funcs
-- Helpers

fn M3E_WriteVec bitStream vec flags bflags:#null =
(
	case flags of
	(
		#v2d: (writeFloat bitStream vec.x; writeFloat bitStream vec.y;)
		#v3d: (writeFloat bitStream vec.x; writeFloat bitStream vec.y; writeFloat bitStream vec.z;)
		#c3: (local tempP3 = (vec as point3)/255; writeFloat bitStream tempP3.x; writeFloat bitStream tempP3.y; writeFloat bitStream tempP3.z;)
		#v3dshort: (writeShort bitStream vec.x #unsigned; writeShort bitStream vec.y #unsigned; writeShort bitStream vec.z #unsigned;)
		#v3dlong: (writeLong bitStream vec.x #unsigned; writeLong bitStream vec.y #unsigned; writeLong bitStream vec.z #unsigned;)
		#v3dbyte: (writeByte bitStream vec.x #unsigned; writeByte bitStream vec.y #unsigned; writeByte bitStream vec.z #unsigned;)
		#v4d: (WriteFloat bitStream vec.x; WriteFloat bitStream vec.y; WriteFloat bitStream vec.z; WriteFloat bitStream vec.w;)
		#v4dshort: (WriteShort bitStream vec.x #signed; WriteShort bitStream vec.y #signed; WriteShort bitStream vec.z #signed; WriteShort bitStream vec.w #signed;)
		#v4dbyte: (WriteByte bitStream vec.x; writeByte bitStream vec.y; writeByte bitStream vec.z; writeByte bitStream vec.w;)
		#v4dbyteFloat:
		(
			WriteByte bitStream ((vec.x * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.y * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.z * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.w * 255) as integer) #unsigned;
		)
		
		#v4dubyte:
		(
			WriteByte bitStream ((vec.x * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.y * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.z * 255) as integer) #unsigned;
			WriteByte bitStream ((vec.w * 255) as integer) #unsigned;
		)
		#v4col:
		(
			WriteByte bitStream ((vec.z * 255 * 255)) #unsigned;
			WriteByte bitStream ((vec.y * 255 * 255)) #unsigned;
			WriteByte bitStream ((vec.x * 255 * 255)) #unsigned;
			WriteByte bitStream ((vec.w * 255 * 255)) #unsigned;
		)
		#matrix: (M3E_WriteVec bitStream vec.row1 #v4d; M3E_WriteVec bitStream vec.row2 #v4d; M3E_WriteVec bitStream vec.row3 #v4d; M3E_WriteVec bitStream vec.row4 #v4d;)
		#sphere: (M3E_WriteVec bitStream vec.emin #v3d; M3E_WriteVec bitStream vec.emax #v3d; WriteFloat bitStream vec.rad;)
	)
	if (bflags == #MD34) then WriteLong bitStream 0x00 #unsigned
)

fn M3E_WriteBlank bitStream count flags:#buffer ret:false =
(
	local bm = ftell bitStream
	local bcount = 0 -- buffer count

	if (flags != #blank) then
	(
		bcount = (16 - (mod count 16)) -- calculate buffer bytes
		if (mod count 16 == 0) then bcount -= 16 -- so we don't get rows of 16 buffer bytes
	)
	
	case flags of
	(
		#buffer: 
		(
			count = 0 -- don't let it write null data
		)
	)
	
	for i = 1 to (count as integer) do (WriteByte bitStream 0) -- write blank bytes
	for i = 1 to (bcount as integer) do (WriteByte bitStream 0xAA #unsigned) -- write buffer bytes
	if (ret != false) then fseek bitStream bm #seek_set
)

fn FloatToShort &val = -- Float to Short func
(
	if (val > 0) then (val -= 1) else (val += 1)
	val *= 32767.0
)

fn M3E_getSize type flags:0 =
(
	local size
	case type of
	(
		#REF: size = 0x0C
		#AREF: size = 0x20
		#MD33: size = 0x14
		#MD34: size = 0x18
		#MODL:
		(
			case flags of
			(
				23: size = 0x310
			)
		)
		#SEQS: size = 0x60
		#STC_: size = 0xCC
		#SDEV: size = M3E_getSize #AREF
		#SD2V: size = M3E_getSize #AREF
		#SD3V: size = M3E_getSize #AREF
		#SD4Q: size = M3E_getSize #AREF
		#SDR3: size = M3E_getSize #AREF
		#SDS6: size = M3E_getSize #AREF
		#EVNT: size = 0x68
		#VEC2: size = 0x08
		#VEC3: size = 0x0C
		#QUAT: size = 0x10
		#REAL: size = 0x04
		#STG_: size = 0x18
		#STS_: size = 0x1C
		#BONE: size = 0xA0
		#VERT: size = 0x20
		#U16_: size = 0x02
		#I16_: size = 0x02
		#U32_: size = 0x04
		#I32_: size = 0x04
		#CHAR: size = 0x01
		#DIV_: size = 0x34
		#REGN: size = 0x24
		#BAT_: size = 0x0E
		#MSEC: size = 0x48
		#ATT_: size = 0x14
		#ATVL: size = 0x74
		#PAR_: size = 0x524
		#PARC: size = 0x28
		#RIB_: size= 0x2EC
		#SRIB: size = 0x110
		#MATM: size = 0x08
		#MAT_: size = 0x10C
		#DIS_: size = 0x44
		#TER_: size = 0x18
		#VOL_: size = 0x54
		#LAYR: size = 0x164
		#IREF: size = 0x40
		#CAM_: size = 0xB4
		#LITE: size= 0xD4
		#SSGS: size=0x6C
		#FOR_: size=0x68
	)
	
	return size
)

fn M3E_Name name flt ext:undefined =
(
	-- Filter name from path
	local mn = filterString name flt
	mn = mn[mn.count]
	local mext = substring mn (mn.count-3) (4)
	-- Remove extension if specified
	if (ext != undefined) then
	(
		if (matchPattern mext pattern:ext ignoreCase:true) then mn = substring mn 1 (mn.count-4)
	)
	return mn
)

fn M3E_FindParentIndex parentName =
(
	local boneIndex = 0
	for b = 1 to m3modl.bones.count do
	(
		local m3bone = m3modl.bones[b]
		if (bname == m3bone.maxObj.name) then boneIndex = b
	)
	
	return boneIndex
)

fn M3E_findParentBone boneList boneName =
(
	local boneIndex
	for b = 1 to boneList.count do
	(
		local m3bone = boneList[b]
		if (boneName == m3bone.maxObj.name) then boneIndex = b
	)
	return boneIndex
)

fn M3E_SceneCheck exporting =
(
	clearListener()
	objArray = #()
	helpersArray = #()
	bonesArray = #()
	foundErrors = #()
	foundMeshErrors = #()
	foundHelpersErrors = #()
	foundTestsErrors = #()
	foundMatErrors = #()
	noBonesError = #()
	noSkinError = #()
	helpersErrors = #()
	meshErrorMessage = ""
	
	classNames = #("sc2pemitter2","sc2ribbon","sc2boundsphere","sc2attachment")
	classNamesTests = #("sc2fuzzy","sc2tight")
	
	objArray = geometry
	helpersArray = helpers

	for i=1 to objArray.count do
	(
		local curObj = objArray[i]
		
		objClas = classOf curObj as string
		objMatClas = classOf curObj as string
		if (objClas != "BoneGeometry" ) then
		(
			if (objClas != "Editable_mesh" )
			then
			(
				append foundErrors curObj.name
			)
			
			if (objMatClas != "sc2material") then
			(
				append foundMatErrors curObj.name
			)
		)
	)

	if (foundErrors.count !=0) then
	(
		meshErrorMessage = "MESHES:\nFollowing objects are not Editable Meshes:\n"
		for i=1 to foundErrors.count do
		(
			meshErrorMessage += "  - " + (foundErrors[i] as string) + "\n"
		)
		meshErrorMessage += "\n  The exporter will not export them.\n\n\n"
	)

	if (foundMatErrors.count !=0) then
	(
		meshErrorMessage += "MATERIALS:\nFollowing objects don't have StarCraft II Material applied:\n"
		for i=1 to foundMatErrors.count do
		(
			meshErrorMessage += "  - " + (foundMatErrors[i] as string) + "\n"
		)
		meshErrorMessage += "\n  If you continue, your mesh will be entierly black in the StarCraft II Editor.\n\n\n"
	)

	if(meshErrorMessage=="") then
	(
		meshErrorMessage = "Good! No meshes or materials errors found.\nContinue with bones check test?\n"
	)
	else
	(
		format "=========SCENE CHECK RESULTS=========\n%" meshErrorMessage
		meshErrorMessage += "All errors have been printed to the listener (press F11 to access Listener).\n\nContinue with bones check?\nOnly Editable Meshes and StarCraft II specific helpers will be tested."
	)
	
	if queryBox meshErrorMessage title:"Scene check results:" beep:true then
	(
		for i=1 to objArray.count do
		(
			local curObj = objArray[i]
			
			objClas = classOf curObj as string
			if (objClas == "Editable_mesh")
			then
			(
				testSkin = curObj.modifiers[curObj.modifiers.count]
				if ((testSkin as string) == "Skin:Skin") then
				(
					if(skinOps.GetNumberBones testSkin == 0) then
					(
						append noBonesError curObj.name
					)
				)
				else
				(
					append noSkinError curObj.name
				)
			)
		)
		
		for i=1 to helpersArray.count do
		(
			local curHelper = helpersArray[i]
			
			helperClass = (classOf curHelper) as string
			for j=1 to classNames.count do
			(
				if (helperClass==classNames[j]) then
				(
					helperParentClass = classOf curHelper.parent as string
					
					if(curHelper.parent == undefined or helperParentClass != "BoneGeometry" or helperParentClass==UndefinedClass) then
					(
						append HelpersErrors curHelper.name
					)
				)
			)
			
			for j=1 to classNamesTests.count do
			(
				if (helperClass==classNamesTests[j]) then
				(
					helperParentClass = classOf curHelper.parent as string
					if(curHelper.parent == undefined or helperParentClass!="BoneGeometry") then
					(
						append foundTestsErrors curHelper.name
					)
				)
			)
		)
		
		meshErrorMessage = ""
		
		if (noSkinError.count!=0) then
		(
			meshErrorMessage = "Following objects don't have skin modifier applied:\n"
			for i=1 to noSkinError.count do
			(
				meshErrorMessage += "  - " + (noSkinError[i] as string) + "\n"
			)
		)
		
		if (noBonesError.count!=0) then
		(
			meshErrorMessage += "\nFollowing objects are not skinned to any bone:\n"
			for i=1 to noBonesError.count do
			(
				meshErrorMessage += "  - " + (noBonesError[i] as string) + "\n"
			)
		)
		
		if (foundTestsErrors.count!=0 or HelpersErrors.count!=0) then
		(
			meshErrorMessage += "\nFollowing helpers don't have parent bones or the parents are not bones:\n"
			for i=1 to foundTestsErrors.count do
			(
				meshErrorMessage += "  - " + (foundTestsErrors[i] as string) + "\n"
			)
			for i=1 to HelpersErrors.count do
			(
				meshErrorMessage += "  - " + (HelpersErrors[i] as string) + "\n"
			)
		)
		
		if (meshErrormessage =="") then
		(
			meshErrorMessage = "Great! No bone or skin errors found."
			format "\n=====END OF SCENE CHECK RESULTS======"
			messagebox meshErrorMessage title:"Bone check results:"
		)
		else
		(		
			meshErrorMessage += "\n\nAll errors have been printed to the listener (press F11 to access Listener).\n\nAll of the helpers need bones and all objects need to be skinned to\nat least one bone.\nTry using ADD BONES option in the M3 - Export dialog.\n"
			format "\n===========BONE TEST RESULTS=========\n%\n=====END OF SCENE CHECK RESULTS======\n" meshErrorMessage
				
			messagebox meshErrorMessage title:"Bone check results:"
		)
	)
	else
	(
		format "=====END OF SCENE CHECK RESULTS======\n"
	)
)

fn M3E_AddBones =
(
	selectionArray = selection as array
	for i=1 to selectionArray.count do
	(
		classNames = #("sc2pemitter2","sc2ribbon","sc2boundsphere","sc2attachment","sc2fuzzy","sc2tight")
		curObj = selectionArray[i]
		curClass = classOf curObj as string
		if (curClass == "Editable_mesh") then
		(
			testSkin = curObj.modifiers[curObj.modifiers.count]
			if ((testSkin as string) != "Skin:Skin") then
			(
				testSkin = Skin name:"Skin"
				addModifier curObj testSkin
			)
			if(skinOps.GetNumberBones testSkin == 0) then
			(
			newBone = BoneSys.createBone curObj.position curObj.position [0,0,1]
			newBone.name = curObj.name
			if(GetCommandPanelTaskMode() != #modify) then 
			(
				SetCommandPanelTaskMode mode:#modify
			)
			modPanel.setCurrentObject testSkin
			skinOps.addBone testSkin newBone 1
			)
		)

		else
		(
			for j=1 to classNames.count do
			(
				if(curClass == classNames[j]) then
				(
					if((classOf curObj.parent as string != "BoneGeometry") or (curObj.parent == undefined)) then
					(
						newBone = BoneSys.createBone curObj.position curObj.position [0,0,1]
						newBone.name = curObj.name
						curObj.parent = newBone
					)
				)
			)
		)
	)
)
-- ********************
--  MAX STRUCTURES
-- ********************
struct M3E_Model
(
	anims,
	vflags, verts, skin, bones, skinBones, 
	submeshes, usedMats, materials, attachments, pemits, parcs, sc2ribbons, sc2forces,
	cameras, sc2lights, sc2fuzzy, sc2tight,
	fn init =
	(
		local modl = M3E_Model()
		modl.bones = #()
		modl.skinBones = #()
		modl.materials = #()
		modl.attachments = #()
		modl.pemits = #()
		modl.parcs = #()
		modl.sc2ribbons = #()
		modl.sc2forces = #()
		modl.cameras = #()
		modl.sc2lights = #()
		modl.sc2fuzzy = #()
		modl.sc2tight = #()
		
		return modl
	)
)

struct M3E_SceneObjects 
(
	meshes, materials, 
	skin, bones, 
	attachments, pemits, parcs, parcIDs, sc2ribbons, sc2forces,
	vertFlags, bndSphere,
	cameras, sc2lights, sc2fuzzy, sc2tight,
	
	fn init =
	(
		local sObjs = M3E_SceneObjects()
		sObjs.meshes = #()
		sObjs.materials = #()
		sObjs.skin = #()
		sObjs.bones = #()
		sObjs.attachments = #()
		sObjs.pemits = #()
		sObjs.parcs = #()
		sObjs.sc2ribbons = #()
		sObjs.sc2forces = #()
		sObjs.parcIDs = #()
		sObjs.vertFlags = 25297021 -- initial vert flags
		sObjs.cameras = #()
		sObjs.sc2lights = #()
		sObjs.sc2fuzzy = #()
		sObjs.sc2tight = #()
		
		return sObjs
	)
)

struct M3E_MaxMaterials 
(
	mats = #(), multimats = #(), fullMatList = #()
)

-- **********************************
--  M3 STRUCTURES & FUNCTIONS
-- ********************************** 
struct M3E_Sphere
(
	emin, emax, rad, flags,
	maxObj,
	
	fn init =
	(
		local sph = M3E_Sphere()
		
		sph.emin = sph.emax = [0, 0, 0]
		sph.rad = 0
		sph.flags = 0
		
		return sph
	),
	fn GetBound objs =
	(
		sph = M3E_Sphere.init()
		clearSelection()
		select objs
		sph.emin = selection.min
		sph.emax = selection.max
		local vec3 = selection.max - selection.center
		clearSelection()
		
		local rad = 0
		for i = 1 to 3 do
		(
			-- set radius to highest x, y or z
			if (vec3[i] > rad) then rad = vec3[i]
		)
		-- increase radius slightly...
		rad *= 1.25
		sph.rad = rad
		
		return sph
	)
)

struct M3E_Matrix
(
	row1, row2, row3, row4,
	
	fn init =
	(
		local mat = M3E_Matrix()
		mat.row1 = [1,0,0,0]
		mat.row2 = [0,1,0,0]
		mat.row3 = [0,0,1,0]
		mat.row4 = [0,0,0,1]
		
		return mat
	),
	fn convObjMatrix objMat =
	(
		m3mat = M3E_Matrix.init()
		for i = 1 to 4 do
		(
			for j = 1 to 3 do
			(
				case i of
				(
					1: (m3mat.row1[j] = objMat.row1[j])
					2: (m3mat.row2[j] = objMat.row2[j])
					3: (m3mat.row3[j] = objMat.row3[j])
					4: (m3mat.row4[j] = objMat.row4[j])
				)
			)
		)
		
		return m3mat
	)
)

fn M3E_Name name flt ext:undefined =
(
	-- Filter name from path
	local mn = filterString name flt
	mn = mn[mn.count]
	-- Remove extension if specified
	if (ext != undefined) then
	(
		local extSize = ext.count
		local mext = substring mn (mn.count-(extSize - 1)) (extSize)
		if (matchPattern mext pattern:ext ignoreCase:true) then mn = substring mn 1 (mn.count-(extSize))
	)
	return mn
)

fn M3E_Open fname flags =
(
	step = "Accessing Model File"
	
	echo ("Writing "+fname+"...")
	local bitStream = fopen fname flags
	if bitStream==undefined then 
	(
		echo "File not found!"
		throw "File not found"
	)
	else return bitStream
)

fn M3E_Close bitStream =
(
	step = "Close"
	fclose bitStream
)

struct M3E_Tag -- Tag data
(
	dataID, ofsData, nData, flags,
	
	fn Write bitStream tag =
	(
		-- head to end of file
		fseek bitStream 0 #seek_end
		
		-- Fix Header info
		head.ofsTag = ftell bitStream
		head.nTag = tag.count
		
		
		-- write tags
		for i = 1 to tag.count do
		(
			t = tag[i]
			writeString bitStream t.dataID -- Write File ID
			fseek bitStream -1 #seek_cur
			writeLong bitStream t.ofsData #unsigned
			writeLong bitStream t.nData #unsigned
			writeLong bitStream t.flags #unsigned
		)
	),
	fn TagAppend bitStream nameID nData tagar flags:0 =
	(
		tag = M3E_Tag()
		local strID = nameID as string
		local tagID = "BLNK" -- Needs to be 4 chars for modification
		for i = 1 to 4 do
		(
			k = 5 - i
			tagID[i] = strID[k]
		)
		tag.dataID = M3E_toUpper tagID -- Uppercase
		tag.ofsData = ftell bitStream
		tag.nData = nData
		tag.flags = flags
		
		append tagar tag
	)
)

struct M3E_Ref
(
	entries, refid, flags = 0, data = #(),
	
	fn Write bitStream ref =
	(
		writeLong bitStream ref.entries #unsigned
		writeLong bitStream ref.refid #unsigned
		WriteLong bitStream ref.flags #unsigned
	)
)

struct M3E_AnimRef -- Animation Reference
(
	flags, animflags, animid, initValue, nullValue,
	
	fn Write bitStream ar type =
	(
		WriteShort bitStream ar.flags #unsigned
		WriteShort bitStream ar.animflags #unsigned
		WriteLong bitStream ar.animid #unsigned
		
		case type of
		(
			#VEC2D:(M3E_WriteVec bitStream ar.initValue #v2d; M3E_WriteVec bitStream ar.nullValue #v2d;)
			#VEC3D:(M3E_WriteVec bitStream ar.initValue #v3d; M3E_WriteVec bitStream ar.nullValue #v3d;)
			#COLOUR3:(M3E_WriteVec bitStream ar.initValue #c3; M3E_WriteVec bitStream ar.nullValue #c3;)
			#VEC3DSHRT:(M3E_WriteVec bitStream ar.initValue #v3dshort; M3E_WriteVec bitStream ar.nullValue #v3dshort;)
			#VEC4D:(M3E_WriteVec bitStream ar.initValue #v4d; M3E_WriteVec bitStream ar.nullValue #v4d;)
			#COLOUR:
			(
				(M3E_WriteVec bitStream ar.initValue #v4col; M3E_WriteVec bitStream ar.nullValue #v4col;)
			)
			#UINT32:(writeLong bitStream ar.initValue #unsigned; writeLong bitStream ar.nullValue #unsigned;)
			#UINT16:(writeShort bitStream ar.initValue #unsigned; writeShort bitStream ar.nullValue #unsigned;)
			#FLOAT:(writeFloat bitStream ar.initValue; writeFloat bitStream ar.nullValue;)
			#SPHERE:(M3E_WriteVec bitStream ar.initValue #sphere; M3E_WriteVec bitStream ar.nullValue #sphere;)
		)
		
		M3E_WriteBlank bitStream 0x04 flags:#blank
	),
	
	fn init type =
	(
		ar = M3E_AnimRef()
		ar.flags = 0
		ar.animflags = 0
		-- random uint32
		ar.animid = Random 1 1e8
		
		local arValue
		
		case type of
		(
			#VEC2D:(arValue = [0, 0])
			#VEC3D:(arValue = [0, 0, 0])
			#VEC3DSHRT:(arValue = [0, 0, 0])
			#VEC4D:(arValue = [0, 0, 0, 1])
			#COLOUR:(arValue = [255, 255, 255, 255])
			#COLOUR3:(arValue = [255, 255, 255])
			#UINT32:(arValue = 0)
			#UINT16:(arValue = 0)
			#FLOAT:(arValue = 0.0)
			#SPHERE:(arValue = M3E_Sphere.init())
		)
		
		ar.initValue = ar.nullValue = arValue
		if (type == #COLOUR) then ar.nullValue = [0,0,0,0]
		if (type == #COLOUR3) then ar.nullValue = [0, 0, 0]
		
		return ar
	)
)

struct M3E_Header
(
	fileID, ofsTag, nTag, mref = M3E_Ref(),
	
	fn Write bitStream head =
	(
		step = "Write Header"
		fseek bstream 0 #seek_set -- move to beginning of file
		
		writeString bitStream head.fileID -- Write File ID
		fseek bitStream -1 #seek_cur -- writeString writes null terminator...so have to move cursor back one space
		writeLong bitStream head.ofsTag #unsigned
		writeLong bitStream head.nTag #unsigned
		M3E_Ref.Write bitStream head.mref
	),
	fn init =
	(
		h = M3E_Header()
		-- Fixed file magic ID
		h.fileID = "43DM"
		-- Blank these for now, fix em up during tag writing
		h.ofsTag = 0
		h.nTag = 0
		-- Use fixed MODL ref...
		h.mref.entries = 1
		h.mref.refid = 1
		
		return h
	)
)

struct M3E_MODL
(
	name,  -- max model name
	versID,
	SEQS, STC, STG, vec1, STS,
	bones, nSkinBones,
	vflags, verts, DIV, bonelu,
    bndSphere, attach, attachlu, attachvol, attachvollu,
	matm, mat, dis, ter, vol,
	par, parc, sc2ribbon, sc2force,
	IREF,
	cameras, sc2lights, sc2fuzzy, sc2tight,
	
	fn Write bitStream modl vers = 
	(
		-- goto modl offset first
		fseek bitStream twrite[2].ofsData #seek_set
		
		-- Write MODL chunk
		--M3E_WriteRef bitStream modl.name #CHAR flags:0
		format "MODL name: %\n" mwrite.name
		M3E_WriteRef bitStream modl.name #CHAR flags:0
		format "MODL name: %\n" mwrite.name
		WriteLong bitStream modl.versID #unsigned
		M3E_WriteRef bitStream modl.SEQS #SEQS flags:1
		M3E_WriteRef bitStream modl.STC #STC_ flags:4
		M3E_WriteRef bitStream modl.STG #STG_ flags:0
		M3E_WriteVec bitStream modl.vec1 #v3d bflags:#MD34
		M3E_WriteRef bitStream modl.STS #STS_
		M3E_WriteRef bitStream modl.bones #BONE flags:1
		WriteLong bitStream modl.nSkinBones #unsigned
		WriteLong bitStream modl.vflags #unsigned
		case of
		(
			(bit.get modl.vflags 19 == true): M3E_WriteRef bitStream modl.verts #VERT subtype:#v36
			default: M3E_WriteRef bitStream modl.verts #VERT
		)
		M3E_WriteRef bitStream modl.DIV #DIV_ flags:2
		M3E_WriteRef bitStream modl.bonelu #U16_ subtype:#uint16
		M3E_WriteVec bitStream modl.bndSphere #sphere bflags:#MD34
		M3E_WriteBlank bitStream 0x1C flags:#blank
		M3E_WriteBlank bitStream 0x1C flags:#blank
		M3E_WriteBlank bitStream 0x04 flags:#blank
		format "Attachment Points: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.attach #ATT_ flags:1
		format "Attachment Points Addons: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.attachlu #U16_ subtype:#int16
		format "Lights: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.sc2lights #LITE flags:7
		M3E_WriteBlank bitStream 0x0C flags:#blank
		format "Cameras: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.cameras #CAM_ flags:3
		M3E_WriteBlank bitStream 0x0C flags:#blank
		--end
--		M3E_WriteBlank bitStream 0x30 flags:#blank
		format "MATM: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.MATM #MATM
		format "MAT: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.MAT #MAT_ flags:15
		--M3E_WriteBlank bitStream 0x18 flags:#blank
		format "DIS: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.DIS #DIS_ flags:4
		M3E_WriteBlank bitStream 0x0C flags:#blank
		format "TER: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.TER #TER_
		format "VOL: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.VOL #VOL_ flags:0
		M3E_WriteBlank bitStream 0x18 flags:#blank
		format "Particles: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.PAR #PAR_ flags:12
		format "Particle Copies: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.PARC #PARC flags:0
		format "Ribbons: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.sc2ribbon #RIB_ flags:6
		M3E_WriteBlank bitStream 0x0C flags:#blank
		M3E_WriteBlank bitStream 0x0C flags:#blank
		--format "Forces: %\n" (ftell bitStream)
		--M3E_WriteRef bitStream modl.sc2force #FOR_ flags:4
		M3E_WriteBlank bitStream 0x78 flags:#blank
		format "IREF: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.iref #IREF
		format "TIGHT: %\n" (ftell bitStream)
		if (modl.sc2tight[1] == undefined) then
		(
			modl.sc2tight[1] = M3E_Tight.init()
		)
		format "modl.sc2tight[1]: %\n" modl.sc2tight[1]
		M3E_WriteTight bitStream modl.sc2tight[1]
		format "FUZZY: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.sc2fuzzy #SSGS flags:1
		format "Attachvol: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.attachvol #ATVL
		format "Attachvoladd: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.attachvollu #U16_ subtype:#int16
		format "Attachvoladd2: %\n" (ftell bitStream)
		M3E_WriteRef bitStream modl.attachvollu #U16_ subtype:#int16 -- duplicate lookups? always 0 anyways
		M3E_WriteBlank bitStream 0x28 flags:#blank
	)
)

struct M3E_SEQS
(
	d1, name, animStart, maxframes, moveSpeed, seqFlags, frequency, d3, bndSphere, d4,
	cstart, cend, -- Max variables
	
	fn Write bitStream ad =
	(
		M3E_WriteArray bitStream ad.d1 flags:#int32
		M3E_WriteRef bitStream ad.name #CHAR
		WriteLong bitStream ad.animStart #unsigned
		WriteLong bitStream ad.maxframes #unsigned
		WriteFloat bitStream ad.moveSpeed
		WriteLong bitStream ad.seqFlags #unsigned
		WriteLong bitStream ad.frequency #unsigned
		M3E_WriteArray bitStream ad.d3 flags:#uint32
		M3E_WriteVec bitStream ad.bndSphere #sphere bflags:#MD34
		M3E_WriteArray bitStream ad.d4 flags:#int32
	),
	fn init =
	(
		a = M3E_SEQS()
		
		a.d1 = #(-1, 1)
		a.name = "Stand"
		a.animStart = 0
		a.maxframes = 10000
		a.moveSpeed = 0
		a.seqFlags = 0
		a.frequency = 100
		a.d3 = #(1, 1, 100, 0)
		a.bndSphere = M3E_Sphere.init()
		a.d4 = #(0, 0)
		
		return a
	)
)

struct M3E_Event
(
	name, d1, s1, s2, mat1, d2,
	
	fn Write bitStream evnt =
	(
		M3E_WriteRef bitStream evnt.name #CHAR
		WriteLong bitStream evnt.d1 #signed
		WriteShort bitStream evnt.s1 #signed
		WriteShort bitStream evnt.s2 #unsigned
		M3E_WriteVec bitStream evnt.mat1 #matrix bflags:#null
		M3E_WriteArray bitStream evnt.d2 flags:#uint32
	),
	fn init =
	(
		local evnt = M3E_Event()
		
		evnt.name = "Evt_SeqEnd"
		evnt.d1 = -1
		evnt.s1 = -1
		evnt.s2 = 0
		evnt.mat1 = M3E_Matrix.init()
		evnt.d2 = #(4, 0, 0, 0)
		
		return evnt
	)
)

struct M3E_AnimBlock -- Sequence Data
(
	frames, flags, fend, keys, animid,
	
	fn Write bitStream ablock sdtype flags:0 =
	(
		M3E_WriteRef bitStream ablock.frames #I32_ subtype:#uint32
		WriteLong bitStream ablock.flags #unsigned
		WriteLong bitStream ablock.fend #unsigned
		M3E_WriteRef bitStream ablock.keys sdtype flags:flags
	),
	fn init =
	(
		local ab = M3E_AnimBlock()
		ab.frames = #()
		ab.flags = 0
		ab.fend = 0
		ab.keys = #()
		
		return ab
	),
	fn initEvnt =
	(
		local a = M3E_AnimBlock.init()
		a.frames = #(9999)
		a.flags = 1
		a.fend = 10000
		a.keys = #(M3E_Event.init())
		
		return a
	)
)

struct M3E_Animoffs 
(
	aind, sdind,
	
	fn Write bitStream aoffs =
	(
		WriteShort bitStream aoffs.aind #unsigned
		WriteShort bitStream aoffs.sdind #unsigned
	),
	fn init =
	(
		local ao = M3E_Animoffs()
		ao.aind = 0
		ao.sdind = 0
		
		return ao
	)
)

struct M3E_STC --Sequence Transformations Collection (STC)
(
	name, d1, seqlu, stglu, animid, animref, d2, sdata,
	seqInd, cstart, cend, frequency, moveSpeed, looping,
	
	fn Write bitStream stc =
	(
		-- M3E_WriteRef bitStream stc.name #CHAR
		-- for some reason, Blizzard M3's are written with the animid and animref references first before the name reference
		M3E_WriteBlank bitStream (M3E_getSize #REF) flags:#blank
		
		WriteLong bitStream stc.d1 #unsigned
		WriteShort bitStream stc.seqlu #unsigned
		WriteShort bitStream stc.stglu #unsigned
		M3E_WriteRef bitStream stc.animid #U32_ subtype:#uint32
		M3E_WriteRef bitStream stc.animref #U32_ subtype:#animind
		-- create file bookmark to return to
		local bm = ftell bitStream
		
		-- seek to char reference now...
		fseek bitStream (-0x2C) #seek_cur
		-- and write the char ref
		M3E_WriteRef bitStream stc.name #CHAR
		-- then return to the bookmark
		fseek bitStream bm #seek_set
		
		WriteLong bitStream stc.d2 #unsigned
		for i = 1 to stc.sdata.count do
		(
			sd = stc.sdata[i]
			case i of
			(
				1: M3E_WriteRef bitStream stc.sdata[1] #SDEV
				2: M3E_WriteRef bitStream stc.sdata[2] #SD2V
				3: M3E_WriteRef bitStream stc.sdata[3] #SD3V
				4: M3E_WriteRef bitStream stc.sdata[4] #SD4Q
				6: M3E_WriteRef bitStream stc.sdata[6] #SDR3
				8: M3E_WriteRef bitStream stc.sdata[8] #SDS6
				default: M3E_WriteRef bitStream stc.sdata[i] #null
			)
		)
	),
	fn init =
	(
		local stc = M3E_STC()
		stc.name = "Stand_full"
		stc.d1 = 0
		stc.seqlu = 0
		stc.stglu = 0
		stc.animid = #(Random 1 1e8) -- must be array
		stc.animref = #(M3E_Animoffs.init())
		stc.d2 = 0
		stc.sdata = #()
		-- initialize array
		stc.sdata[1] = #(M3E_AnimBlock.initEvnt())
		
		return stc
	),
	fn Blank =
	(
		local stc = M3E_STC.init()
		stc.d1 = 0
		stc.d2 = 0
		stc.animid = #()
		stc.animref = #()
		stc.sdata[13] = 0
		for i = 1 to 13 do stc.sdata[i] = #()
		
		return stc
	)
)

struct M3E_STG
(
	name, stcind,
	
	fn Write bitStream stg =
	(
		M3E_WriteRef bitStream stg.name #CHAR
		M3E_WriteRef bitStream stg.stcind #U32_ subtype:#uint32
	),
	fn init =
	(
		local stg = M3E_STG()
		stg.name = "Stand"
		stg.stcind = #(0)
		
		return stg
	)
)

struct M3E_STS
(
	animid, d1, s1, s2,
	
	fn Write bitStream sts =
	(
		M3E_WriteRef bitStream sts.animid #U32_ subtype:#uint32
		M3E_WriteArray bitStream sts.d1 flags:#int32
		WriteShort bitStream sts.s1 #signed
		WriteShort bitStream sts.s2 #unsigned
	),
	fn init =
	(
		local sts = M3E_STS()
		sts.animid = #()
		sts.d1 = #(-1,-1,-1)
		sts.s1 = -1
		sts.s2 = 0
		
		return sts
	),
	fn initBlank modl =
	(
		local sts = M3E_STS.init()
		append sts.animid modl.stc[1].animid[1]
		
		return sts
	)
)

struct M3E_MaxBone
(
	name, boneIndex, parentptr, parent, children,
	bindmat, invbindmat, maxObj, iref,
	skinned
)

struct M3E_Bone
(
	d1, name, flags, parent, s1, 
	trans, rot, bscale, ar1,
	iref, maxObj,
	
	fn Write bitStream bone =
	(
		WriteLong bitStream bone.d1 #signed
		M3E_WriteRef bitStream bone.name #CHAR
		WriteLong bitStream bone.flags #unsigned
		WriteShort bitStream bone.parent #signed
		WriteShort bitStream bone.s1 #unsigned
		M3E_AnimRef.Write bitStream bone.trans #VEC3D
		M3E_AnimRef.Write bitStream bone.rot #VEC4D
		M3E_AnimRef.Write bitStream bone.bscale #VEC3D
		M3E_AnimRef.Write bitStream bone.ar1 #UINT32
	),
	
	fn init =
	(
		b = M3E_Bone()
		
		b.d1 = -1
		b.name = "Main"
		local flags = 0
		b.flags = flags
		b.parent = -1
		b.s1 = 0
		
		b.trans = M3E_AnimRef.init #VEC3D
		b.rot = M3E_AnimRef.init #VEC4D
		
		-- setup scale
		b.bscale = M3E_AnimRef.init #VEC3D
		b.bscale.initValue = [1, 1, 1]
		b.bscale.nullValue = [1, 1, 1]
		
		b.ar1 = M3E_AnimRef.init #UINT32
		b.ar1.initValue = 1
		b.ar1.nullValue = 1
		
		b.iref = M3E_Matrix.init()
		
		return b
	)		
)

struct M3E_Vertex
(
	pos, bw, bi, normal, uv, d1, d2,
	faceref, origvertindx, origtvertindx, -- for script purposes only
	
	fn Write bitStream vert =
	(
		M3E_WriteVec bitStream vert.pos #v3d
		M3E_WriteArray bitStream vert.bw flags:#uint8
		M3E_WriteArray bitStream vert.bi flags:#uint8
		M3E_WriteArray bitStream vert.normal flags:#uint8
		M3E_WriteArray bitStream vert.uv flags:#uvshorts
		--WriteLong bitStream vert.d1 #unsigned
		WriteLong bitStream vert.d2 #unsigned
	),
	fn init =
	(
		local v = M3E_Vertex()
		v.pos = [0,0,0]
		v.bw = #(0,0,0,0)
		v.bi = #(0,0,0,0)
		v.normal = #(0,0,0,0)
		v.uv = #()
		v.d1 = 0
		v.d2 = 0
		v.faceref = #()
		
		return v
	)
)

struct M3E_REGN
(
	s1, indvert, nvert, 
	indface, nface, 
	bcount, indbonelu, nbonelu,
	s2, b1, s3,
	verts, faces, vertsList, facesList, skinBones, maxMesh, maxSkin,-- maxscript holders

	fn Write bitStream regn =
	(
		M3E_WriteArray bitStream regn.s1 flags:#uint32
		WriteLong bitStream regn.indvert #unsigned
		WriteLong bitStream regn.nvert #unsigned
		WriteLong bitStream regn.indface #unsigned
		WriteLong bitStream regn.nface #unsigned
		WriteShort bitStream regn.bcount #unsigned
		WriteShort bitStream regn.indbonelu #unsigned
		WriteShort bitStream regn.nbonelu #unsigned
		WriteShort bitStream regn.s2 #unsigned
		M3E_WriteArray bitStream regn.b1 flags:#uint8
		WriteShort bitStream regn.s3 #unsigned
	),
	fn init =
	(
		local r = M3E_REGN()
		r.s1 = #(0,0)
		r.bcount = 1
		r.indbonelu = 0
		r.nbonelu = 1
		r.s2 = 0
		r.b1 = #(1, 1)
		r.s3 = 0
		r.verts = #()
		r.faces = #()
		r.vertsList = #()
		r.facesList = #()
		r.skinBones = #()
		
		return r
	)
)

struct M3E_BAT
(
	d1, subid, d2, matid, s1,
	
	fn Write bitStream bat =
	(
		WriteLong bitStream bat.d1 #unsigned
		WriteShort bitStream bat.subid #unsigned
		WriteLong bitStream bat.d2 #unsigned
		WriteShort bitStream bat.matid #unsigned
		WriteShort bitstream bat.s1 #signed
	),
	fn init =
	(
		local bat = M3E_BAT()
		bat.d1 = 0
		bat.d2 = 0
		bat.s1 = -1
		
		return bat
	)
)

struct M3E_MSEC
(
	d1 = 0, ar1 = M3E_AnimRef.init #SPHERE,
	
	fn Write bitStream msec =
	(
		WriteLong bitStream msec.d1 #unsigned
		M3E_AnimRef.Write bitStream msec.ar1 #SPHERE
	)
)

struct M3E_DIV -- Model divisions
(
	faces, regn, bat, msec, d1,
	
	fn Write bitStream div =
	(
		M3E_WriteRef bitStream div.faces #U16_ subtype:#uint16
		M3E_WriteRef bitStream div.regn #REGN flags:3
		M3E_WriteRef bitStream div.bat #BAT_ flags:1
		M3E_WriteRef bitStream div.msec #MSEC flags:1
		WriteLong bitStream div.d1 #unsigned
	),
	fn init =
	(
		local div = M3E_DIV()
		div.faces = #()
		div.regn = #()
		div.bat = #()
		div.msec = #()
		div.d1 = 1
		
		return div
	)
)

struct M3E_Subref -- Submesh refrences
(
	subid, matid
)

struct M3E_Attachment
(
	flags, name, bone, maxObj,
	
	fn Write bitStream m3attach =
	(
		WriteLong bitStream m3attach.flags #signed
		M3E_WriteRef bitStream m3attach.name #CHAR
		WriteLong bitStream m3attach.bone #unsigned
	),
	fn init =
	(
		local att = M3E_Attachment()
		att.flags = -1
		
		return att
	)
)

struct M3E_AttachVolume
(
	bone, d1, mat, rad,
	
	fn Write bitStream m3attvol =
	(
		for i = 1 to 2 do
		(
			WriteLong bitStream m3attvol.bone #unsigned
		)
		
		WriteLong bitStream m3attvol.d1 #unsigned
		WriteLong bitStream m3attvol.bone #unsigned
		M3E_WriteVec bitStream m3attvol.mat #matrix
		M3E_WriteBlank bitStream 0x18 flags:#blank
		WriteFloat bitStream m3attvol.rad
		M3E_WriteBlank bitStream 0x08 flags:#blank
	),
	fn init =
	(
		local av = M3E_AttachVolume()
		av.d1 = 1
		av.mat = M3E_Matrix.init()
		
		return av
	)
)

struct M3E_Tight
(
	bone, matrix, tightType, tightSize0, tightSize1, tightSize2, maxObj,
	
	fn Write bitStream tight =
	(
		WriteLong bitStream (tight.tightType - 1) #unsigned
		WriteShort bitStream tight.bone #unsigned
		WriteShort bitStream 0
		M3E_WriteVec bitStream tight.matrix #matrix
				
		M3E_WriteBlank bitStream 24 flags:#blank
		
		WriteFloat bitStream tight.tightSize0
		WriteFloat bitStream tight.tightSize1
		WriteFloat bitStream tight.tightSize2
	),
	
	fn init =
	(
		local tht = M3E_Tight()
		
		tht.bone = -1
		tht.matrix = M3E_Matrix.init()
		tht.tightType = 2
		tht.tightSize0 = 0
		tht.tightSize1 = 0
		tht.tightSize2 = 0
		
		return tht
	)
)

struct M3E_SSGS
(
	bone, matrix, fuzzyType, fuzzySize0, fuzzySize1, fuzzySize2, maxObj,
	
	fn Write bitStream ssgs =
	(
		WriteLong bitStream (ssgs.fuzzyType - 1) #unsigned
		WriteShort bitStream ssgs.bone #unsigned
		WriteShort bitStream 0
		M3E_WriteVec bitStream ssgs.matrix #matrix
		
		M3E_WriteBlank bitStream 24 flags:#blank
		
		WriteFloat bitStream ssgs.fuzzySize0
		WriteFloat bitStream ssgs.fuzzySize1
		WriteFloat bitStream ssgs.fuzzySize2
	),
	
	fn init =
	(
		local fht = M3E_SSGS()
		
		fht.bone = 0
		fht.matrix = M3E_Matrix.init()
		fht.fuzzyType = 0
		fht.fuzzySize0 = 0
		fht.fuzzySize1 = 0
		fht.fuzzySize2 = 0
		
		return fht
	)
)

struct M3E_MATM
(
	matType, matind,
	
	fn Write bitStream matm =
	(
		WriteLong bitStream matm.matType #unsigned
		WriteLong bitStream matm.matind #unsigned
	),
	fn init =
	(
		local matm = M3E_MATM()
		matm.matType = 4
		
		return matm
	),
	fn Read mats =
	(
		marray = #()
		for i = 1 to mats.count do
		(
			m = M3E_MATM.init()
			m.matind = i - 1
			
			append marray m
		)
		
		return marray
	)
)

struct M3E_LAYR --more work to be done at a later point
(
	name, flags, mappingSource, alphaFlags, FalloffFlags, FalloffStrength, FalloffStart, FalloffCutout, FalloffUnk,
	layerColour,
	bright_mult, MidtoneOffset,
	f1, f2, d1, d2,
	UVOffset, UVAngle, UVTiling, unkMapInt,
	ar1, ar2, ar3, ar4, ar5,
	brightness,
	maxBitmap,
	
	fn Write bitStream layr =
	(
		WriteLong bitStream 0 #unsigned
		M3E_WriteRef bitStream layr.name #CHAR flag:0
		M3E_AnimRef.Write bitStream layr.layerColour #COLOUR
		WriteLong bitStream layr.flags #unsigned
		WriteLong bitStream layr.mappingSource #unsigned
		WriteLong bitStream layr.alphaFlags #unsigned
		M3E_AnimRef.Write bitStream layr.bright_mult #FLOAT
		M3E_AnimRef.Write bitStream layr.MidtoneOffset #FLOAT
		M3E_WriteBlank bitStream 0x04 flags:#blank
		WriteLong bitStream -1 #signed
		M3E_WriteBlank bitStream 0x08 flags:#blank
		WriteLong bitStream layr.d1 #signed
		M3E_WriteBlank bitStream 0x08 flags:#blank
		M3E_AnimRef.Write bitStream layr.ar1 #UINT32
		M3E_AnimRef.Write bitStream layr.ar2 #VEC2D
		M3E_AnimRef.Write bitStream layr.ar3 #UINT16
		M3E_AnimRef.Write bitStream layr.UVOffset #VEC2D
		M3E_AnimRef.Write bitStream layr.UVAngle #VEC3D
		M3E_AnimRef.Write bitStream layr.UVTiling #VEC2D
		M3E_AnimRef.Write bitStream layr.ar4 #UINT32
		M3E_AnimRef.Write bitStream layr.ar5 #FLOAT
		M3E_AnimRef.Write bitStream layr.brightness #FLOAT
		WriteLong bitStream layr.unkMapInt #signed
		WriteLong bitStream layr.FalloffFlags #unsigned
		WriteFloat bitStream layr.FalloffStrength
		WriteFloat bitStream layr.FalloffStart
		WriteFloat bitStream layr.FalloffCutout
		WriteFloat bitStream layr.FalloffUnk
		
	),
	fn init =
	(
		local layr = M3E_LAYR()
		
		layr.flags = 236
		layr.mappingSource = 0
		layr.alphaFlags = 0
		layr.layerColour = M3E_AnimRef.init #COLOUR
		layr.layerColour.initValue = layr.layerColour.nullValue = [0,0,0,0]
		layr.bright_mult = M3E_AnimRef.init #FLOAT
		layr.bright_mult.initValue = layr.bright_mult.nullValue = 1.0
		layr.MidtoneOffset = M3E_AnimRef.init #FLOAT
		layr.MidtoneOffset.initValue = layr.MidtoneOffset.nullValue = 0.0
		layr.f2 = 1.0
		layr.d1 = 0
		layr.ar1 = M3E_AnimRef.init #UINT32
		layr.ar2 = M3E_AnimRef.init #VEC2D
		layr.ar3 = M3E_AnimRef.init #UINT16
		layr.UVOffset = M3E_AnimRef.init #VEC2D
		layr.UVAngle = M3E_AnimRef.init #VEC3D
		layr.UVTiling = M3E_AnimRef.init #VEC2D
		layr.UVTiling.initValue = layr.UVTiling.nullValue = [1.0, 1.0]
		layr.ar4 = M3E_AnimRef.init #UINT32
		layr.ar5 = M3E_AnimRef.init #FLOAT
		layr.ar5.initValue = layr.ar5.nullValue = 1.0
		layr.brightness = M3E_AnimRef.init #FLOAT
		layr.brightness.initValue = layr.brightness.nullValue = 1.0
		layr.unkMapInt = 1
		layr.FalloffFlags = 0
		layr.FalloffStrength = 4.0
		layr.FalloffStart = 0.0
		layr.FalloffCutout = 1.0
		layr.FalloffUnk = 1.0
		
		return layr
	)
)

struct M3E_Dis
(
	name, flags, priorityDis, layrs, maxMat, globStr,
	
	fn Write bitStream dis =
	(
		M3E_WriteRef bitStream dis.name #CHAR flags:0
		M3E_WriteBlank bitStream 0x04 flags:#blank
		M3E_AnimRef.Write bitStream dis.globStr #FLOAT
		for i = 1 to 2 do M3E_WriteRef bitStream dis.layrs[i] #LAYR flags:22
		WriteLong bitStream dis.flags #unsigned
		WriteLong bitStream dis.priorityDis #unsigned
		
	),
	fn init mmat = 
	(
		local dis = M3E_Dis()
		dis.layrs = #()
		dis.flags = 0
		dis.priorityDis = 0
		dis.globStr = M3E_AnimRef.init #FLOAT
		dis.globStr.initValue = dis.globStr.NullValue = 0.0
		
		return dis
	)
	
)

struct M3E_Mat
(
	name, flags, blendMode, priority, obFlags,
	spec, specF, alphaMaskEnable, spec_mult, emis_mult, cutoutThresh,
	layerBlendType, emisBlendType, emisMode, specType,
	layrs, ar1, ar2,
	maxMat, -- max variable
	
	fn Write bitStream mat =
	(
		M3E_WriteRef bitStream mat.name #CHAR
		WriteLong bitStream mat.obFlags #unsigned
		--WriteLong bitStream 512 #unsigned
		WriteLong bitStream mat.flags #unsigned
		WriteLong bitStream mat.blendMode #unsigned
		WriteLong bitStream mat.priority #unsigned
		M3E_WriteBlank bitStream 0x04 flags:#blank
		WriteFloat bitStream mat.spec
		WriteLong bitStream mat.specF #unsigned
		WriteLong bitStream mat.cutoutThresh #unsigned
		WriteFloat bitStream mat.spec_mult
		WriteFloat bitStream mat.emis_mult
		-- Layer Refs
		for i = 1 to 13 do M3E_WriteRef bitStream mat.layrs[i] #LAYR flags:22
		
		M3E_WriteBlank bitStream 0x04 flags:#blank
		WriteLong bitStream mat.layerBlendType #unsigned
		WriteLong bitStream mat.emisBlendType #unsigned
		WriteLong bitStream mat.emisMode #unsigned
		WriteLong bitStream mat.specType #unsigned
		
		M3E_AnimRef.Write bitStream mat.ar1 #UINT32
		M3E_AnimRef.Write bitStream mat.ar2 #UINT32
	),
	fn init = 
	(
		local mat = M3E_Mat()
		mat.obFlags = 0
		mat.flags = 0
		mat.blendMode = 0
		mat.priority = 0
		mat.spec = 0
		mat.specF = 0.0
		mat.alphaMaskEnable = 0
		mat.spec_mult = 1
		mat.emis_mult = 1
		mat.cutoutThresh = 0
		mat.layerBlendType = 2
		mat.emisBlendType = 2
		mat.emisMode = 2
		mat.specType = 0
		mat.layrs = #()
		
		-- unknown animref's
		mat.ar1 = M3E_AnimRef.init #UINT32
		mat.ar2 = M3E_AnimRef.init #UINT32
		
		return mat
	)
)

struct M3E_Vol
(
	name, layrs, maxMat, volDensity, --volFloat1, volFloat2, volInt1, volInt2,
	
	fn Write bitStream vol =
	(
		M3E_WriteRef bitStream vol.name #CHAR flags:0
		
		WriteLong bitStream 0
		WriteLong bitStream 0
		M3E_AnimRef.Write bitStream vol.volDensity #FLOAT
		for i = 1 to 3 do M3E_WriteRef bitStream vol.layrs[i] #LAYR flags:22
		WriteLong bitStream 0
		WriteLong bitStream 32

	),
	fn init = 
	(
		local vol = M3E_Vol()
		vol.layrs = #()
		
		vol.volDensity = M3E_AnimRef.init #FLOAT
		vol.volDensity.initValue = vol.volDensity.nullValue = 0.0
		--vol.volInt1 = 0
		--vol.volInt2 = 0
		--vol.volFloat1 = 0
		--vol.volFloat2 = 0
		
		return vol
	)
	
)

struct M3E_Ter
(
	name, map,
	maxMat,
	
	fn Write bitStream ter =
	(
		M3E_WriteRef bitStream ter.name #CHAR
		M3E_WriteRef bitStream ter.map #LAYR flags:22
	),
	fn init =
	(
		local ter = M3E_Ter()
		ter.map = M3E_Layr.init
		
		return ter
	)
)

struct M3E_PAR
(
	-- known values
	/*bone, matmIndex, 
	initEmissSpeed, speedVar, enableSpeedVar, angleX, angleY, spreadX, spreadY,
	lifespan, decay, enableDecay,
	pemitScale, colour1, enableReverseFlow,
	minParticles, maxParticles, emissRate, partEmit, type, emissArea, pivotSpread,
	enableRotate, pemitRotate, enableColour2, colour2, columns, rows, pemitFlags,
	enableTrailing, enableStreak, enableRadialEmission,
	
	-- unknown values
	emissSpeed2, emissSpeed3, scaleRatio, f1, speedUnk1, lifespanRatio, f2, f3, tailUnk1, spreadUnk1,
	enableScale2, pemitScale2, speedUnk2, f4, f5, i1, f6, f7, ar2, ar3,
	
	-- stupid massive amount of unused anim ref's
	ar4, ar5, ar6, ar7, ar8, ar9, ar10, ar11, ar12, ar13, ar14, 
	ar15, ar16, ar17, ar18, ar19, ar20, ar21, ar22, ar23, ar24,*/
	
	bone, matmIndex	, emissionSpeed1, emissionSpeed2,
	randomizeWithEmissionSpeed2, emissionAngleX, emissionAngleY,
	emissionSpreadX, emissionSpreadY, lifespan1, lifespan2,
	randomizeWithLifespan2, zAcceleration,
	unknownFloat1a, unknownFloat1b, unknownFloat1c, unknownFloat1d,
	particleSizes1, rotationValues1, initialColor1, finalColor1,
	unknownColor1, slowdown, unknownFloat2a, unknownFloat2b,
	unknownFloat2c, unknown4, trailingEnabled, forceFlags,
	forceFlagsCopy, unknownFloat3a, unknownFloat3b, unknownFloat3c,
	unknownFloat3d, indexPlusHighestIndex, maxParticles, emissionRate,
	emissionAreaType, emissionAreaSize, tailUnk1, emissionAreaRadius,
	spreadUnk, emissionType, randomizeWithParticleSizes2, particleSizes2, randomizeWithRotationValues2,
	rotationValues2, randomizeWithColor2,	initialColor2, finalColor2, unknownColor2, unknown7, partEmit,
	phase1StartImageIndex, phase1EndImageIndex, phase2EndImageIndex,
	phase2StartImageIndex, relativePhase1Length, numberOfColumns,numberOfRows,
	columnWidth, rowHeight, unknownFloat4, unknownFloat5, unknown9, particleType, lengthWidthRatio, unknown7, flags,
	
	-- stupid massive amount of unused anim ref's
	ar2, ar4, ar5, ar6, ar7, ar8, ar9, ar10, ar11, ar12, ar13, ar14, 
	ar15, ar16, ar17, ar18, ar19, ar20, ar21, ar22, ar23, ar24,
	
	-- instance ref
	parcRef,
	
	-- maxObject
	maxObj,
	
	fn Write bitStream par =
	(
		WriteLong bitStream par.bone #unsigned
		
		WriteLong bitStream par.matmIndex #unsigned
		
		M3E_AnimRef.Write bitStream par.emissionSpeed1 #FLOAT
		M3E_AnimRef.Write bitStream par.emissionSpeed2 #FLOAT
		WriteLong bitStream par.randomizeWithEmissionSpeed2 #unsigned
		
		M3E_AnimRef.Write bitStream par.emissionAngleX #FLOAT
		M3E_AnimRef.Write bitStream par.emissionAngleY #FLOAT
		
		M3E_AnimRef.Write bitStream par.emissionSpreadX #FLOAT
		M3E_AnimRef.Write bitStream par.emissionSpreadY #FLOAT
		
		M3E_AnimRef.Write bitStream par.lifespan1 #FLOAT
		M3E_AnimRef.Write bitStream par.lifespan2 #FLOAT
		WriteLong bitStream par.randomizeWithLifespan2 #unsigned
		
		--M3E_WriteBlank bitStream 12 flags:#blank
		WriteLong bitStream 0 #unsigned
		WriteLong bitStream 0 #unsigned
		WriteLong bitStream 0 #unsigned
		
		WriteFloat bitStream par.zAcceleration
		
		WriteFloat bitStream par.unknownFloat1a
		WriteFloat bitStream par.unknownFloat1b
		WriteFloat bitStream par.unknownFloat1c
		WriteFloat bitStream par.unknownFloat1d
		
		M3E_AnimRef.Write bitStream par.particleSizes1 #VEC3D
		
		M3E_AnimRef.Write bitStream par.rotationValues1 #VEC3D
		
		M3E_AnimRef.Write bitStream par.initialColor1 #COLOUR
		M3E_AnimRef.Write bitStream par.finalColor1 #COLOUR
		M3E_AnimRef.Write bitStream par.unknownColor1 #COLOUR
		
		WriteFloat bitStream par.slowdown
		
		WriteFloat bitStream par.unknownFloat2a
		WriteFloat bitStream par.unknownFloat2b
		WriteFloat bitStream par.unknownFloat2c
		
		WriteFloat bitStream par.unknown4
		
		WriteLong bitStream par.trailingEnabled #unsigned
		
		WriteLong bitStream par.forceFlags #unsigned
		WriteLong bitStream par.forceFlagsCopy #unsigned
		
		WriteFloat bitStream par.unknownFloat3a
		WriteFloat bitStream par.unknownFloat3b
		WriteFloat bitStream par.unknownFloat3c
		WriteFloat bitStream par.unknownFloat3d
		
		WriteLong bitStream (g_pemittersCount)
		
		WriteLong bitStream par.maxParticles #unsigned
		
		M3E_AnimRef.Write bitStream par.emissionRate #FLOAT
		WriteLong bitStream (par.emissionAreaType - 1)
		M3E_AnimRef.Write bitStream par.emissionAreaSize #VEC3D
		
		M3E_AnimRef.Write bitStream par.tailUnk1 #VEC3D
		
		M3E_AnimRef.Write bitStream par.emissionAreaRadius #FLOAT
		M3E_AnimRef.Write bitStream par.spreadUnk #FLOAT
		WriteLong bitStream (par.emissionType - 1) #unsigned
		
		
		
		WriteLong bitStream par.randomizeWithParticleSizes2 #unsigned
		M3E_AnimRef.Write bitStream par.particleSizes2 #VEC3D
		WriteLong bitStream par.randomizeWithRotationValues2 #unsigned
		M3E_AnimRef.Write bitStream par.rotationValues2 #VEC3D
		WriteLong bitStream par.randomizeWithColor2 #unsigned
		
		M3E_AnimRef.Write bitStream par.initialColor2 #COLOUR
		M3E_AnimRef.Write bitStream par.finalColor2 #COLOUR
		M3E_AnimRef.Write bitStream par.unknownColor2 #COLOUR
		
		WriteLong bitStream par.unknown7 #unsigned
		
		M3E_AnimRef.Write bitStream par.partEmit #UINT16
		
		WriteByte bitStream par.phase1StartImageIndex #unsigned
		WriteByte bitStream par.phase1EndImageIndex #unsigned
		WriteByte bitStream par.phase2EndImageIndex #unsigned
		WriteByte bitStream par.phase2StartImageIndex #unsigned
		WriteFloat bitStream par.relativePhase1Length
		
		WriteShort bitStream par.numberOfColumns #unsigned
		WriteShort bitStream par.numberOfRows #unsigned
		WriteFloat bitStream par.columnWidth
		WriteFloat bitStream par.rowHeight
		WriteFloat bitStream par.unknownFloat4
		WriteFloat bitStream par.unknownFloat5
		
		WriteLong bitStream par.unknown9 #unsigned
		M3E_WriteBlank bitStream 20 flags:#blank
		WriteLong bitStream (par.particleType - 1) #unsigned
		
		
		WriteFloat bitStream par.lengthWidthRatio
		
		M3E_WriteBlank bitStream 8 flags:#blank
		
		WriteFloat bitStream 1
		WriteLong bitStream 0 #unsigned
		
		--M3E_WriteBlank bitStream 416 flags:#blank
		M3E_AnimRef.Write bitStream par.ar4 #UINT32
		M3E_AnimRef.Write bitStream par.ar5 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar6 #UINT32
		M3E_AnimRef.Write bitStream par.ar7 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar8 #UINT32
		M3E_AnimRef.Write bitStream par.ar9 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar10 #UINT32
		M3E_AnimRef.Write bitStream par.ar11 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar12 #UINT32
		M3E_AnimRef.Write bitStream par.ar13 #VEC3DSHRT
		
		M3E_AnimRef.Write bitStream par.ar14 #UINT32
		M3E_AnimRef.Write bitStream par.ar15 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar16 #UINT32
		M3E_AnimRef.Write bitStream par.ar17 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar18 #UINT32
		M3E_AnimRef.Write bitStream par.ar19 #VEC3DSHRT
		M3E_AnimRef.Write bitStream par.ar20 #UINT32
		M3E_AnimRef.Write bitStream par.ar21 #UINT32
		M3E_AnimRef.Write bitStream par.ar22 #UINT32
		
		WriteLong bitStream par.flags #unsigned
		
		M3E_WriteBlank bitStream 16 flags:#blank
		
		WriteLong bitStream 0 #unsigned
		WriteLong bitStream 0 #unsigned
		
		M3E_AnimRef.Write bitStream par.ar23 #UINT32
		M3E_AnimRef.Write bitStream par.ar2 #FLOAT
		
		WriteLong bitStream -1 #signed
		WriteFloat bitStream 0
		M3E_AnimRef.Write bitStream par.ar24 #FLOAT
		WriteLong bitStream -1 #signed
		WriteLong bitStream 0 #signed
		
		M3E_WriteBlank bitStream 12 flags:#blank
		
		M3E_WriteRef bitStream par.parcRef #U32_ subtype:#uint32
		),
	
	fn init =
	(
		local par = M3E_PAR()
		
		par.emissionSpeed1 = M3E_AnimRef.init #FLOAT
		par.emissionSpeed2 = M3E_AnimRef.init #FLOAT
		
		par.emissionAngleX = M3E_AnimRef.init #FLOAT
		
		par.emissionAngleY = M3E_AnimRef.init #FLOAT
		
		par.emissionSpreadX = M3E_AnimRef.init #FLOAT
		par.emissionSpreadY = M3E_AnimRef.init #FLOAT
		par.lifespan1 = M3E_AnimRef.init #FLOAT
		par.lifespan2 = M3E_AnimRef.init #FLOAT
		par.particleSizes1 = M3E_AnimRef.init #VEC3D
		par.rotationValues1 = M3E_AnimRef.init #VEC3D
		par.initialColor1  = M3E_AnimRef.init #COLOUR
		par.initialColor1.initValue = par.initialColor1.nullValue = [0,0,0,0]
		par.finalColor1 = M3E_AnimRef.init #COLOUR
		par.finalColor1.initValue = par.finalColor1.nullValue = [0,0,0,0]
		par.unknownColor1 = M3E_AnimRef.init #COLOUR
		par.unknownColor1.initValue = par.unknownColor1.nullValue = [0,0,0,0]
		par.emissionRate = M3E_AnimRef.init #FLOAT
		par.emissionAreaSize = M3E_AnimRef.init #VEC3D
		par.tailUnk1 = M3E_AnimRef.init #VEC3D
		par.emissionAreaRadius = M3E_AnimRef.init #FLOAT
		par.spreadUnk = M3E_AnimRef.init #FLOAT
		par.particleSizes2 = M3E_AnimRef.init #VEC3D
		par.rotationValues2 = M3E_AnimRef.init #VEC3D
		par.initialColor2 = M3E_AnimRef.init #COLOUR
		par.initialColor2.initValue = par.initialColor2.nullValue = [0,0,0,0]
		par.finalColor2 = M3E_AnimRef.init #COLOUR
		par.finalColor2.initValue = par.finalColor2.nullValue = [0,0,0,0]
		par.unknownColor2 = M3E_AnimRef.init #COLOUR
		par.unknownColor2.initValue = par.unknownColor2.nullValue = [0,0,0,0]
		par.partEmit = M3E_AnimRef.init #UINT16
		
		par.ar2 = M3E_AnimRef.init #FLOAT
		par.ar4 = M3E_AnimRef.init #UINT32
		par.ar5 = M3E_AnimRef.init #VEC3DSHRT
		par.ar6 = M3E_AnimRef.init #UINT32
		par.ar7 = M3E_AnimRef.init #VEC3DSHRT
		par.ar8 = M3E_AnimRef.init #UINT32
		par.ar9 = M3E_AnimRef.init #VEC3DSHRT
		par.ar10 = M3E_AnimRef.init #UINT32
		par.ar11 = M3E_AnimRef.init #VEC3DSHRT
		par.ar12 = M3E_AnimRef.init #UINT32
		par.ar13 = M3E_AnimRef.init #VEC3DSHRT
		par.ar14 = M3E_AnimRef.init #UINT32
		par.ar15 = M3E_AnimRef.init #VEC3DSHRT
		par.ar16 = M3E_AnimRef.init #UINT32
		par.ar17 = M3E_AnimRef.init #VEC3DSHRT
		par.ar18 = M3E_AnimRef.init #UINT32
		par.ar19 = M3E_AnimRef.init #VEC3DSHRT
		par.ar20 = M3E_AnimRef.init #UINT32
		par.ar21 = M3E_AnimRef.init #UINT32
		par.ar22 = M3E_AnimRef.init #UINT32
		
		par.ar23 = M3E_AnimRef.init #UINT32
		par.ar24 = M3E_AnimRef.init #FLOAT
		
		par.parcRef = #()

		return par
	)
)

struct M3E_PARC
(
	emissionRate, partEmit, bone,
	origPar,
	maxObj, 
	
	fn Write bitStream parc =
	(
		M3E_AnimRef.Write bitStream parc.emissionRate #FLOAT
		M3E_AnimRef.Write bitStream parc.partEmit #UINT16
		WriteLong bitStream parc.bone #unsigned
	),
	fn init = 
	(
		local parc = M3E_PARC()
		parc.emissionRate = M3E_AnimRef.init #FLOAT
		parc.partEmit = M3E_AnimRef.init #UINT16
		
		return parc
	)
)

struct M3E_Force
(
	forceType, forceStrenght, forceRange, forceUnk1, forceUnk2, bone, maxObj,
	
	fn Write bitStream rforce =
	(
		WriteLong bitStream rforce.forceType #unsigned
		M3E_WriteBlank bitStream 8 flags:#blank
		WriteLong bitStream rforce.bone #unsigned
		M3E_WriteBlank bitStream 8 flags:#blank
		M3E_AnimRef.Write bitStream rforce.forceStrenght #FLOAT
		M3E_AnimRef.Write bitStream rforce.forceRange #FLOAT
		M3E_AnimRef.Write bitStream rforce.forceUnk1 #FLOAT
		M3E_AnimRef.Write bitStream rforce.forceUnk2 #FLOAT
	),
	
	fn init =
	(
		local f = M3E_Force()
		
		f.forceType				= 0
		
		f.forceStrenght		= M3E_AnimRef.init #FLOAT
		f.forceRange			= M3E_AnimRef.init #FLOAT
		f.forceUnk1				= M3E_AnimRef.init #FLOAT
		f.forceUnk2				= M3E_AnimRef.init #FLOAT
		
		return f
	)
)

struct M3E_Ribbon
(
	-- known values
	bone, matmIndex, waveLength, tipOffsetZ, endRadius, Anim1, Anim2, Anim3, Anim4, Anim5, Anim6, BGRAref1, BGRAref2, BGRAref3,
	radiusScale, ribbonTwist, surfaceNoiseAmplitude, surfaceNoiseNumberOfWaves, surfaceNoiseFrequency,
	surfaceNoiseScale, ribbonType, ribbonDivisions, ribbonSides, ribbonLength, ribbonFlags,
	directionVariationBool, directionVariationAmount, directionVariationFrequency,
	amplitudeVariationBool, amplitudeVariationAmount, amplitudeVariationFrequency,
	lengthVariationBool, lengthVariationAmount, lengthVariationFrequency,
	radiusVariationBool, radiusVariationAmount, radiusVariationFrequency,
	srib,
	
	maxObj,
	
	fn Write bitStream rib =
	(
		WriteByte bitStream rib.bone #unsigned
		M3E_WriteBlank bitStream 3 flags:#blank
		WriteLong bitStream rib.matmIndex #unsigned
		M3E_AnimRef.Write bitStream rib.waveLength #FLOAT
		
		M3E_WriteBlank bitStream 24 flags:#blank
		
		M3E_AnimRef.Write bitStream rib.Anim1 #FLOAT
		M3E_AnimRef.Write bitStream rib.Anim2 #FLOAT
		M3E_AnimRef.Write bitStream rib.Anim3 #FLOAT
		M3E_AnimRef.Write bitStream rib.Anim4 #FLOAT
		M3E_AnimRef.Write bitStream rib.Anim5 #FLOAT
		M3E_AnimRef.Write bitStream rib.Anim6 #FLOAT
		
		M3E_WriteBlank bitStream 16 flags:#blank
		
		WriteFloat bitStream rib.tipOffsetZ
		WriteFloat bitStream rib.endRadius
		
		--M3E_WriteBlank bitStream 12 flags:#blank
		
		WriteFloat bitStream 0.996
		WriteFloat bitStream 0.996
		WriteFloat bitStream 0.996
		
		M3E_AnimRef.Write bitStream rib.radiusScale #VEC3D
		M3E_AnimRef.Write bitStream rib.ribbonTwist #FLOAT
		
		M3E_WriteBlank bitStream 16 flags:#blank
		
		M3E_AnimRef.Write bitStream rib.BGRAref1 #COLOUR
		M3E_AnimRef.Write bitStream rib.BGRAref2 #COLOUR
		M3E_AnimRef.Write bitStream rib.BGRAref3 #COLOUR
		
		WriteFloat bitStream 1.0
		WriteFloat bitStream 0.0
		WriteFloat bitStream 1.5
		
		M3E_WriteBlank bitStream 8 flags:#blank
		
		WriteLong bitStream 1
		WriteLong bitStream 0
		WriteLong bitStream 0
		
		WriteFloat bitStream rib.surfaceNoiseAmplitude
		WriteFloat bitStream rib.surfaceNoiseNumberOfWaves
		WriteFloat bitStream rib.surfaceNoiseFrequency
		WriteFloat bitStream rib.surfaceNoiseScale
		
		WriteLong bitStream 3
		
		WriteLong bitStream rib.ribbonType #unsigned
		
		WriteLong bitStream 1
		
		WriteFloat bitStream rib.ribbonDivisions
		WriteLong bitStream rib.ribbonSides #unsigned
		
		M3E_WriteBlank bitStream 4 flags:#blank
		
		M3E_AnimRef.Write bitStream rib.ribbonLength #FLOAT
		
		M3E_WriteBlank bitStream 4 flags:#blank
		
		M3E_WriteRef bitStream rib.srib #SRIB --flags:22
		
		M3E_WriteBlank bitStream 8 flags:#blank
		
		WriteLong bitStream 1
		
		WriteLong bitStream 0
		WriteLong bitStream 0
		
		WriteLong bitStream rib.ribbonFlags #unsigned
		
		M3E_WriteBlank bitStream 8 flags:#blank
		
		WriteLong bitStream 2
		WriteLong bitStream 0
		
		WriteLong bitStream rib.directionVariationBool #unsigned
		M3E_AnimRef.Write bitStream rib.directionVariationAmount #FLOAT
		M3E_AnimRef.Write bitStream rib.directionVariationFrequency #FLOAT
		
		WriteLong bitStream rib.amplitudeVariationBool #unsigned
		M3E_AnimRef.Write bitStream rib.amplitudeVariationAmount #FLOAT
		M3E_AnimRef.Write bitStream rib.amplitudeVariationFrequency #FLOAT
		
		WriteLong bitStream rib.lengthVariationBool #unsigned
		M3E_AnimRef.Write bitStream rib.lengthVariationAmount #FLOAT
		M3E_AnimRef.Write bitStream rib.lengthVariationFrequency #FLOAT
		
		WriteLong bitStream rib.radiusVariationBool #unsigned
		M3E_AnimRef.Write bitStream rib.radiusVariationAmount #FLOAT
		M3E_AnimRef.Write bitStream rib.radiusVariationFrequency #FLOAT
		
		M3E_WriteBlank bitStream 84 flags:#blank
		
	),
	
	fn init =
	(
		local r = M3E_Ribbon()
		
		-- setup animation references
		r.waveLength		= M3E_AnimRef.init #FLOAT
		r.Anim1				= M3E_AnimRef.init #FLOAT
		r.Anim2				= M3E_AnimRef.init #FLOAT
		r.Anim3				= M3E_AnimRef.init #FLOAT
		r.Anim4				= M3E_AnimRef.init #FLOAT
		r.Anim5				= M3E_AnimRef.init #FLOAT
		r.Anim6				= M3E_AnimRef.init #FLOAT
		
		r.radiusScale		= M3E_AnimRef.init #VEC3D
		r.ribbonTwist		= M3E_AnimRef.init #FLOAT
		
		r.BGRAref1			= M3E_AnimRef.init #COLOUR
		r.BGRAref2			= M3E_AnimRef.init #COLOUR
		r.BGRAref3			= M3E_AnimRef.init #COLOUR
		
		r.ribbonLength		= M3E_AnimRef.init #FLOAT
		
		r.directionVariationAmount	= M3E_AnimRef.init #FLOAT
		r.directionVariationFrequency	= M3E_AnimRef.init #FLOAT
		
		r.amplitudeVariationAmount	= M3E_AnimRef.init #FLOAT
		r.amplitudeVariationFrequency	= M3E_AnimRef.init #FLOAT
		
		r.lengthVariationAmount	= M3E_AnimRef.init #FLOAT
		r.lengthVariationFrequency	= M3E_AnimRef.init #FLOAT
		
		r.radiusVariationAmount	= M3E_AnimRef.init #FLOAT
		r.radiusVariationFrequency	= M3E_AnimRef.init #FLOAT
		
		r.srib = #()
		
		
		return r
	)
)

struct M3E_SRIB
(
	sRibFloat1, sRibFloat2, sRibFloat3, sRibFloat4,
	sRibFloat5, sRibFloat6, sRibFloat7, sRibFloat8,
	sRibFloat9, sRibFloat10, 
	sRibInteger1, sRibInteger2, sRibInteger3, sRibInteger4,
	sRibBool1, sRibBool2, sRibBool3,
	sRibAnim1, sRibAnim2, sRibAnim3, sRibAnim4, sRibAnim5,
	sRibAnim6, sRibAnim7, sRibAnim8, sRibAnim9, sRibAnim10,
	bone,
	
	maxObj,
	
	fn Write bitStream srib =
	(
		WriteLong bitStream srib.sRibInteger1 #unsigned
		WriteFloat bitStream srib.sRibFloat1
		WriteFloat bitStream srib.sRibFloat1
		WriteFloat bitStream srib.sRibFloat1
		WriteFloat bitStream srib.sRibFloat1
		WriteFloat bitStream srib.sRibFloat1
		WriteLong bitStream srib.sRibInteger2 #unsigned
		WriteLong bitStream srib.sRibInteger3 #unsigned
		WriteFloat bitStream srib.sRibFloat6
		WriteLong bitStream srib.sRibInteger4 #unsigned
		WriteFloat bitStream srib.sRibFloat7
		WriteFloat bitStream srib.sRibFloat8
		WriteLong bitStream srib.bone #unsigned
		M3E_AnimRef.Write bitStream srib.sRibAnim1 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim2 #FLOAT
		WriteLong bitStream srib.sRibBool1 #unsigned
		M3E_AnimRef.Write bitStream srib.sRibAnim3 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim4 #FLOAT
		WriteLong bitStream srib.sRibBool2 #unsigned
		M3E_AnimRef.Write bitStream srib.sRibAnim5 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim6 #FLOAT
		WriteLong bitStream srib.sRibBool3 #unsigned
		M3E_AnimRef.Write bitStream srib.sRibAnim7 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim8 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim9 #FLOAT
		M3E_AnimRef.Write bitStream srib.sRibAnim10 #FLOAT
		WriteFloat bitStream srib.sRibFloat9
		WriteFloat bitStream srib.sRibFloat10
	),
	fn init =
	(
		local s = M3E_SRIB()
		s.sRibAnim1 = M3E_AnimRef.init #FLOAT
		s.sRibAnim2= M3E_AnimRef.init #FLOAT
		s.sRibAnim3 = M3E_AnimRef.init #FLOAT
		s.sRibAnim4 = M3E_AnimRef.init #FLOAT
		s.sRibAnim5 = M3E_AnimRef.init #FLOAT
		s.sRibAnim6 = M3E_AnimRef.init #FLOAT
		s.sRibAnim7 = M3E_AnimRef.init #FLOAT
		s.sRibAnim8 = M3E_AnimRef.init #FLOAT
		s.sRibAnim9 = M3E_AnimRef.init #FLOAT
		s.sRibAnim10 = M3E_AnimRef.init #FLOAT
		
		return s
	)
)

struct M3E_Light
(
	-- known values
	bone, lightType, lightFlags, lightColor, lightIntensity, lightSpot, lightRange,
	specColor, specIntensity, ulPr3, ulPr3init, ulPr4,
	
	-- maxObject
	maxObj,
	
	fn Write bitStream lght =
	(
		WriteByte bitStream lght.lightType
		WriteByte bitStream 0
		WriteShort bitStream lght.bone
		WriteLong bitStream lght.lightFlags
		WriteLong bitStream 0
		WriteLong bitStream -1
		M3E_AnimRef.Write bitStream lght.lightColor #COLOUR3
		M3E_AnimRef.Write bitStream lght.lightIntensity #FLOAT
		M3E_AnimRef.Write bitStream lght.specColor #COLOUR3
		M3E_AnimRef.Write bitStream lght.specIntensity #FLOAT
		M3E_AnimRef.Write bitStream lght.ulPr3 #FLOAT
		WriteFloat bitStream lght.ulPr3init
		M3E_AnimRef.Write bitStream lght.ulPr4 #FLOAT
		M3E_AnimRef.Write bitStream lght.lightRange #FLOAT
		M3E_AnimRef.Write bitStream lght.lightSpot #FLOAT
	),
	fn init =
	(
		local l = M3E_Light()
		
		l.lightColor = M3E_AnimRef.init #VEC3D
		l.lightIntensity = M3E_AnimRef.init #FLOAT
		l.specColor = M3E_AnimRef.init #VEC3D
		l.specIntensity = M3E_AnimRef.init #FLOAT
		l.ulPr3 = M3E_AnimRef.init #FLOAT
		l.ulPr4 = M3E_AnimRef.init #FLOAT
		l.lightRange = M3E_AnimRef.init #FLOAT
		l.lightSpot = M3E_AnimRef.init #FLOAT
		
		l.lightFlags = 8
		l.lightType = 0
		l.ulPr3init = 0
		
		return l
	)
)

-- ****************
--  EXPORT CODE
-- ****************
---------------------
-- SCENE CODE --
---------------------
fn M3E_getObjects m3objs m3modl objClass =
(
	local mObjs = #()
	local mObjCont
	local mObjClassString
	
	case objClass of
	(
		#attachment: 
		(
			mObjCont = m3objs.attachments
			mObjClassString = "attachments"
		)
		/*#pemitter: 
		(
			mObjCont = m3objs.pemits
			mObjClassString = "particle emitters"
		)*/
		#sc2pemitter2: 
		(
			mObjCont = m3objs.pemits
			mObjClassString = "particle emitters"
		)
		#pinstance:
		(
			mObjCont = m3objs.parcs
			mObjClassString = "particle emitter instances"
		)
		#sc2ribbon:
		(
			mObjCont = m3objs.sc2ribbons
			mObjClassString = "Ribbon"
		)
		#sc2force:
		(
			mObjCont = m3objs.sc2forces
			mObjClassString = "Force"
		)
		#sc2light:
		(
			mObjCont = m3objs.sc2lights
			mObjClassString = "Lights"
		)
		#sc2tight:
		(
			mObjCont = m3objs.sc2tight
			mObjClassString = "Tight test"
		)
		#sc2fuzzy:
		(
			mObjCont = m3objs.sc2fuzzy
			mObjClassString = "Fuzzy Test"
		)
	)
	
	if (mObjCont != undefined) then
	(
		if (mObjCont.count > 0) then
		(
			echo ("Getting "+mObjClassString+"...")
			for i = 1 to mObjCont.count do
			(
				local maxObj = mObjCont[i]
				local m3Obj
				
				-- get custom parameters
				case objClass of
				(
					#attachment:
					(
						m3Obj = M3E_Attachment.init()
						
						m3Obj.name = maxObj.attachName
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
					)
					/*#pemitter:
					(
						-- gather UI parameters
						set time g_baseFrame
						m3Obj = M3E_PAR.init()
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.matmIndex = (findItem m3objs.materials.fullMatList maxObj.pMaterial) - 1
						m3Obj.initEmissSpeed.initValue = maxObj.initEmissSpeed
						m3Obj.speedVar.initValue = maxObj.speedVar
						m3Obj.enableSpeedVar = M3E_BoolToInt maxObj.enableSpeedVar
						m3Obj.angleX.initValue = maxObj.emissAngleX
						m3Obj.angleY.initValue = maxObj.emissAngleY
						m3Obj.spreadX.initValue = maxObj.emissSpreadX
						m3Obj.spreadY.initValue = maxObj.emissSpreadY
						m3Obj.lifespan.initValue = maxObj.lifespan
						m3Obj.decay.initValue = maxObj.decay
						m3Obj.enableDecay = M3E_BoolToInt maxObj.enableDecay
						
						m3Obj.pemitScale.initValue = [maxObj.pemitScaleStart, maxObj.pemitScaleEnd, maxObj.pemitScaleSpread]
						
						m3Obj.colour1[1].initValue = maxObj.colour1start
						m3Obj.colour1[2].initValue = maxObj.colour1mid
						m3Obj.colour1[3].initValue = maxObj.colour1end
						
						m3Obj.minParticles = maxObj.minParticles
						m3Obj.maxParticles = maxObj.maxParticles
						m3Obj.emissRate.initValue = maxObj.rate
						m3Obj.partEmit.initValue = maxObj.partEmit
						m3Obj.type = maxObj.type - 1
						
						m3Obj.emissArea.initValue = [maxObj.pemitAreaWidth, maxObj.pemitAreaLength, maxObj.pemitAreaSpread]
						
						m3Obj.pivotSpread.initValue = maxObj.pivotSpread
						
						m3Obj.enableRotate = M3E_BoolToInt maxObj.enablePemitRotate
						m3Obj.pemitRotate.initValue = [maxObj.pemitRotateSpread1, maxObj.pemitRotateSpread2, maxObj.pemitRotateSpinSpeed]
						
						m3Obj.enableColour2 = M3E_BoolToInt maxObj.enableColour2
						
						m3Obj.colour2[1].initValue = maxObj.colour2start
						m3Obj.colour2[2].initValue = maxObj.colour2mid
						m3Obj.colour2[3].initValue = maxObj.colour2end
						
						m3Obj.columns = maxObj.columns
						m3Obj.rows = maxObj.rows
						
						m3Obj.enableTrailing = M3E_BoolToInt maxObj.enableTrailing
						m3Obj.enableStreak = M3E_BoolToInt maxObj.enableStreak
						m3Obj.enableRadialEmission = M3E_BoolToInt maxObj.enableRadialEmission
						
						-- particle flags
						--{
						local pflags = 0
						pflags = bit.set pflags 1 maxObj.sort
						pflags = bit.set pflags 2 maxObj.collideTerrain
						pflags = bit.set pflags 3 maxObj.collideObjects
						pflags = bit.set pflags 4 maxObj.spawnOnBounce
						pflags = bit.set pflags 5 maxObj.useInnerShape
						pflags = bit.set pflags 6 maxObj.inheritEmissionParams
						pflags = bit.set pflags 7 maxObj.inheritParentVel
						pflags = bit.set pflags 8 maxObj.sortByZHeight
						pflags = bit.set pflags 9 maxObj.reverseIteration
						pflags = bit.set pflags 10 maxObj.smoothRotation
						pflags = bit.set pflags 11 maxObj.bezSmoothRotation
						pflags = bit.set pflags 12 maxObj.smoothSize
						pflags = bit.set pflags 13 maxObj.bezSmoothSize
						pflags = bit.set pflags 14 maxObj.smoothColour
						pflags = bit.set pflags 15 maxObj.bezSmoothColour
						pflags = bit.set pflags 16 maxObj.litParts
						pflags = bit.set pflags 17 maxObj.randFlipBookStart
						pflags = bit.set pflags 18 maxObj.multiplyByGravity
						pflags = bit.set pflags 19 maxObj.clampTailParts
						pflags = bit.set pflags 20 maxObj.spawnTrailingParts
						pflags = bit.set pflags 21 maxObj.fixLengthTailParts
						pflags = bit.set pflags 22 maxObj.useVertexAlpha
						pflags = bit.set pflags 23 maxObj.modelParts
						pflags = bit.set pflags 24 maxObj.swapYZonModelParts
						pflags = bit.set pflags 25 maxObj.scaleTimeByParent
						pflags = bit.set pflags 26 maxObj.useLocalTime
						pflags = bit.set pflags 27 maxObj.simulateOnInit
						pflags = bit.set pflags 28 maxObj.copy
						
						m3Obj.pemitFlags = pflags
						--}
						
						-- kinda unknowns
						m3Obj.emissSpeed2 = maxObj.emissSpeed1
						m3Obj.emissSpeed3 = maxObj.emissSpeed2
						m3Obj.scaleRatio = maxObj.scaleRatio
						
						m3Obj.speedUnk1.initValue = [maxObj.emissSpeed3X, maxObj.emissSpeed3Y, maxObj.emissSpeed3Z]
						
						m3Obj.lifespanRatio = maxObj.lifespanRatio
						
						m3Obj.tailUnk1.initValue = [maxObj.tailUnkX, maxObj.tailUnkY, maxObj.tailUnkZ]
						m3Obj.spreadUnk1.initValue = maxObj.spreadUnk			
						m3Obj.speedUnk2 = [maxObj.speedUnk1X, maxObj.speedUnk1Y, maxObj.speedUnk1Z, maxObj.speedUnk1W]
						m3Obj.enableScale2 = M3E_BoolToInt maxObj.enableScale2
						m3Obj.pemitScale2.initValue = [maxObj.scale2start, maxObj.scale2end, maxObj.scale2spread]
						
						-- COMPLETE UNKNOWNS
						-- Anim Ref's
						m3Obj.ar2.initValue = maxObj.Unk15
		
						-- Single values
						append m3Obj.f1 maxObj.Unk1
						append m3Obj.f1 maxObj.Unk2
						append m3Obj.f1 maxObj.Unk3
						
						append m3Obj.f2 maxObj.Unk4
						append m3Obj.f2 maxObj.Unk5
						
						append m3Obj.f3 maxObj.Unk6
						append m3Obj.f3 maxObj.Unk7
						append m3Obj.f3 maxObj.Unk8
						append m3Obj.f3 maxObj.Unk9
						
						append m3Obj.f4 maxObj.Unk10
						append m3Obj.f4 maxObj.Unk11
						
						m3Obj.f5 = maxObj.Unk12
						m3Obj.f6 = maxObj.Unk13
						m3Obj.f7 = maxObj.Unk14
						
						-- handle p. emitter copies as PARC index references
						for j = 1 to m3objs.parcIDs.count do
						(
							local parcID = m3objs.parcIDs[j]
							if (parcID == i) then
							(
								local parcRef = j - 1 -- make it zero-based
								append m3Obj.parcRef parcRef
							)
						)
					)*/
					#sc2pemitter2:
					(
						set time g_baseFrame
						m3Obj = M3E_PAR.init()
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.matmIndex = (findItem m3objs.materials.fullMatList maxObj.particleMaterial) - 1
						
						m3Obj.emissionRate.initValue = maxObj.emissionRate
						--m3Obj.partEmit.initValue = maxObj.partEmit
						
						m3Obj.emissionSpeed1.initValue 			= maxObj.emissionSpeed1
						m3Obj.emissionSpeed2.initValue			= maxObj.emissionSpeed2
						m3Obj.randomizeWithEmissionSpeed2	= maxObj.randomizeWithEmissionSpeed2
						
						m3Obj.emissionAngleX.initValue			= maxObj.emissionAngleX
						m3Obj.emissionAngleY.initValue			= maxObj.emissionAngleY
						m3Obj.emissionSpreadX.initValue			= maxObj.emissionSpreadX
						m3Obj.emissionSpreadY.initValue			= maxObj.emissionSpreadY
						
						m3Obj.lifespan1.initValue					= maxObj.lifespan1
						m3Obj.lifespan2.initValue					= maxObj.lifespan2
						m3Obj.randomizeWithLifespan2			=  maxObj.randomizeWithLifespan2
						
						m3Obj.zAcceleration							= maxObj.zAcceleration
						
						m3Obj.unknownFloat1a						= maxObj.unknownFloat1a
						m3Obj.unknownFloat1b						= maxObj.unknownFloat1b
						m3Obj.unknownFloat1c						= maxObj.unknownFloat1c
						m3Obj.unknownFloat1d						= maxObj.unknownFloat1d
						
						m3Obj.particleSizes1.initValue			= [maxObj.particleSizesX,maxObj.particleSizesY,maxObj.particleSizesZ]
						m3Obj.rotationValues1.initValue			= [maxObj.rotationValuesX,maxObj.rotationValuesY,maxObj.rotationValuesZ]
						
						local reversedOrder = undefined
						reversedOrder									= [(maxObj.initialColor1.x * 255) as integer,(maxObj.initialColor1.y * 255) as integer,(maxObj.initialColor1.z * 255) as integer,(maxObj.initialColor1.w * 255) as integer]
						m3Obj.initialColor1.initValue				= (reversedOrder)
						reversedOrder									= [(maxObj.finalColor1.x * 255) as integer,(maxObj.finalColor1.y * 255) as integer,(maxObj.finalColor1.z * 255) as integer,(maxObj.finalColor1.w* 255) as integer]
						m3Obj.finalColor1.initValue					= (reversedOrder)
						reversedOrder									= [(maxObj.unknownColor1.x * 255) as integer,(maxObj.unknownColor1.y * 255) as integer,(maxObj.unknownColor1.z * 255) as integer,(maxObj.unknownColor1.w* 255) as integer]
						m3Obj.unknownColor1.initValue			= (reversedOrder)
						
						m3Obj.slowdown								= maxObj.slowdown
						
						m3Obj.unknownFloat2a						= maxObj.unknownFloat2a
						m3Obj.unknownFloat2b						= maxObj.unknownFloat2b
						m3Obj.unknownFloat2c						= maxObj.unknownFloat2c
						
						m3Obj.unknown4								= maxObj.unknown4
						
						m3Obj.trailingEnabled						= maxObj.trailingEnabled
						
						m3Obj.forceFlags								= maxObj.forceFlags
						m3Obj.forceFlagsCopy						= maxObj.forceFlagsCopy
						
						m3Obj.unknownFloat3a						= maxObj.unknownFloat3a
						m3Obj.unknownFloat3b						= maxObj.unknownFloat3b
						m3Obj.unknownFloat3c						= maxObj.unknownFloat3c
						m3Obj.unknownFloat3d						= maxObj.unknownFloat3d
						
						--m3Obj.indexPlusHighestIndex				= g_pemittersCount
						--g_pemittersCount += 1
						--format "g_pemittersCount: %\n" g_pemittersCount
						
						m3Obj.maxParticles							= maxObj.maxParticles
						m3Obj.emissionRate.initValue				= maxObj.emissionRate
						
						m3Obj.emissionAreaType					= maxObj.emissionAreaType
						m3Obj.emissionAreaSize.initValue		= [maxObj.emissionAreaSizeX,maxObj.emissionAreaSizeY,maxObj.emissionAreaSizeZ]
						
						m3Obj.tailUnk1.initValue					= [maxObj.tailUnk1X,maxObj.tailUnk1Y,maxObj.tailUnk1Z]
						
						m3Obj.emissionAreaRadius.initValue		= maxObj.emissionAreaRadius
						
						m3Obj.spreadUnk.initValue					= maxObj.spreadUnk
						
						m3Obj.emissionType							= maxObj.emissionType
						
						m3Obj.randomizeWithParticleSizes2		= maxObj.randomizeWithParticleSizes2
						m3Obj.particleSizes2.initValue			= [maxObj.particleSizes2X,maxObj.particleSizes2Y,maxObj.particleSizes2Z]
						m3Obj.randomizeWithRotationValues2	= maxObj.randomizeWithRotationValues2
						
						m3Obj.rotationValues2.initValue			= [maxObj.rotationValues2X,maxObj.rotationValues2Y,maxObj.rotationValues2Z]
						
						m3Obj.randomizeWithColor2				= maxObj.randomizeWithColor2
						
						reversedOrder									= [(maxObj.initialColor2.x * 255) as integer,(maxObj.initialColor2.y * 255) as integer,(maxObj.initialColor2.z * 255) as integer,(maxObj.initialColor2.w * 255) as integer]
						m3Obj.initialColor2.initValue				= (reversedOrder)
						reversedOrder									= [(maxObj.finalColor2.x * 255) as integer,(maxObj.finalColor2.y * 255) as integer,(maxObj.finalColor2.z * 255) as integer,(maxObj.finalColor2.w * 255) as integer]
						m3Obj.finalColor2.initValue					= (reversedOrder)
						reversedOrder									= [(maxObj.unknownColor2.x * 255) as integer,(maxObj.unknownColor2.y * 255) as integer,(maxObj.unknownColor2.z * 255) as integer,(maxObj.unknownColor2.w * 255) as integer]
						m3Obj.unknownColor2.initValue			= (reversedOrder)
						
						m3Obj.unknown7								= maxObj.unknown7
						
						m3Obj.partEmit.initValue					= maxObj.partEmit
						
						m3Obj.phase1StartImageIndex			= maxObj.phase1StartImageIndex
						m3Obj.phase1EndImageIndex				= maxObj.phase1EndImageIndex
						m3Obj.phase2EndImageIndex				= maxObj.phase2EndImageIndex
						m3Obj.phase2StartImageIndex			= maxObj.phase2StartImageIndex
						m3Obj.relativePhase1Length				= maxObj.relativePhase1Length
						m3Obj.numberOfColumns					= maxObj.numberOfColumns
						m3Obj.numberOfRows						= maxObj.numberOfRows
						m3Obj.columnWidth							= maxObj.columnWidth
						m3Obj.rowHeight								= maxObj.rowHeight
						
						m3Obj.unknownFloat4						= maxObj.unknownFloat4
						m3Obj.unknownFloat5						= maxObj.unknownFloat5
						m3Obj.unknown9								= maxObj.unknown9
						--m3Obj.unknownAt732						= maxObj.unknownAt732
						
						m3Obj.particleType							= maxObj.particleType
						m3Obj.lengthWidthRatio					= maxObj.lengthWidthRatio
						
						--m3Obj.unknown11							= maxObj.unknown11
						
						/*m3Obj.unknownFloat7						= maxObj.unknownFloat7
						m3Obj.unknownAt772						= maxObj.unknownAt772
						m3Obj.unknownAt776						= maxObj.unknownAt776
						m3Obj.unknownAt784						= maxObj.unknownAt784
						m3Obj.unknownAt796						= maxObj.unknownAt796
						m3Obj.unknownAt804						= maxObj.unknownAt804
						m3Obj.unknownAt820						= maxObj.unknownAt820
						m3Obj.unknownAt828						= maxObj.unknownAt828
						m3Obj.unknownAt840						= maxObj.unknownAt840
						m3Obj.unknownAt848						= maxObj.unknownAt848
						m3Obj.unknownAt864						= maxObj.unknownAt864
						m3Obj.unknownAt872						= maxObj.unknownAt872
						m3Obj.unknownAt884						= maxObj.unknownAt884
						m3Obj.unknownAt892						= maxObj.unknownAt892
						m3Obj.unknownAt908						= maxObj.unknownAt908
						m3Obj.unknownAt916							= maxObj.unknownAt916
						m3Obj.unknownAt928						= maxObj.unknownAt928
						m3Obj.unknownAt936						= maxObj.unknownAt936
						m3Obj.unknownAt952						= maxObj.unknownAt952
						m3Obj.unknownAt960						= maxObj.unknownAt960
						m3Obj.unknownAt972						= maxObj.unknownAt972
						m3Obj.unknownAt980						= maxObj.unknownAt980
						m3Obj.unknownAt996						= maxObj.unknownAt996
						m3Obj.unknownAt1004						= maxObj.unknownAt1004
						m3Obj.unknownAt1016						= maxObj.unknownAt1016
						m3Obj.unknownAt1024						= maxObj.unknownAt1024
						m3Obj.unknownAt1040						= maxObj.unknownAt1040
						m3Obj.unknownAt1048						= maxObj.unknownAt1048
						m3Obj.unknownAt1060						= maxObj.unknownAt1060
						m3Obj.unknownAt1068						= maxObj.unknownAt1068
						m3Obj.unknownAt1084						= maxObj.unknownAt1084
						m3Obj.unknownAt1092						= maxObj.unknownAt1092
						m3Obj.unknownAt1104						= maxObj.unknownAt1104
						m3Obj.unknownAt1112						= maxObj.unknownAt1112
						m3Obj.unknownAt1128						= maxObj.unknownAt1128
						m3Obj.unknownAt1136						= maxObj.unknownAt1136
						m3Obj.unknownAt1148						= maxObj.unknownAt1148
						m3Obj.unknownAt1156						= maxObj.unknownAt1156
						m3Obj.unknownAt1168						= maxObj.unknownAt1168
						m3Obj.unknownAt1188						= maxObj.unknownAt1188*/
						
						local pflags = 0
						pflags = bit.set pflags 1 maxObj.pFsort
						pflags = bit.set pflags 2 maxObj.pFcollideTerrain
						pflags = bit.set pflags 3 maxObj.pFcollideObjects
						pflags = bit.set pflags 4 maxObj.pFspawnOnBounce
						pflags = bit.set pflags 5 maxObj.pFuseInnerShape
						pflags = bit.set pflags 6 maxObj.pFinheritEmissionParams
						pflags = bit.set pflags 7 maxObj.pFinheritParentVel
						pflags = bit.set pflags 8 maxObj.pFsortByZHeight
						pflags = bit.set pflags 9 maxObj.pFreverseIteration
						pflags = bit.set pflags 10 maxObj.pFsmoothRotation
						pflags = bit.set pflags 11 maxObj.pFbezSmoothRotation
						pflags = bit.set pflags 12 maxObj.pFsmoothSize
						pflags = bit.set pflags 13 maxObj.pFbezSmoothSize
						pflags = bit.set pflags 14 maxObj.pFsmoothColor
						pflags = bit.set pflags 15 maxObj.pFbezSmoothColor
						pflags = bit.set pflags 16 maxObj.pFlitParts
						pflags = bit.set pflags 17 maxObj.pFrandFlipBookStart
						pflags = bit.set pflags 18 maxObj.pFmultiplyByGravity
						pflags = bit.set pflags 19 maxObj.pFclampTailParts
						pflags = bit.set pflags 20 maxObj.pFspawnTrailingParts
						pflags = bit.set pflags 21 maxObj.pFfixLengthTailParts
						pflags = bit.set pflags 22 maxObj.pFuseVertexAlpha
						pflags = bit.set pflags 23 maxObj.pFmodelParts
						pflags = bit.set pflags 24 maxObj.pFswapYZonModelParts
						pflags = bit.set pflags 25 maxObj.pFscaleTimeByParent
						pflags = bit.set pflags 26 maxObj.pFuseLocalTime
						pflags = bit.set pflags 27 maxObj.pFsimulateOnInit
						pflags = bit.set pflags 28 maxObj.pFcopy
						
						m3Obj.flags = pflags
						
						/*m3Obj.unknownAt1192						= maxObj.unknownAt1192
						m3Obj.unknownAt1202						= maxObj.unknownAt1202
						m3Obj.unknownAt1212						= maxObj.unknownAt1212
						m3Obj.unknownAt1216						= maxObj.unknownAt1216
						m3Obj.unknownAt1224						= maxObj.unknownAt1224
						m3Obj.unknownAt1236						= maxObj.unknownAt1236
						m3Obj.unknownAt1256						= maxObj.unknownAt1256
						m3Obj.unknownAt1260						= maxObj.unknownAt1260
						m3Obj.unknownAt1264						= maxObj.unknownAt1264
						m3Obj.unknownAt1284						= maxObj.unknownAt1284
						m3Obj.unknownAt1288						= maxObj.unknownAt1288*/
						
						
						--m3Obj.usedModel								= maxObj.usedModel
						--m3Obj.copyIndices							= maxObj.copyIndices
						
					)
					#pinstance:
					(
						m3Obj = M3E_PARC.init()
						m3Obj.emissionRate.initValue = maxObj.emissionRate
						m3Obj.partEmit.initValue = maxObj.partEmit
						
						--m3Obj = M3E_PARC.init()
					)
					#sc2ribbon:
					(
						-- gather UI parameters
						set time g_baseFrame
						m3Obj = M3E_Ribbon.init()
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.matmIndex = (findItem m3objs.materials.fullMatList maxObj.ribbonMaterial) - 1
						m3Obj.waveLength.initValue = maxObj.waveLength
						
						m3Obj.tipOffsetZ		  	= maxObj.tipOffsetZ
						m3Obj.endRadius			= maxObj.endRadius
						
						m3Obj.surfaceNoiseAmplitude				= maxObj.surfaceNoiseAmplitude
						m3Obj.surfaceNoiseNumberOfWaves	= maxObj.surfaceNoiseNumberOfWaves
						m3Obj.surfaceNoiseFrequency			= maxObj.surfaceNoiseFrequency
						m3Obj.surfaceNoiseScale					= maxObj.surfaceNoiseScale
						
						m3Obj.ribbonType							= maxObj.ribbonType - 1
						m3Obj.ribbonDivisions						= maxObj.ribbonDivisions
						m3Obj.ribbonSides							= maxObj.ribbonSides
						
						m3Obj.Anim1.initValue 	= maxObj.Anim1
						m3Obj.Anim2.initValue 	= maxObj.Anim2
						m3Obj.Anim3.initValue 	= maxObj.Anim3
						m3Obj.Anim4.initValue 	= maxObj.Anim4
						m3Obj.Anim5.initValue 	= maxObj.Anim5
						m3Obj.Anim6.initValue 	= maxObj.Anim6
						
						--m3Obj.radiusScale[1].initValue = maxObj.startRadiusScale
						--m3Obj.radiusScale[2].initValue = maxObj.centerRadiusScale
						--m3Obj.radiusScale[3].initValue = maxObj.endRadiusScale
						m3Obj.radiusScale.initValue = [maxObj.startRadiusScale, maxObj.centerRadiusScale, maxObj.endRadiusScale]
						
						m3Obj.BGRAref1.initValue = (maxObj.BGRAref1 * 255) --[255,255,255,255]
						m3Obj.BGRAref2.initValue = (maxObj.BGRAref2 * 255) --[255,255,255,255]
						m3Obj.BGRAref3.initValue = (maxObj.BGRAref3 * 255)---[255,255,255,255]
						
						m3Obj.ribbonLength.initValue				= maxObj.ribbonLength
						
						local rflags = 0
						rflags = bit.set rflags 1 maxObj.rF1
						rflags = bit.set rflags 2 maxObj.rF2
						rflags = bit.set rflags 3 maxObj.rF3
						rflags = bit.set rflags 4 maxObj.rF4
						rflags = bit.set rflags 5 maxObj.rF5
						rflags = bit.set rflags 6 maxObj.rF6
						rflags = bit.set rflags 7 maxObj.rF7
						rflags = bit.set rflags 8 maxObj.rF8
						rflags = bit.set rflags 9 maxObj.rF9
						rflags = bit.set rflags 10 maxObj.rF10
						rflags = bit.set rflags 11 maxObj.rF11
						rflags = bit.set rflags 12 maxObj.rF12
						rflags = bit.set rflags 14 maxObj.rF14
						
						m3Obj.ribbonFlags = rflags
						
						m3Obj.directionVariationBool							= M3E_BoolToInt maxObj.directionVariationBool
						m3Obj.directionVariationAmount.initValue			= maxObj.directionVariationAmount
						m3Obj.directionVariationFrequency.initValue		= maxObj.directionVariationFrequency
						
						m3Obj.amplitudeVariationBool							= M3E_BoolToInt maxObj.amplitudeVariationBool
						m3Obj.amplitudeVariationAmount.initValue		= maxObj.amplitudeVariationAmount
						m3Obj.amplitudeVariationFrequency.initValue		= maxObj.amplitudeVariationFrequency
						
						m3Obj.lengthVariationBool								= M3E_BoolToInt maxObj.lengthVariationBool
						m3Obj.lengthVariationAmount.initValue				= maxObj.lengthVariationAmount
						m3Obj.lengthVariationFrequency.initValue			= maxObj.lengthVariationFrequency
						
						m3Obj.radiusVariationBool								= M3E_BoolToInt maxObj.radiusVariationBool
						m3Obj.radiusVariationAmount.initValue				= maxObj.radiusVariationAmount
						m3Obj.radiusVariationFrequency.initValue			= maxObj.radiusVariationFrequency
						
						/*if (maxObj.enblSRib == true) then
						(
							m3Obj.srib = M3E_SRIB.init()
							
							local sbname 		= maxObj.sribbone.name
							m3Obj.srib.bone	= (M3E_findParentBone m3modl.bones sbname) - 1
							
							m3Obj.srib.sribBool1				= maxObj.sribBool1
							m3Obj.srib.sribBool2				= maxObj.sribBool2
							m3Obj.srib.sribBool3				= maxObj.sribBool3	
							
							m3Obj.srib.sribFloat1			= maxObj.sribFloat1
							m3Obj.srib.sribFloat2			= maxObj.sribFloat2
							m3Obj.srib.sribFloat3			= maxObj.sribFloat3
							m3Obj.srib.sribFloat4			= maxObj.sribFloat4
							m3Obj.srib.sribFloat5			= maxObj.sribFloat5
							m3Obj.srib.sribFloat6			= maxObj.sribFloat6
							m3Obj.srib.sribFloat7			= maxObj.sribFloat7
							m3Obj.srib.sribFloat8			= maxObj.sribFloat8
							m3Obj.srib.sribFloat9			= maxObj.sribFloat9
							m3Obj.srib.sribFloat10			= maxObj.sribFloat10
							
							m3Obj.srib.sribInteger1			= maxObj.sribInteger1
							m3Obj.srib.sribInteger2			= maxObj.sribInteger2
							m3Obj.srib.sribInteger3			= maxObj.sribInteger3
							m3Obj.srib.sribInteger4			= maxObj.sribInteger4
							
							m3Obj.srib.sribAnim1.initValue		= maxObj.sribAnim1
							m3Obj.srib.sribAnim2.initValue		= maxObj.sribAnim2
							m3Obj.srib.sribAnim3.initValue		= maxObj.sribAnim3
							m3Obj.srib.sribAnim4.initValue		= maxObj.sribAnim4
							m3Obj.srib.sribAnim5.initValue		= maxObj.sribAnim5
							m3Obj.srib.sribAnim6.initValue		= maxObj.sribAnim6
							m3Obj.srib.sribAnim7.initValue		= maxObj.sribAnim7
							m3Obj.srib.sribAnim8.initValue		= maxObj.sribAnim8
							m3Obj.srib.sribAnim9.initValue		= maxObj.sribAnim9
							m3Obj.srib.sribAnim10.initValue	= maxObj.sribAnim10
						)*/
					)
					#sc2force:
					(
						-- gather UI parameters
						set time g_baseFrame
						m3Obj = M3E_Force.init()
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.forceType						= maxObj.forceType - 1
						
						m3Obj.forceStrenght.initValue 	= maxObj.forceStrenght
						m3Obj.forceRange.initValue 		= maxObj.forceRange
						m3Obj.forceUnk1.initValue 		= maxObj.forceUnk1
						m3Obj.forceUnk2.initValue 		= maxObj.forceUnk2
					)
					#sc2light:
					(
						set time g_baseFrame
						m3Obj = M3E_Light.init()
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.lightType = maxObj.lightType
						
						local lflags = 0
						lflags = bit.set lflags 1 maxObj.lightFlag1
						lflags = bit.set lflags 2 maxObj.lightFlag2
						lflags = bit.set lflags 3 maxObj.lightFlag3
						lflags = bit.set lflags 4 maxObj.lightFlag4
						
						m3Obj.lightFlags = lflags
						m3Obj.lightColor.initValue = maxObj.lightColor
						m3Obj.lightIntensity.initValue = maxObj.lightIntensity
						m3Obj.specColor.initValue 		= maxObj.specColor
						m3Obj.specIntensity.initValue 	= maxObj.specIntensity
						m3Obj.ulPr3.initValue 		= maxObj.ulPr3
						m3Obj.ulPr3init				= maxObj.ulPr3
						m3Obj.ulPr4.initValue 		= maxObj.ulPr4
						m3Obj.lightRange.initValue	= degtorad maxObj.lightRange
						m3Obj.lightSpot.initValue 	= degtorad maxObj.lightSpot
						
					)
					#sc2tight:
					(
						m3Obj = M3E_Tight.init()
						
						
						local preMatrix 	= maxObj.transform * inverse maxObj.parent.transform
						m3Obj.matrix		= M3E_Matrix.convObjMatrix preMatrix
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.tightType = maxObj.tightType
						m3Obj.tightSize0	= maxObj.tightSize0
						m3Obj.tightSize1	= maxObj.tightSize1
						m3Obj.tightSize2	= maxObj.tightSize2
					)
					
					#sc2fuzzy:
					(
						m3Obj = M3E_SSGS.init()
						
						local preMatrix = maxObj.transform * inverse maxObj.parent.transform
						m3Obj.matrix		= M3E_Matrix.convObjMatrix preMatrix
						
						local bname 		= maxObj.parent.name
						m3Obj.bone		= (M3E_findParentBone m3modl.bones bname) - 1
						
						m3Obj.fuzzyType = maxObj.fuzzyType
						m3Obj.fuzzySize0	= maxObj.fuzzySize0
						m3Obj.fuzzySize1	= maxObj.fuzzySize1
						m3Obj.fuzzySize2	= maxObj.fuzzySize2
					)
				)
				
				-- max object reference
				m3Obj.maxObj = maxObj
				
				for j = 1 to m3modl.bones.count do
				(
					b = m3modl.bones[j]
					if (maxObj == b.maxObj) then m3Obj.bone = j - 1
				)
				
				if (m3Obj.bone != undefined) then 
				(
					append mObjs m3Obj
				)
			) -- for i loop
		) -- if mObjCont.count > 0
		
		-- sort by bone parents
		fn comparePar a1 a2 = if (a1.bone > a2.bone) then return 1 else if (a1.bone < a2.bone) then return -1 else return 0
		qsort mObjs comparePar
	)
	
	return mObjs
)

fn M3E_GetBndSphere =
(
	local retSph
	
	if (m3scene.bndSphere != undefined) then
	(
		local bSph = m3scene.bndSphere
		local m3bSph = M3E_Sphere.init()
		
		m3bSph.maxObj = bSph
		m3bSph.emin = bSph.min
		m3bSph.emax = bSph.max
		
		-- check the scale transform...
		local bsScale = bSph.transform.scale
		local lrgScale = 0.0
		for i = 1 to 3 do 
		(
			if (lrgScale < bsScale[i]) then lrgScale = bsScale[i]
		)
		
		local rad = bSph.bndRadius * lrgScale
		
		m3bSph.rad = rad
		
		retSph = m3bSph
	)
	else
	(
		if (m3scene.meshes.count > 0) then
		(
			retSph = M3E_Sphere.GetBound m3scene.meshes
		)
		else 
		(
			retSph = M3E_Sphere.init()
			retSph.rad = 2 -- seems to be default radius in Blizz models
		)
	)
	
	return retSph
)

struct M3E_animFrameKey
(
	stcInd, frame, key
)

fn M3E_findSTCframe frame =
(
	for i = 1 to awrite.count do
	(
		local seq = awrite[i]
		
		if (frame >= seq.cstart and frame <= seq.cend) then
		(
			return i
		)
		
		if (i == awrite.count) do return undefined
	)
)

fn M3E_addArefToSTC aref akeys type = 
(
	-- handle collected keys
	for i = 1 to akeys.count do
	(
		local akey = akeys[i]
		local seq = awrite[akey.stcInd]
		local idFound = findItem seq.animid aref.animid
		
		if (idFound > 0) then
		(
			local animOffs = seq.animref[idFound]
			local stcData = seq.sdata[animOffs.sdind + 1][animOffs.aind + 1] -- arrays not 0-based
			
			local localFrame = akey.frame - seq.cstart
			-- convert to M3 fps
			localFrame = localFrame * (1000 / g_useFPS)
			
			append stcData.frames localFrame
			append stcData.keys akey.key
		)
		else
		(
			-- code for initializing new animblock when I'm not feeling lazy
			append seq.animid aref.animid
			local aoff = M3E_Animoffs.init()
			
			case type of
			(
				#EVENT: aoff.sdind = 0
				#VEC2D: aoff.sdind = 1
				#VEC3D: aoff.sdind = 2
				#QUAT: aoff.sdind = 3
				#COLOUR: aoff.sdind = 4
				#FLOAT: aoff.sdind = 5
				#UINT16: aoff.sdind = 7
			)
			
			local aindcount = seq.sdata[aoff.sdind + 1].count
			aoff.aind = aindcount
			
			append seq.animref aoff

			-- initialise animblock
			local ab = M3E_AnimBlock.init()
			ab.fend = seq.cend - seq.cstart
			
			local localFrame = akey.frame - seq.cstart
			-- convert to M3 fps
			localFrame = localFrame * (1000 / g_useFPS)
			
			append ab.frames localFrame
			append ab.keys akey.key
			
			append seq.sdata[aoff.sdind + 1] ab
		)
		
		-- update original array
		--format "seq: %\n" seq
		awrite[akey.stcInd] = seq
	)
)

fn M3E_buildArefData aref rootObj props type =
(
	-- collect keys
	local adata = #()
	local propsClass = classOf props as string
	local aprops = #()
	if (propsClass != "Array") then
	(
		-- turn into array if not passed as array
		append aprops props
	) else aprops = props
	
	local frames = #()
	local prevq
	
	if (type == #QUAT) then
	(
		set time g_baseFrame 
		prevq = rootObj.rotation
	)

	for i = 1 to aprops.count do
	(
		local prop = aprops[i]
		local acont
		-- need special cases for translation/rotation/scaling controllers
		case prop of
		(
			#position: acont = rootObj.position.controller
			#rotation: acont = rootObj.rotation.controller
			#scale: acont = rootObj.scale.controller
			default: acont = getPropertyController rootObj prop
		)

		if (acont != undefined) then
		(
			for j = 1 to acont.keys.count do
			(
				local key = acont.keys[j]
				local frame = (key.time/TicksPerFrame) as integer
				
				-- check for found frame
				if (findItem frames frame == 0) then
				(
					local akey = M3E_animFrameKey()
					
					akey.frame = frame
					akey.stcInd = M3E_findSTCframe akey.frame
					
					if (akey.stcInd != undefined) then
					(
						-- append frame to list to check for later
						append frames frame
					
						-- go to key time
						set time key.time
						
						case type of
						(
							#VEC2D:
							(
								local v2d = [0.0, 0.0]
								-- get values of each as point2
								v2d.x = getProperty rootObj props[1]
								v2d.y = getProperty rootObj props[2]
								akey.key = v2d
							)
							#VEC3D:
							(
								case prop of
								(
									#position:
									(
										akey.key = acont.value
									)
									#scale:
									(
										akey.key = acont.value
									)
									default:
									(
										local v3d = [0.0, 0.0, 0.0]
										-- get values of each as point3
										v3d.x = getProperty rootObj props[1]
										v3d.y = getProperty rootObj props[2]
										v3d.z = getProperty rootObj props[3]
										akey.key = v3d
									)
								)
							)
							#VEC4D:
							(
								local tempv4d
								local v4d = [0.0, 0.0, 0.0, 0.0]
								-- get values of each as point4
								tempv4d = getProperty rootObj props
								v4d.x = tempv4d[1]
								v4d.y = tempv4d[2]
								v4d.z = tempv4d[3]
								v4d.w = tempv4d[4]
								akey.key = v4d
							)
							#QUAT: 
							(
								--q = normalize (M3E_getNodeParentRotation mbone.maxObj)
								local mat
								if (rootObj.parent != undefined) then 
								(
									mat = rootObj.transform * (inverse rootObj.parent.transform)
								)
								else
								(
									mat = rootObj.transform
								)
								q = normalize (inverse mat.rotationpart)
								-- if dot product less than one, reverse signs
								if (qdot q prevq < 0) then q = -q
								akey.key = q
								prevq = q
							)
							#COLOUR:
							(
								local tempRcolour
								local rcolour = [0, 0, 0, 0]
								tempRcolour = getProperty rootObj props
								format "TEMP COLOUR: %\n" temRcolour
								rcolour.x = (tempRcolour[3] * 255)
								rcolour.y = (tempRcolour[2] * 255)
								rcolour.z = (tempRcolour[1] * 255)
								rcolour.w = (tempRcolour[4] * 255)
								akey.key = rcolour
								format "AKEY: %\n" rcolour
							)
							#DEGTORAD:
							(
								akey.key = degtorad (getProperty rootObj prop)
							)
							default: akey.key = getProperty rootObj prop
						)
					
						append adata akey
						
						/*case prop of
						(
							#position: aref.flags = 1
							#rotation: aref.flags = 1
							#scale: aref.flags = 1
							default:
							(
								if (acont.keys[1].outTangentType == #step or acont.keys[1].inTangentType == #step) then
								(
									aref.flags = 0
								)
								else
								(
									aref.flags = 1
								)
							)
						)*/
						if (aref.animflags != 6) then
						(
							aref.animflags = 6
							case type of
							(
								#UINT16:
								(
									aref.flags = 0
								)
								default:
								(
									aref.flags = 1
								)
							)
						)
						
						
						
					)
				)
			) -- for j loop
			
			/*if(acont.keys[1] != undefined) then
			(
				if (acont.keys[1].outTangentType == #step or acont.keys[1].inTangentType == #step) then
				(
					aref.flags = 1
				)
				else
				(
					aref.flags = 0
				)
			)*/
		) -- if != undefined
	) -- for i loop
	
	--format "aref: %\n" aref
	--format "adata: %\n" adata
	--format "type: %\n" type
	
	M3E_addArefToSTC aref adata type
	
)

struct M3E_Cam
(
	d1, name, fov, clip_near, clip_far, focal_depth,
	clip_2, falloff_start, falloff_end, dof,
	
	fn Write bitStream cam =
	(
		WriteLong bitStream cam.d1 #signed
		M3E_WriteRef bitStream cam.name #CHAR

		M3E_AnimRef.Write bitStream cam.fov #FLOAT

		WriteLong bitStream 1 #unsigned;
		
		M3E_AnimRef.Write bitStream cam.clip_far #FLOAT
		M3E_AnimRef.Write bitStream cam.clip_near #FLOAT
		M3E_AnimRef.Write bitStream cam.clip_2 #FLOAT
		M3E_AnimRef.Write bitStream cam.focal_depth #FLOAT
		M3E_AnimRef.Write bitStream cam.falloff_start #FLOAT
		M3E_AnimRef.Write bitStream cam.falloff_end #FLOAT
		M3E_AnimRef.Write bitStream cam.dof #FLOAT
	),
	
	fn Init cam =
	(
		c = M3E_Cam();
		c.name = cam.name;
		
		c.fov = M3E_AnimRef.init #FLOAT;
		c.fov.initValue = degtorad cam.curFOV;
		M3E_buildArefData c.fov cam #undef #FLOAT
		
		c.clip_far =  M3E_AnimRef.init #FLOAT;
		c.clip_far.initValue = cam.farClip;
		c.clip_far.nullValue = 10.0;
		M3E_buildArefData c.clip_far cam #undef #FLOAT

		c.clip_near =  M3E_AnimRef.init #FLOAT;
		c.clip_near.initValue = cam.nearClip;
		c.clip_near.nullValue = 0.1;
		M3E_buildArefData c.clip_near cam #undef #FLOAT

		c.clip_2 =  M3E_AnimRef.init #FLOAT;
		c.clip_2.initValue = 10.0;
		M3E_buildArefData c.clip_2 cam #undef #FLOAT

		c.focal_depth =  M3E_AnimRef.init #FLOAT;
		if (cam.targetDistance == undefined)	then 
		(
			c.focal_depth.initValue = 9.9
			M3E_buildArefData c.focal_depth cam #undef #FLOAT
		) else (
			c.focal_depth.initValue = cam.targetDistance;
			M3E_buildArefData c.focal_depth cam #undef #FLOAT
		)

		c.falloff_start =  M3E_AnimRef.init #FLOAT;
		c.falloff_start.initValue = cam.nearRange;
		M3E_buildArefData c.falloff_start cam #undef #FLOAT

		c.falloff_end =  M3E_AnimRef.init #FLOAT;
		c.falloff_end.initValue = cam.farRange;
		M3E_buildArefData c.falloff_end cam #undef #FLOAT
			
		c.dof =  M3E_AnimRef.init #FLOAT;
		c.dof.initValue = 1.0;
		M3E_buildArefData c.dof cam #undef #FLOAT

		c
	)
)

fn M3E_buildPemitAnims m3parts =
(
	echo "Building particle emitter animation data..."
	
	for i = 1 to m3parts.count do
	(
		local par = m3parts[i]
		local maxObj = par.maxObj
		
		M3E_buildArefData par.emissionSpeed1 maxObj #emissionSpeed1 #FLOAT
		M3E_buildArefData par.emissionSpeed2 maxObj #emissionSpeed2 #FLOAT
		M3E_buildArefData par.emissionAngleX maxObj #emissionAngleX #FLOAT
		M3E_buildArefData par.emissionAngleY maxObj #emissionAngleY #FLOAT
		M3E_buildArefData par.emissionSpreadX maxObj #emissionSpreadX #FLOAT
		M3E_buildArefData par.emissionSpreadY maxObj #emissionSpreadY #FLOAT
		M3E_buildArefData par.lifespan1 maxObj #lifespan1 #FLOAT
		M3E_buildArefData par.lifespan2 maxObj #lifespan2 #FLOAT
		
		local particleSizes1Vec3d = #(#particleSizesX,#particleSizesY,#particleSizesZ)
		M3E_buildArefData par.particleSizes1 maxObj particleSizes1Vec3d #VEC3D
		
		local rotationValuesVec3d = #(#rotationValuesX, #rotationValuesY, #rotationValuesZ)
		M3E_buildArefData par.rotationValues1 maxObj rotationValuesVec3d #VEC3D
		
		M3E_buildArefData par.initialColor1 maxObj #initialColor1 #COLOUR
		M3E_buildArefData par.finalColor1 maxObj #finalColor1 #COLOUR
		M3E_buildArefData par.unknownColor1 maxObj #unknownColor1 #COLOUR
		
		M3E_buildArefData par.emissionRate maxObj #emissionRate #FLOAT
		
		local emissionAreaSizeVec3d = #(#emissionAreaSizeX,#emissionAreaSizeY,#emissionAreaSizeZ)
		M3E_buildArefData par.emissionAreaSize maxObj emissionAreaSizeVec3D #VEC3D
		
		local tailUnk1Vec3d = #(#tailUnk1X, #tailUnk1Y, #tailUnk1Z)
		M3E_buildArefData par.tailUnk1 maxObj #tailUnk1 #VEC3D
		
		M3E_buildArefData par.emissionAreaRadius maxObj #emissionAreaRadius #FLOAT
		M3E_buildArefData par.spreadUnk maxObj #spreadUnk #FLOAT
		
		local particleSizes2Vec3d = #(#particleSizes2X,#particleSizes2Y,#particleSizes2Z)
		M3E_buildArefData par.particleSizes2 maxObj particleSizes2Vec3D #VEC3D
		
		local rotationValues2Vec3d = #(#rotationValues2X,#rotationValues2Y,#rotationValues2Z)
		M3E_buildArefData par.rotationValues2 maxObj rotationValues2Vec3d #VEC3D
		
		M3E_buildArefData par.initialColor2 maxObj #initialColor2 #COLOUR
		M3E_buildArefData par.finalColor2 maxObj #finalColor2 #COLOUR
		M3E_buildArefData par.unknownColor2 maxObj #unknownColor2 #COLOUR
		M3E_buildArefData par.partEmit maxObj #partEmit #UINT16
		
		/*M3E_buildArefData par.initEmissSpeed maxObj #initEmissSpeed #FLOAT
		M3E_buildArefData par.speedVar maxObj #speedVar #FLOAT
		M3E_buildArefData par.angleX maxObj #emissAngleX #FLOAT
		M3E_buildArefData par.angleY maxObj #emissAngleY #FLOAT
		M3E_buildArefData par.spreadX maxObj #emissSpreadX #FLOAT
		M3E_buildArefData par.spreadY maxObj #emissSpreadY #FLOAT
		M3E_buildArefData par.lifespan maxObj #lifespan #FLOAT
		M3E_buildArefData par.decay maxObj #decay #FLOAT
		
		-- messy but no better way to do it due to the way max handles values and controllers
		local pemitScaleVec3D = #(#pemitScaleStart, #pemitScaleEnd, #pemitScaleSpread)
		M3E_buildArefData par.pemitScale maxObj pemitScaleVec3D #VEC3D
		
		M3E_buildArefData par.emissRate maxObj #rate #FLOAT
		M3E_buildArefData par.partEmit maxObj #partEmit #UINT16
		
		local emissAreaVec3D = #(#pemitAreaWidth, #pemitAreaLength, #pemitAreaSpread)
		M3E_buildArefData par.emissArea maxObj emissAreaVec3D #VEC3D
		
		M3E_buildArefData par.pivotSpread maxObj #pivotSpread #FLOAT
		
		local pemitRotateVec3D = #(#pemitRotateSpread1, #pemitRotateSpread2, #pemitRotateSpinSpeed)
		M3E_buildArefData par.pemitRotate maxObj pemitRotateVec3D #VEC3D
		
		-- colours
		M3E_buildArefData par.colour1[1] maxObj #colour1start #VEC4D
		M3E_buildArefData par.colour1[2] maxObj #colour1mid #VEC4D
		M3E_buildArefData par.colour1[3] maxObj #colour1end #VEC4D
		
		M3E_buildArefData par.colour2[1] maxObj #colour2start #VEC4D
		M3E_buildArefData par.colour2[2] maxObj #colour2mid #VEC4D
		M3E_buildArefData par.colour2[3] maxObj #colour2end #VEC4D
		
		local speedUnk1Vec3D = #(#emissSpeed3X, #emissSpeed3Y, #emissSpeed3Z)
		M3E_buildArefData par.speedUnk1 maxObj speedUnk1Vec3D #VEC3D
		
		M3E_buildArefData par.spreadUnk1 maxObj #spreadUnk #FLOAT
		
		local scale2Vec3D = #(#scale2start, #scale2end, #scale2spread)
		M3E_buildArefData par.pemitScale2 maxObj scale2Vec3D #VEC3D
		
		M3E_buildArefData par.ar2 maxObj #Unk15 #FLOAT*/
	)
)

fn M3E_buildRibbonAnims m3sc2ribbons =
(
	echo "Building ribbon animation data..."
	
	for i = 1 to m3sc2ribbons.count do
	(
		local rib = m3sc2ribbons[i]
		local maxObj = rib.maxObj
		
		M3E_buildArefData rib.waveLength maxObj #waveLength #FLOAT
		
		M3E_buildArefData rib.Anim1 maxObj #Anim1 #FLOAT
		M3E_buildArefData rib.Anim2 maxObj #Anim2 #FLOAT
		M3E_buildArefData rib.Anim3 maxObj #Anim3 #FLOAT
		M3E_buildArefData rib.Anim4 maxObj #Anim4 #FLOAT
		M3E_buildArefData rib.Anim5 maxObj #Anim5 #FLOAT
		M3E_buildArefData rib.Anim6 maxObj #Anim6 #FLOAT
		
		M3E_buildArefData rib.BGRAref1 maxObj #BGRAref1 #COLOUR
		M3E_buildArefData rib.BGRAref2 maxObj #BGRAref2 #COLOUR
		M3E_buildArefData rib.BGRAref3 maxObj #BGRAref3 #COLOUR
		
		local radiusScale = #(#startRadiusScale, #centerRadiusScale, #endRadiusScale)
		M3E_buildArefData rib.radiusScale maxObj radiusScale #VEC3D
		
		M3E_buildArefData rib.ribbonTwist maxObj #ribbonTwist #FLOAT
		M3E_buildArefData rib.ribbonLength maxObj #ribbonLength #FLOAT
		
		M3E_buildArefData rib.directionVariationAmount maxObj #directionVariationAmount #FLOAT
		M3E_buildArefData rib.directionVariationFrequency maxObj #directionVariationFrequency #FLOAT
		
		M3E_buildArefData rib.amplitudeVariationAmount maxObj #amplitudeVariationAmount #FLOAT
		M3E_buildArefData rib.amplitudeVariationFrequency maxObj #amplitudeVariationFrequency #FLOAT
		
		M3E_buildArefData rib.lengthVariationAmount maxObj #lengthVariationAmount #FLOAT
		M3E_buildArefData rib.lengthVariationFrequency maxObj #lengthVariationFrequency #FLOAT
		
		M3E_buildArefData rib.radiusVariationAmount maxObj #radiusVariationAmount #FLOAT
		M3E_buildArefData rib.radiusVariationFrequency maxObj #radiusVariationFrequency #FLOAT
		
		/*if (maxObj.enblSRib == true) then
		(
			M3E_buildArefData rib.srib.sribAnim1 maxObj #sribAnim1 #FLOAT
			M3E_buildArefData rib.srib.sribAnim2 maxObj #sribAnim2 #FLOAT
			M3E_buildArefData rib.srib.sribAnim3 maxObj #sribAnim3 #FLOAT
			M3E_buildArefData rib.srib.sribAnim4 maxObj #sribAnim4 #FLOAT
			M3E_buildArefData rib.srib.sribAnim5 maxObj #sribAnim5 #FLOAT
			M3E_buildArefData rib.srib.sribAnim6 maxObj #sribAnim6 #FLOAT
			M3E_buildArefData rib.srib.sribAnim7 maxObj #sribAnim7 #FLOAT
			M3E_buildArefData rib.srib.sribAnim8 maxObj #sribAnim8 #FLOAT
			M3E_buildArefData rib.srib.sribAnim9 maxObj #sribAnim9 #FLOAT
			M3E_buildArefData rib.srib.sribAnim10 maxObj #sribAnim10 #FLOAT
		)*/
	)
)

fn M3E_buildForcesAnims m3sc2forces =
(
	echo "Building forces animation data..."
	
	for i = 1 to m3sc2forces.count do
	(
		local fo = m3sc2forces[i]
		local maxObj = fo.maxObj
		
		M3E_buildArefData fo.forceStrenght maxObj #forceStrenght #FLOAT
		M3E_buildArefData fo.forceRange maxObj #forceRange #FLOAT
		M3E_buildArefData fo.forceUnk1 maxObj #forceUnk1 #FLOAT
		M3E_buildArefData fo.forceUnk2 maxObj #forceUnk2 #FLOAT
	)
)

fn M3E_buildParcAnims m3parcs =
(
	echo "Building particle emitter instance animation data..."
	
	for i = 1 to m3parcs.count do
	(
		local parc = m3parcs[i]
		local maxObj = parc.maxObj
		
		M3E_buildArefData parc.emissRate maxObj #rate #FLOAT
		M3E_buildArefData parc.partEmit maxObj #partEmit #UINT16
	)
)

fn M3E_buildAttachments m3data m3attachs =
(
	local m3attachslu = #()
	local m3attachsvol = #()
	local m3attachsvollu = #()
	
	for i = 1 to m3attachs.count do
	(
		local m3attach = m3attachs[i]
		local m3attachlu = -1 -- attachment lookup always -1?
		
		if ((matchPattern m3attach.name pattern:"Ref_Target*") or (matchPattern m3attach.name pattern:"Ref_Shield*")) then
		(
			local m3avol = M3E_AttachVolume.init()
			local m3avollu = 0 -- attachment volume lookup always 0?
			
			-- assign volume variables
			m3avol.bone = m3attach.bone
			m3avol.rad = m3attach.maxObj.attachRadius
			
			append m3attachsvol m3avol
			append m3attachsvollu m3avollu
		)
		
		append m3attachslu m3attachlu
	)
	
	-- assign built arrays to main container
	m3data.attach = m3attachs
	m3data.attachlu = m3attachslu
	m3data.attachvol = m3attachsvol
	m3data.attachvollu = m3attachsvollu
)

fn M3E_buildLightAnims m3lights =
(
	echo "Building lights animation data..."
	
	for i = 1 to m3lights.count do
	(
		local lightd = m3lights[i]
		local maxObj = lightd.maxObj
		
		M3E_buildArefData lightd.lightColor maxObj #lightColor #COLOUR3
		M3E_buildArefData lightd.lightIntensity maxObj #lightIntensity #FLOAT
		M3E_buildArefData lightd.specColor maxObj #specColor #COLOUR3
		M3E_buildArefData lightd.specIntensity maxObj #specIntensity #FLOAT
		M3E_buildArefData lightd.ulPr3 maxObj #ulPr3 #FLOAT
		M3E_buildArefData lightd.ulPr4 maxObj #ulPr4 #FLOAT
		M3E_buildArefData lightd.lightRange maxObj #lightRange #DEGTORAD
		M3E_buildArefData lightd.lightSpot maxObj #lightSpot #DEGTORAD
	)
)

fn M3E_compareKeys cont1 cont2 =
(
	if (cont1.keys.count != cont2.keys.count) then return false
	for i = 1 to cont1.keys.count do
	(
		local key1 = cont1.keys[i]
		local key2 = cont2.keys[i]
		
		if (key1.time != key2.time) then return false
		if (key2.value != key2.value) then return false
	)
	return true
)

fn M3E_comparePar par1 par2 =
(
	set time g_baseFrame
	local propNames = getPropNames par1
	for i = 1 to propNames.count do
	(
		-- check all the properties/controllers
		-- return false if we find any that don't match
		local propName = propNames[i]
		-- ignore rate and particles emitter, as these are present in PARC structures
		if (propName != #rate and propName != #partEmit and propName != #boxsize and propName != #Dummy) then
		(
			local prop1 = getProperty par1 propName
			local prop2 = getProperty par2 propName
			local prop1cont = getPropertyController par1 propName
			local prop2cont = getPropertyController par2 propName
			
			if (prop1 != prop2) then 
			(
				--echo (propName as string)
				--echo "exit1"
				return false
			)
			
			if (prop1cont == undefined) and (prop2cont != undefined) then 
			(
				--echo (propName as string)
				--echo "exit2"
				return false
			)
			
			if (prop1cont != undefined) and (prop2cont == undefined) then 
			(
				--echo (propName as string)
				--echo "exit3"
				return false
			)
			
			if (prop1cont != undefined) and (prop2cont != undefined) then
			(
				local keyMatch = M3E_compareKeys prop1cont prop2cont
				if (keyMatch == false) then return false
			)
		)
	)
	
	-- return true if we found no differences! yay!
	return true
)

fn M3E_getSceneObjects =
(
	echo "Getting scene objects..."
	
	max modify mode
	set coordsys world
	
	local m3objs = M3E_SceneObjects.init()
	
	if (objects.count < 1) then
	(
		-- throw error if scene is void of objects
		messagebox "No Scene Objects Found!"
		throw "No scene objects found"
	)
	
	-- Mesh sort, deselect unsuitable meshes
	for i = 1 to objects.count do
	(
		local obj = objects[i]
		objClass = classOf obj.baseObject as string
		local addObj = true
		if (obj.isHidden == true and g_exportHidden != true) then addObj = false -- check to see if we're exporting hidden
		if (obj.isFrozen == true and g_exportFrozen != true) then addObj = false -- or frozen
		if (objClass == "sc2boundsphere") then addObj = true -- override checks if we're grabbing the bound sphere
		if (obj.name == "_M3_Anim_Data" and objClass == "Point") then addObj = false -- ignore animation data container
		if (addObj == true) then 
		(
			case objClass of
			(
				"Editable_mesh": 
				(
					-- only grab meshes with assigned materials
					if (obj.material != undefined) then
					(
						local objMatClass = classOf obj.material as string
						-- and only grab meshes with the appropriate material type
						if (objMatClass == "sc2material" or objMatClass == "standardmaterial" or objMatClass == "multimaterial") then
						(
							append m3objs.meshes obj
						)
					)
				)
				"sc2boundsphere": m3objs.bndSphere = obj
				"sc2attachment": append m3objs.attachments obj
				"sc2pemitter2": 
				(
					-- only grab particle emitters with assigned materials
					if (obj.particleMaterial != undefined) then
					(
						local objMatClass = classOf obj.particleMaterial as string
						-- and only grab particle emitters with sc2material types
						if (objMatClass == "sc2material") then
						(
							-- check to see if the p. emitter is a copy
							local parcBool = false
							local parcID
							for j = 1 to m3objs.pemits.count do
							(
								local pemit = m3objs.pemits[j]
								local boolCheck
								boolCheck = M3E_comparePar pemit obj
								
								/*if (boolCheck == true) then 
								(
									parcBool = true
									parcID = j
								)*/
							)
							
							
							if (parcBool == true) then
							(
								-- add particle emitter to parc array (particle emitter copies)
								append m3objs.parcs obj
								append m3objs.parcIDs parcID
							)
							else
							(
								-- else it's a fresh particle emitter
								append m3objs.pemits obj
							)
						)
					)
					g_pemittersCount += 1
				)
				"Targetobject": ()
				"sc2ribbon": append m3objs.sc2ribbons obj
				"sc2force": append m3objs.sc2forces obj
				"sc2light": append m3objs.sc2lights obj
				"sc2fuzzy": append m3objs.sc2fuzzy obj
				"sc2tight": append m3objs.sc2tight obj
				default: append m3objs.bones obj
			)
		)
	)
	
	
	
	clearSelection()
	format "Scene OBJ: %\n" m3obj
	return m3objs
)

fn M3E_getUsedMats m3objs =
(
	echo "Gathering used materials..."
	local maxMats = M3E_MaxMaterials()
	
	-- gather mesh materials
	for i = 1 to m3objs.meshes.count do
	(
		local mesh = m3objs.meshes[i]
		
		-- If multimaterial used...
		if (parentmatclass == "Multimaterial") then
		(
			local mmats = #()
			for k = 1 to mesh.numfaces do
			(
				-- #MATERIALS/SUBMESHES
				-- Gather Used Materials
				local matid = getfacematid mesh k
			
				-- prevent error for out of range matid's
				if (matid > mesh.material.count) then 
				(
					local subid = (mod matid mesh.material.count) as integer
					if (subid == 0) then subid = mesh.material.count
					matid = subid
				)
				
				if ((findItem mmats mesh.material[matid]) == 0) then 
				(
					append mmats mesh.material[matid]
				)
			)
			
			-- setup multimaterial container
			local mmatInd = findItem maxMats.mats mesh.material
			if (mmatInd == 0) then 
			(
				append maxMats.mats mesh.material
				append maxMats.multimats mmats
			)
			else
			(
				append maxMats.multimats[mmatInd] mmats
			)
		)
		else
		(
			if ((findItem maxMats.mats mesh.material) == 0) then 
			(
				append maxMats.mats mesh.material
				append maxMats.multimats undefined -- set multimaterial component undefined
			)
		)
	)
	
	-- gather particle emitter materials
	for i = 1 to m3objs.pemits.count do
	(
		local pemit = m3objs.pemits[i]
		
		if ((findItem maxMats.mats pemit.particleMaterial) == 0) then
		(
			append maxMats.mats pemit.particleMaterial
			append maxMats.multimats undefined -- set multimaterial component null
		)
	)
	
	-- gather ribbon materials
	for i = 1 to m3objs.sc2ribbons.count do
	(
		local ribbon = m3objs.sc2ribbons[i]
		
		if ((findItem maxMats.mats ribbon.ribbonMaterial) == 0) then
		(
			append maxMats.mats ribbon.ribbonMaterial
			append maxMats.multimats undefined -- set multimaterial component null
		)
	)
	
	-- build full material list
	for i = 1 to maxMats.mats.count do
	(
		local mmat = maxMats.mats[i]
		local mmatClass = classOf mmat as string
		
		if (mmatClass == "Multimaterial") then
		(
			for j = 1 to maxMats.multimats[i].count do
			(
				local multmat = maxMats.multimats[i][j]
				append maxMats.fullMatList multmat
			)
		)
		else
		(
			append maxMats.fullMatList mmat
		)
	)
	
	return maxMats
)

fn M3E_getSkin m3objs =
(
	--format "Checking for skin..." to:listener
	local cskin = #()
	-- pre-intialize skin array to same size as mesh array
	if (m3objs.meshes.count > 0) then cskin[m3objs.meshes.count] = 0
	-- Find Skin classes for each mesh
	for i = 1 to m3objs.meshes.count do
	(
		local mesh = m3objs.meshes[i]
		local smod
		for j = 1 to mesh.modifiers.count do
		(
			local meshMod = mesh.modifiers[j]
			local modClass = classOf meshMod as string
			
			if (modClass == "Skin") then
			(
				smod = meshMod
			)
		)
		
		if (smod != undefined) then
		(
			if ((skinOps.getNumberBones smod) < 1) then smod = undefined
		)
		
		cskin[i] = smod
	)
	
	return cskin
)

fn M3E_Bone_ParentDepth bind mbones boneDepth =
(
	-- For organising bones based on depth
	local boneParent = mbones[bind].parent + 1
	if boneParent != 0 then 
	(
		boneDepth[boneParent].y += 1
		M3E_Bone_ParentDepth boneParent mbones boneDepth
	)
)

fn M3E_Bone_ChildDepth bind mbones =
(
	-- For organising bones based on depth
	local boneParent = mbones[bind].parent + 1
	if boneParent == 0 then 0
	else return ( 1 + M3E_Bone_ChildDepth boneParent mbones  )
)

fn M3E_Get_Bone_Depth linkBones =
(
	-- sort bones by depth
	bDepth = #()
	bDepth[linkBones.count] = 0
	
	-- do child depth
	for i=1 to linkBones.count do
	(
		-- set parent to 0 for now, update later
		bonerec = [(M3E_Bone_ChildDepth i linkBones), 0, i]
		bDepth[i] = bonerec
	)
	
	-- do parent depth
	for i = 1 to linkBones.count do
	(
		M3E_Bone_ParentDepth i linkBones bDepth
	)

	fn compfn a b = 
	( 
		-- order by child depth criteria
		if a.y < b.y then 
		(
			return 1
		)
		else if a.y > b.y then 
		(
			return -1
		)
		else if a.y == b.y then
		(
			-- order by parent depth criteria
			if a.x < b.x then
			(
				return 1
			)
			else if a.x > b.x then
			(
				return -1
			)
			else return 0
		)
	)
	
	qsort bDepth compfn

	return bDepth
)

fn M3E_buildBoneChain mbones bindex skinArray nonskinArray = 
(
	local mbone = mbones[bindex]
	local obj = mbone.maxObj
	local boneExists = false
	
	-- check for bone in skin array
	for i = 1 to skinArray.count do
	(
		if (obj == skinArray[i]) then boneExists = true
	)
	
	-- check for bone in non-skin array
	if (boneExists == false) then
	(
		for i = 1 to nonskinArray.count do
		(
			if (obj == nonskinArray[i]) then boneExists = true
		)
	)
	
	-- if not found, added it to the appropriate chain
	if (boneExists == false) then
	(
		-- if it's skinned, grow skin array
		if (mbone.skinned == true) then
		(
			append skinArray obj
		)
		else
		(
			-- else grow non skin array
			append nonskinArray obj
		)
		-- if it has children, go down the chain
		for i = 1 to mbone.children.count do
		(
			local cindex = mbone.children[i]
			M3E_buildBoneChain mbones cindex skinArray nonskinArray
		)
	)
)

fn M3E_SetSkinned mbones bindex =
(
	mbones[bindex].skinned = true
	local mb = mbones[bindex]
	local bpar = mb.parent
	if bpar != -1 then
	(
		M3E_SetSkinned mbones (bpar + 1)
	)
)

fn M3E_buildCameras m3obj =
(
	echo "Building camera list..."
	format "Cameras: %\n" m3obj.cameras.count;
	cam_objs = m3obj.cameras;
	m3obj.cameras = #();
	for cam in cam_objs do
	(
		format "Processing camera: %\n" cam.name;
		m3_cam = M3E_Cam.Init cam;
		m3_cam.d1 = 0;
		for b = 1 to m3obj.bones.count do
			if (m3obj.bones[b]).name == cam.name do m3_cam.d1 = b - 1;
		format "Bone id: %\n" m3_cam.d1;
		append m3obj.cameras m3_cam;
		
	)
)

fn M3E_buildBones m3obj =
(
	echo "Building bone list..."
	
	global mbones = m3obj.bones
	
	-- temp. parents for ordering
	for i = 1 to mbones.count do
	(
		bchild = mbones[i]
		if bchild.parentptr != undefined then
		(
			for j = 1 to mbones.count do
			(
				local bpar = mbones[j]
				if (bchild.parentptr == bpar.maxObj) then (bchild.parent = (j - 1))
			)
		)
		else
		(
			bchild.parent = -1
		)
		
		mbones[i] = bchild
	)

	-- setup skinned chains
	global sknBones = m3obj.skinBones
	for i = 1 to sknBones.count do
	(
		local boneid = sknBones[i]
		M3E_SetSkinned mbones boneid
	)
	
	-- order by child/parent depth
	local boneDepth = M3E_Get_Bone_Depth mbones
	
	-- children assignment for building chains
	for i = 1 to mbones.count do
	(
		local obj = mbones[i].maxObj
		--print obj to:listener
		mbones[i].children = #()
		for j = 1 to obj.children.count do
		(
			local childObj = obj.children[j]
			local childIndex
			for k = 1 to mbones.count do
			(
				if (childObj.name == mbones[k].name) then childIndex = k
			)
			
			if (childIndex != undefined) then 
			(
				append mbones[i].children childIndex
			)
		)
	)
	
	-- build bone chains
	local skinChains = #()
	local nonskinChains = #()
	global finalChain = #()
	for i = 1 to mbones.count do
	(
		-- build chains based on depth
		local j = boneDepth[i].z
		M3E_buildBoneChain mbones j skinChains nonskinChains
	)

	finalChain = join skinChains nonskinChains
	
	-- build proper index
	global realBones = #()
	realBones[finalChain.count] = 0
	for i = 1 to finalChain.count do
	(
		for j = 1 to mbones.count do
		(
			if (finalChain[i] == mbones[j].maxObj) then
			(
				realBones[i] = mbones[j]
			)
		)
	)
	
	-- Setup M3 bone list
	local m3bones = #()
	for i = 1 to realBones.count do
	(
		local m3bone = M3E_Bone.init()
		bchild = realBones[i]
		m3bone.name = bchild.name
		-- setup base pose info
		if bchild.parentptr != undefined then
		(
			for j = 1 to realBones.count do
			(
				bpar = realBones[j]
				-- set correct parent
				if (bchild.parentptr == bpar.maxObj) then 
				(
					m3bone.parent = (j - 1)
					local wbindmat = bchild.bindmat * bpar.invbindmat
					m3bone.rot.initValue = inverse wbindmat.rotationpart
					m3bone.trans.initValue = wbindmat.translation
				)
			)
		)
		else
		(
			m3bone.parent = -1
			m3bone.rot.initValue = inverse bchild.bindmat.rotationpart
			m3bone.trans.initValue = bchild.bindmat.translation
		)
		
		in coordsys parent
		(
			m3bone.bscale.initValue = bchild.maxObj.scale
		)
		
		m3bone.iref = bchild.iref
		m3bone.maxObj = bchild.maxObj
		
		append m3bones m3bone
	)
	
	for k =1 to mbones.count do
	(
		local boneFlags = 0
		boneFlags		= bit.set boneFlags 1 (if((getUserProp mbones[k].maxObj "InheritT")==undefined)then(false)else((getUserProp mbones[k].maxObj "InheritT")))
		boneFlags		= bit.set boneFlags 2 (if((getUserProp mbones[k].maxObj "InheritS")==undefined)then(false)else((getUserProp mbones[k].maxObj "InheritS")))
		boneFlags		= bit.set boneFlags 3 (if((getUserProp mbones[k].maxObj "InheritR")==undefined)then(false)else((getUserProp mbones[k].maxObj "InheritR")))
		boneFlags		= bit.set boneFlags 5 (if((getUserProp mbones[k].maxObj "BillboardBone")==undefined)then(false)else((getUserProp mbones[k].maxObj "BillboardBone")))
		boneFlags		= bit.set boneFlags 9 (if((getUserProp mbones[k].maxObj "2DProjection")==undefined)then(false)else((getUserProp mbones[k].maxObj "2DProjection")))
		boneFlags		= bit.set boneFlags 10 (if((getUserProp mbones[k].maxObj "Animated")==undefined)then(true)else((getUserProp mbones[k].maxObj "Animated")))
		boneFlags		= bit.set boneFlags 11 (if((getUserProp mbones[k].maxObj "EnableIK")==undefined)then(false)else((getUserProp mbones[k].maxObj "EnableIK")))
		boneFlags		= bit.set boneFlags 12 (if((getUserProp mbones[k].maxObj "Skinned")==undefined)then(true)else((getUserProp mbones[k].maxObj "Skinned")))
		boneFlags		= bit.set boneFlags 14 true
		
		m3bones[k].flags = boneFlags
	)
	
	-- Update skinned bone list
	global newSkinbones = #()
	for i = 1 to sknBones.count do
	(
		local sbone = sknBones[i]
		local newsbone
		for j = 1 to realBones.count do
		(
			local bindex = realBones[j].boneIndex
			if (sbone == bindex) then 
			(
				newsbone = (j - 1)
				local flags = m3bones[j].flags
				flags = bit.set flags 10 true
				flags = bit.set flags 12 true
				m3bones[j].flags = flags
			)
		)
		
		append newSkinbones newsbone
	)
	
	
	
	m3obj.skinBones = newSkinbones
		
	-- build M3 list
	m3obj.bones = #()
	for i = 1 to m3bones.count do
	(
		append m3obj.bones m3bones[i]
	)
)

fn M3E_getBones m3objs =
(
	echo "Getting bones..."
	local mbones = #()
	
	-- Add bones for non-skinned meshes
	/*for i = 1 to m3objs.meshes.count do
	(
		local msh = m3objs.meshes[i]
		local mskin = m3objs.skin[i]
		
		if (mskin == undefined) then
		(
			append m3objs.bones msh
		)
	)*/
	
	-- add code for attachments too...
	/*for i = 1 to m3objs.attachments.count do
	(
		local m3attach = m3objs.attachments[i]
		append m3objs.bones m3attach
	)*/
	
	-- ...and particle emitters
	/*for i = 1 to m3objs.pemits.count do
	(
		local m3pemit = m3objs.pemits[i]
		append m3objs.bones m3pemit
	)*/
	
	-- AND particle emitter instances!
	/*for i = 1 to m3objs.parcs.count do
	(
		local m3parc = m3objs.parcs[i]
		append m3objs.bones m3parc
	)*/
	
	-- ribbons are boned.. get it? boned? ;)
	/*for i = 1 to m3objs.sc2ribbons.count do
	(
		local m3ribbon = m3objs.sc2ribbons[i]
		append m3objs.bones m3ribbon
		format "m3ribbon %\n" m3ribbon
	)*/
	
	-- ...and lights have to have bones too right?
	/*for i = 1 to m3objs.sc2lights.count do
	(
		local m3light = m3objs.sc2lights[i]
		append m3objs.bones m3light
		format "m3light %\n" m3light
	)*/
		
	-- Process scene objects into bones
	for i = 1 to m3objs.bones.count do
	(
		local sbone = m3objs.bones[i]
		local maxBone = M3E_MaxBone()
		
		maxBone.maxObj = sbone
		local maxObjClass = classOf sbone as string
		maxBone.name = sbone.name
		-- if (maxObjClass != "sc2attachment") then maxBone.name = sbone.name else maxBone.name = sbone.attachName
		maxBone.boneIndex = i
		if (sbone.parent != undefined) then 
		(
			local parent = sbone.parent
			maxBone.parentptr = parent
			if (parent.isHidden == true and g_exportHidden != true) then
			(
				throw ("Hidden object ("+parent.name+") found in bone system!\nEither check 'Export Hidden Objects' or unhide the object")
			)
			if (parent.isFrozen == true and g_exportFrozen != true) then
			(
				throw ("Frozen object ("+parent.name+") found in bone system!\nEither check 'Export Frozen Objects' or unfreeze the bone")
			)
		)
		
		-- get IREF transform
		at time g_bindposeFrame
		(
			local mat = inverse sbone.transform
			maxBone.iref = M3E_Matrix.convObjMatrix mat
		)
		
		-- get bindpose transform
		at time g_baseFrame
		(
			maxBone.bindmat = sbone.transform
		)
		
		maxBone.invbindmat = inverse maxBone.bindmat

		append mbones maxBone
	)

	return mbones
)

fn M3E_Get_ssData stringStrm property type:#value =
(
	-- reposition at start of stream
	seek stringStrm 0
	if (skipToString stringStrm property != undefined) then
	(
		local val
		case type of
		(
			#string: val = readLine stringStrm
			#value: val = readValue stringStrm
			#boolean: val = readValue stringStrm as booleanClass
		)

		return val
	)
)

fn M3E_Get_AnimData animss sc2Vers =
(
	local anim = M3E_STC.Blank()
	
	anim.name 		= sc2_Get_ssData animss ".name:" type:#string
	anim.seqInd	= sc2_Get_ssData animss ".seqInd:"
	anim.cstart 	= sc2_Get_ssData animss ".cstart:"
	anim.cend 		= sc2_Get_ssData animss ".cend:"
	if (sc2vers != undefined) then
	(
		case of
		(
			default:
			(
				anim.frequency	= sc2_Get_ssData animss ".freq:"
				anim.moveSpeed 	= sc2_Get_ssData animss ".moveSpeed:"
				anim.looping		= sc2_Get_ssData animss ".looping:" type:#boolean
			)
		)
	)
	else
	(
		anim.frequency	= 100
		anim.moveSpeed 	= 0.0
		anim.looping		= false
	)
	
	return anim
)

fn M3E_getAnimations =
(
	local aObj = getnodebyname g_AnimObj
	local aObjClass = classOf aObj as string
	if (isValidNode aObj == true and aObjClass == "Point") then
	(
		echo "Getting animation scene information..."
		local adata = #()
		local acss = (getAppData aObj 1) as stringstream
		local sc2vers = M3E_Get_ssData acss ".version:"
		local animCount = M3E_Get_ssData acss ".count:"
		
		local animInd = 2 -- where anims start
		
		for i = 1 to animCount do
		(
			local stcss = (getAppData aObj animInd) as stringstream
			local anim = M3E_Get_AnimData stcss sc2vers
			
			-- exclude global animations from being exported for now, since they screw up the model animations
			if ((matchPattern anim.name pattern:"GL*") != true) then
			(
				-- convert bool to integer
				if (anim.looping == true) then anim.looping = 0 else anim.looping = 1
				
				append adata anim
			)
			
			animInd += 1
		)
		
		return adata
	)
	else return undefined
)

fn M3E_getNodeParentRotation theNode = 
( 
	tRot = in coordsys parent theNode.rotation
  	--in coordsys (transmatrix theNode.transform.pos) return theNode.rotation
  	return tRot
)

fn M3E_buildBoneAnims mbones =
(
	echo "Building bone animation data..."
	for i = 1 to mbones.count do
	(
		local mbone = mbones[i]
		local maxObj = mbone.maxObj
		
		-- Gather the Keys 
		for a = 1 to 3 do
		(
			local animType
			local animRef
			local prop

			case a of
			(
				1: 
				(
					animType = #VEC3D
					animRef = mbone.trans
					prop = #position
				)
				2: 
				(
					animType = #QUAT
					animRef = mbone.rot
					prop = #rotation
				)
				3:
				(
					animType = #VEC3D
					animRef = mbone.bscale
					prop = #scale
				)
			)
			M3E_buildArefData animRef maxObj prop animType
		)
	)
)

fn M3E_buildAnimations m3data animData =
(
	-- build SEQ, STG, STS data from STC data
	local SEQs = #()
	local STGs = #()
	local STSs = #()
	
	-- all end events have the same ID
	local evntanimid = Random 1 1e8
	
	-- ...and the same location in the STC data arrays
	local evntanimref = M3E_Animoffs.init()
	evntanimref.sdind = 0
	evntanimref.aind = 0
	
	for i = 1 to animData.count do
	(
		local stc = animData[i]
		local seq = M3E_SEQS.init()
		seq.name = stc.name
		seq.maxframes = stc.cend - stc.cstart
		seq.maxframes = M3E_Convert_Time seq.maxframes
		seq.moveSpeed = stc.moveSpeed
		local seqFlags = 0
		local loopBool
		if stc.looping > 0 then loopBool = true else loopBool = false
		seq.seqFlags = bit.set seqFlags 1 loopBool
		seq.frequency = stc.frequency
		
		-- Gather bound sphere at first frame for each animation
		set time stc.cstart
		seq.bndSphere = M3E_GetBndSphere()
		
		local stg = M3E_STG.init()
		stg.name = seq.name
		local stcind = i - 1
		stg.stcind = #(stcind)
		
		-- Generate end event data
		-- add an end event to each animation
		local evntend = M3E_AnimBlock.initEvnt()
		evntend.frames[1] = seq.maxframes
		evntend.fend = seq.maxframes
		
		-- append generated data to STC
		append stc.animid evntanimid
		append stc.animref evntanimref
		append stc.sdata[1] evntend
		
		-- sort animid's by ascending value
		local animSorted = #()
		local animidSorted = #()
		local animrefSorted = #()
		
		for j = 1 to stc.animid.count do
		(
			local animid = stc.animid[j]
			local animref = stc.animref[j]
			append animSorted #(animid, animref)
		)
		
		-- sort by animid values
		fn compareAnimid a1 a2 = if (a1[1] > a2[1]) then return 1 else if (a1[1] < a2[1]) then return -1 else return 0
		qsort animSorted compareAnimid
		
		for j = 1 to animSorted.count do
		(
			local animid = animSorted[j][1]
			local animref = animSorted[j][2]
			
			-- update animid array as we build animref array
			append animidSorted animid
			append animrefSorted animref
		)
		
		-- update STC with sorted ID's and Offs
		stc.animid = animidSorted
		stc.animref = animrefSorted
		
		local sts = M3E_STS.init()
		sts.animid = stc.animid
		
		stc.name = seq.name + "_full"
		stc.seqlu = stcind
		stc.stglu = stcind
		
		append SEQs seq
		append STGs stg
		append STSs sts
		animData[i] = stc
	)
	
	m3data.SEQS = SEQs
	m3data.STG = STGs
	m3data.STS = STSs
	
	-- assign STC data
	m3data.STC = animData
)

fn M3E_buildFaces obj triindx regn uvChannels =
(
	-- annoyingly complicated!
	vertindices = getface obj triindx
	if (meshop.getMapSupport obj g_mapchannel) then
		tvertindices = meshop.getmapface obj g_mapchannel triindx
	else
		tvertindices = [0,0,0]
	vertindices = #(vertindices.x as integer, vertindices.y as integer, vertindices.z as integer)
	tvertindices = #(tvertindices.x as integer, tvertindices.y as integer, tvertindices.z as integer)
	m3vertindices = #()
	for i=1 to 3 do
	(
		vind = vertindices[i]
		tvind = tvertindices[i]
		-- findItem vastly more efficient over old function method
		local index = findItem regn.vertsList [vind, tvind]
		if (index == 0) then
		(
			v = M3E_Vertex.init()
			v.pos = getVert obj vind
			--v.normal	= getNormal obj vind
			local uvarray = #()
			
			for c = 1 to uvChannels.count do
			(
				local uv
				local uvChan = uvChannels[c]
				if (uvChan != undefined) then
				(
					if (meshop.getMapSupport obj uvChan) then uv = meshop.getmapvert obj uvChan tvind
					else uv = [0,0,0]
				)
				else uv = [0,0,0]
				append uvarray uv
			)
			
			v.uv = uvarray
			v.origvertindx = vind
			v.origtvertindx = tvind
			
			append regn.verts v
			append regn.vertsList [vind, tvind]
			m3vertindices[i] = regn.verts.count - 1
		)
		else
		(
			m3vertindices[i] = index - 1
		)
	)
	
	append regn.faces m3vertindices
	append regn.facesList vertindices
)

fn M3E_buildDIV m3obj =
(
	-- run through submeshes
	echo "Building submeshes..."
	local div = m3obj.submeshes
	for i = 1 to div.regn.count do
	(
		regn = div.regn[i]
		
		-- Order submesh verts and faces
		regn.nvert = regn.verts.count
		regn.nface = regn.faces.count * 3
		if (i > 1) then
		(
			prevregn = div.regn[i-1]
			regn.indvert = prevregn.indvert + prevregn.nvert
			regn.indface = prevregn.indface + prevregn.nface
		)
		else
		(
			regn.indvert = 0
			regn.indface = 0
		)
		
		-- handle skinned bones
		regn.bcount = regn.skinBones.count
		if (i > 1) then 
		(
			local prevregn = div.regn[i - 1]
			regn.indbonelu = prevregn.indbonelu + prevregn.bcount
		)
		else
		(
			regn.indbonelu = 0
		)
		
		regn.nbonelu = regn.bcount
		
		-- build single faces list
		for j = 1 to regn.faces.count do
		(
			for k = 1 to 3 do
			(
				append div.faces regn.faces[j][k]
			)
		)
	)
	
	div.msec = #(M3E_MSEC())
	
	-- assign new div back to main container
	m3obj.submeshes = div
)

fn M3E_getSubmesh m3objs =
(
	echo "Getting submeshes..."
	div = M3E_DIV.init()
	
	max modify mode
	set time g_bindposeFrame
	
	for i = 1 to m3objs.meshes.count do
	(
		-- Create Submeshes
		local mesh = m3objs.meshes[i]
		local matInd = findItem m3scene.materials.mats mesh.material
		local mmatClass = classOf mesh.material as string
		
		if (mmatClass == "Multimaterial") then
		(
			for j = 1 to m3scene.materials.multimats[matInd].count do
			(
				local multiMat = m3scene.materials.multimats[matInd][j]
				r = M3E_REGN.init()
				r.maxMesh = mesh
				b = M3E_BAT.init()
				b.subid = div.regn.count
				b.matid = (findItem m3scene.materials.fullMatList multiMat) - 1
				
				local uvChannels = #()
				append uvChannels g_mapchannel
				local mClass = classOf multiMat as string
				if (mClass == "sc2material") then
				(
					if (bit.get m3objs.vertFlags 19 == true) then
					(
						local DUVchan
						if (multiMat.decalMap != undefined) then
						(
							if (multiMat.decalMap.coords.mapping == 0) then 
							(
								if (multiMat.decalMap.mappingSource > 1) then DUVchan = multiMat.decalMap.coords.MapChannel
							)
						)
						
						append uvChannels DUVchan
					)
				)
				
				for k = 1 to mesh.numfaces do
				(
					local matid = getfacematid mesh k
					if (matid > mesh.material.count) then 
					(
						local subid = (mod matid mesh.material.count) as integer
						if (subid == 0) then subid = mesh.material.count
						matid = subid
					)
					
					if (mesh.material[matid] == multiMat) then
					(
						M3E_buildFaces mesh k r uvChannels
					)
				)
				
				append div.regn r
				append div.bat b
			)
		)
		else
		(
			local mat = mesh.material
			r = M3E_REGN.init()
			r.maxMesh = mesh
			b = M3E_BAT.init()
			b.subid = div.regn.count
			b.matid = (findItem m3scene.materials.fullMatList mat) - 1
			
			local uvChannels = #()
			append uvChannels g_mapchannel
			local mClass = classOf mat as string
			if (mClass == "sc2material") then
			(
				if (bit.get m3objs.vertFlags 19 == true) then
				(
					local DUVchan
					if (mat.decalMap != undefined) then
					(
						if (mat.decalMap.coords.mapping == 0) then 
						(
							if (mat.decalMap.mappingSource > 1) then DUVchan = mat.decalMap.coords.MapChannel
						)
					)
					
					append uvChannels DUVchan
				)
			)
			
			for k = 1 to mesh.numfaces do
			(
				M3E_buildFaces mesh k r uvChannels
			)
			
			append div.regn r
			append div.bat b
		)
	)

	return div
)

fn M3E_buildVertices regn =
(
	local varray = #()
	for i = 1 to regn.count do
	(
		local sm = regn[i]
		for j = 1 to sm.verts.count do
		(
			local v = sm.verts[j]
			local cnorm, cuv
				
			-- convert from float to byte
			for k = 1 to 4 do
			(
				v.bw[k] = (v.bw[k] * 255) as integer
			)
			
			-- compress normals
			cnorm = #(0,0,0,0)
			for k = 1 to 3 do
			(
				local cn = 1 + v.normal[k]
				cn = (cn / 2) * 255
				cnorm[k] = cn as integer
			)
			v.normal = cnorm
			
			-- compress UV's
			local cuvarray = #()
			for k = 1 to v.uv.count do
			(
				local vuv = copy v.uv[k]
				cuv = #(0,0)
				if (g_flipuvy) then vuv[2] = 1 - vuv[2]
				for h = 1 to 2 do
				(
					cuv[h] = (vuv[h] * 2046) as integer
				)
				append cuvarray cuv
			)
			
			v.uv = cuvarray
			
			append varray v
		)
	)
	
	return varray
)

struct M3E_faceNorm
(
	normal, area
)

fn M3E_getFaceSmoothGroupB obj face =
(
	local sgroup_val=getFaceSmoothGroup obj face
	local sg_bitarray=#{}

	if sgroup_val < 0 do
	(
		sg_bitarray[32] = true
		sgroup_val -= 2^31
	)

	for i = 1 to 31 do
	(
		sg_bitarray[i]= (mod sgroup_val 2 > .5)
		sgroup_val /= 2
	)

	(sg_bitarray as array)
)

fn M3E_getVertexNormals regn =
(
	local lastAnimRange, lastSliderTime
	
	if (g_doCustomVertNormals == true) then
	(
		-- setup time for edit_normals modifier
		lastAnimRange = animationRange
		lastSliderTime = sliderTime
		
		local int1 = g_bindposeFrame
		local int2 = int1 + 1
		animationRange = interval int1 int2
		sliderTime = int1
	)
	
	for r = 1 to regn.count do
	(
		local sm = regn[r]
		local mesh = sm.maxMesh

		local en
		local en_GetNormalID
		local en_GetNormal
		if (g_doCustomVertNormals == true) then
		(
			en = edit_normals()
			addModifier mesh en
			modPanel.setCurrentObject en
			en_GetNormalID = en.GetNormalID
			en_GetNormal = en.GetNormal
		)
		
		local usedVerts = #() -- used for FindItem
		local usedVertsNormals = #() -- used for normal comparison
		local newVertsReference = #()
		local newVertsList = #() -- used for building new vertex list
		for i = 1 to sm.faces.count do
		(
			local faces = getFace mesh i
			local m3faces = sm.faces[i]
			local faceSG = M3E_getFaceSmoothGroupB mesh i
			
			-- echo ("Face " + i as string)
			
			for v = 1 to 3 do
			(
				-- echo ("Vert " + v as string)
				local vInd = faces[v]
				local m3vInd = (m3faces[v] + 1)
				local m3vert = copy sm.verts[m3vInd]
				local my_vert = #{vInd}
				
				local vFaces = meshop.getFacesUsingVert mesh my_vert
				vFaces = vFaces as array

				--echo ("Shared faces: " + vFaces as string)
				local vertNormal = [0,0,0]
				
				if (g_doCustomVertNormals == true) then
				(
					local normID = en_GetNormalID i v
					--vertNormal = normalize (en_GetNormal normID)
					vertNormal = en_GetNormal normID
				)
				else
				(
					-- old smoothing group method
					if (faceSG.count > 0) then
					(
						for n = 1 to vFaces.count do
						(
							local vFace = vFaces[n]
							local vFaceSG = M3E_getFaceSmoothGroupB mesh vFace
							local vFaceProc = false
							
							for s = 1 to vFaceSG.count do
							(
								local vFaceInd = findItem faceSG vFaceSG[s]
								if (vFaceInd > 0 and vFaceProc == false) then
								(
									--echo ("Face: " + vFace as string)
									local faceNormal = getFaceNormal mesh vFace
									--echo ("Face Normal: "+ faceNormal as string)
									vertNormal = normalize (vertNormal + faceNormal)
									vFaceProc = true
								)
							)
						)
					)
					else
					(
						local faceNormal = getFaceNormal mesh i
						vertNormal = normalize (vertNormal + faceNormal)
					)
				)
				
				-- echo ("vertNormal: "+vertNormal as string)

				local usedVertsInd = findItem usedVerts m3vInd
				local vertNormalInd = 0
				
				if (usedVertsInd > 0) then
				(
					local uVertNormals = usedVertsNormals[usedVertsInd]
					-- max can't compare float values properly sometimes (what the hell???)
					-- must compare the point3's as strings to bypass the error
					vertNormalInd = findItem uVertNormals (vertNormal as string)
				)
				
				if (vertNormalInd > 0) then 
				(
					local vRef = newVertsReference[usedVertsInd][vertNormalInd]
					sm.faces[i][v] = (vRef - 1)
				)
				else
				(
					-- copy original vertex and update the normal
					local newm3vert = copy m3vert
					newm3vert.normal = vertNormal
					
					append newVertsList newm3vert
					
					sm.faces[i][v] = newVertsList.count - 1 -- update original array
					
					-- for searching for the same vertex with the same normal purposes
					local vListInd = findItem usedVerts m3vInd
					
					if (vListInd > 0) then
					(
						append usedVertsNormals[vListInd] vertNormal -- append new normal to nested array
						append newVertsReference[vListInd] newVertsList.count
					)
					else
					(
						append usedVerts m3vInd
						append usedVertsNormals #(vertNormal as string) -- create nested array with normal (needs to be string for comparison!!!)
						append newVertsReference #(newVertsList.count) -- create same nested array with vertex reference
					)
				)
			) -- end v for loop
		) -- end faces for loop
			
		/*
		echo ("new vert list: "+newVertsList as string)
		echo ("usedVertNormals: "+usedVertsNormals as string)
		echo ("vertReferences: "+newVertsReference as string)
		echo ("used verts: "+usedVerts as string)
		*/
		
		if (en != undefined) then deleteModifier mesh mesh.modifiers[1]
		
		sm.verts = newVertsList

		-- update original array
		regn[r] = sm
	) -- end regn for loop
	
	if (g_doCustomVertNormals == true) then
	(
		-- return to original time if edit_normals modifier used
		animationRange = lastAnimRange
		sliderTime = lastSliderTime		
	)
)

fn M3E_normWeights weights =
(
	local bw = #()
	local totalWeight = 0
	
	-- get total
	for i = 1 to 4 do
	(
		total += weights[i]
	)
	
	-- get ratio out of 1
	for i = 1 to 4 do
	(
		local normWeight = weights[i] / totalWeight
		append bw normWeight
	)
	
	return bw
)

fn M3E_getVertices m3objs m3modl =
(
	echo "Getting vertex list..."
	local varray = #()
	
	local regn = m3modl.submeshes.regn
	
	for i = 1 to regn.count do
	(
		local sm = regn[i]
		local smMesh = sm.maxMesh
		local meshInd = findItem m3objs.meshes smMesh
		local smSkin
		if (meshInd > 0) then smSkin = m3objs.skin[meshInd]
		
		if (smSkin != undefined) then
		(
			modPanel.setCurrentObject smSkin
		)
		
		for j = 1 to sm.verts.count do
		(
			local v = sm.verts[j]
			local vIndex = v.origvertindx
			
			local weights
			local bIndices
			if (smSkin != undefined) then
			(
				weights = #()
				bIndices = #()
				
				local nWeights = skinops.getvertexweightcount smSkin vIndex
				for w = 1 to nWeights do
				(
					local weightval = skinops.getvertexweight smSkin vIndex w
					
					-- only process non-zero weights
					if (weightval > 0) then
					(
						local boneid = skinops.getvertexweightboneid smSkin vIndex w
						local bname = skinops.getBoneName smSkin boneid 1
						local vboneid
						local skinBoneid
						
						for b = 1 to m3modl.bones.count do
						(
							local m3bone = m3modl.bones[b]
							if (bname == m3bone.maxObj.name) then vboneid = b
						)
					
						if (vboneid != undefined) then
						(
							skinBoneid = (findItem sm.skinBones vboneid) - 1
						)
						else throw "Vertex weighted to undefined bone!"
						
						if (skinBoneid == -1) then 
						(
							-- create local list
							append sm.skinBones vboneid
							-- create global list
							append m3modl.skinBones vboneid
							-- adjust boneid to be based on local list for verts
							skinBoneid = (sm.skinBones.count - 1)
						)
						
						append weights weightval
						append bIndices skinBoneid
					)
				)
				
				-- Sort weights into 4
				local newWeights = #(0,0,0,0)
				local newBindices = #(0,0,0,0)
	
				for k = 1 to 4 do
				(
					local weight = 0
					local bIndex = 0
					if (k <= weights.count) do
					(
						for w = 1 to weights.count do
						(
							if (k == 1) then
							(
								if (weights[w] > weight) then (weight = weights[w]; bIndex = bIndices[w])
							)
							else
							(
								if (weights[w] > weight and weights[w] <= newWeights[k-1]) then 
								(
									if ((findItem newBindices bIndices[w]) == 0) then
									(
										weight = weights[w]
										bIndex = bIndices[w]
									)
								)
							)
						)
					)
					
					newWeights[k] = weight
					newBindices[k] = bIndex
				)
				
				weights = newWeights
				bIndices = newBindices
			)
			else --if (smSkin != undefined) ... else
			(
				local bname = smMesh.name
				local vboneid
				local skinBoneid
				
				weights = #(1,0,0,0)
				bIndices = #(0,0,0,0)
				
				for b = 1 to m3modl.bones.count do
				(
					local m3bone = m3modl.bones[b]
					if (bname == m3bone.maxObj.name) then vboneid = b
				)
			
				if (vboneid != undefined) then
				(
					skinBoneid = (findItem sm.skinBones vboneid) - 1
				)
				else throw "Vertex weighted to undefined bone!"
				
				if (skinBoneid == -1) then 
				(
					-- create local list
					append sm.skinBones vboneid
					-- create global list
					append m3modl.skinBones vboneid
					-- adjust boneid to be based on local list for verts
					skinBoneid = (sm.skinBones.count - 1)
				)
				
				bIndices[1] = skinBoneid
			)
			
			v.bw = weights
			v.bi = bIndices
			
			-- update sm verts in array
			sm.verts[j] = v
		)
		
		regn[i] = sm
	)
	
	-- get proper normals
	echo "Getting vertex normals..."
	M3E_getVertexNormals regn
	
	-- build vertices for export
	echo "Building vertex list..."
	varray = M3E_buildVertices regn

	return varray
)

fn M3E_checkPath cPath =
(
	local modPath = cPath
	-- check for type of path 
	-- hard disk 
	if (matchPattern modPath pattern:"*:\\*") then
	(
		-- check if hard disk path has end backslash, add it if not
		if (modPath[modPath.count] != "\\") then modPath += "\\"
	)
	else
	(
		-- internal MPQ
		if (modPath[modPath.count] != "/") then modPath += "/"
	)
	
	return modPath
)

fn M3E_getMaterials m3mats =
(
	echo "Getting materials..."
	
	mtlarray = #()

	for i = 1 to m3mats.fullMatList.count do
	(
		local mmat = m3mats.fullMatList[i]
		local matType = mmat.materialType
		
		local mtl
		case matType of
		(
			1: mtl = M3E_Ter.init()
			2: mtl = M3E_Mat.init()
			3: mtl = M3E_Dis.init mmat;
			4: mtl = M3E_Vol.init()
		)
		
		mtl.name = mmat.name
		
		local matclass = classOf mmat as string
		
		if (matclass != "Standardmaterial" and matclass != "sc2material") then
		(
			--messagebox "Materials must be of Standard or Starcraft 2 type!"
			throw "Materials must be of Standard or StarCraft II Material type "				
		)
		
		-- Export UI settings
		-- 1. Standard Material
		if (matType == 2) then
		(
			if (matclass == "Standardmaterial") then
			(
				mtl.spec	= mmat.specularLevel
			)
			
			-- 2. Starcraft 2 Material
			if (matclass == "sc2material") then
			(
				-- Flag set
				local flags = 0
				--{
				flags = bit.set flags 3 mmat.Unfogged
				flags = bit.set flags 4 mmat.TwoSided
				flags = bit.set flags 5 mmat.Unshaded
				flags = bit.set flags 6 mmat.NoShadowsCast
				flags = bit.set flags 7 mmat.NoHitTest
				flags = bit.set flags 8 mmat.NoShadowsReceived
				flags = bit.set flags 9 mmat.DepthPrepass
				flags = bit.set flags 10 mmat.UseTerrainHDR
				flags = bit.set flags 12 mmat.SplatUVfix
				flags = bit.set flags 13 mmat.SoftBlending
				flags = bit.set flags 15 mmat.ForParticles
				--}
				
				local obFlags = 0
				obFlags = bit.set obFlags 1 mmat.OF1
				obFlags = bit.set obFlags 2 mmat.OF2
				obFlags = bit.set obFlags 3 mmat.OF3
				obFlags = bit.set obFlags 4 mmat.OF4
				
				mtl.obFlags			= obFlags
				mtl.flags				= flags
				mtl.blendMode 	= (mmat.blendMode - 1) -- -1 due to using 1 based index
				mtl.priority			= mmat.priority
				mtl.spec				= mmat.specVal
				mtl.spec_mult		= mmat.specMult
				mtl.specF			= mmat.specF
				mtl.emis_mult		= mmat.emisMult
				mtl.cutoutThresh = mmat.cutoutThresh
				mtl.layerBlendType = (mmat.layerBlendType - 1)
				mtl.emisBlendType = (mmat.emissiveBlendType - 1)
				
				case mmat.emisMode of
				(
					1: mtl.emisMode = 0
					2: mtl.emisMode = 2
					3: mtl.emisMode = 4
				)
				
				mtl.emisMode		= mmat.emisMode
				mtl.specType		= mmat.specType
			)
				
			-- Map handling
			for k = 1 to 13 do
			(
				local layr = M3E_LAYR.init()
				local fullPath = ""
				local layrBitmap
				local curMap

				-- Different Material Handling
				-- 1. Standard Material
				if (matclass == "Standardmaterial") then
				(
					case k of
					(
						1: curMap = mmat.diffuseMap
						3: curMap = mmat.specularMap
						4: curMap = mmat.selfIllumMap
						6: curMap = mmat.reflectionMap
						10: curMap = mmat.bumpMap
					)
				)
				
				-- 2. Starcraft 2 Material
				if (matclass == "sc2material") then
				(
					case k of
					(
						1: curMap = mmat.diffuseMap
						2: curMap = mmat.decalMap
						3: curMap = mmat.specularMap
						4: curMap = mmat.selfIllumMap
						5: curMap = mmat.emisMap2
						6: curMap = mmat.envioMap
						7: curMap = mmat.envioMaskMap
						8: curMap = mmat.alphaMaskMap
						10: curMap = mmat.bumpMap
						11: curMap = mmat.heightMap
						1: curMap = mmat.normDisMap
						2: curMap = mmat.strenghtDisMap
					)
				)
				
				
				if (curMap != undefined) then
				(
					layr.maxBitmap = curMap
					local mapClass = classOf curMap as string
					
					if (mapClass == "sc2bitmap") then
					(
						local scBitmap
						local bool_CM = false
						
						try
						(
							if (curMap.scCustomMap.count > 0) then 
							(
								scBitmap = curMap.scCustomMap
								bool_CM = true
							)
							else if (curMap.scbitmap != undefined) then 
							(
								scBitmap = curMap.scbitmap.filename
								bool_CM = false
							)
						)
						catch
						(
							scBitmap = undefined
						)
						
						--if (scBitmap != undefined) then
						--(
							set time g_baseFrame
							layrBitmap = scBitmap
							
							local aflags = 0
							aflags = bit.set aflags 1 curMap.TeamColour
							aflags = bit.set aflags 2 curMap.TexAlphaOnly
							aflags = bit.set aflags 3 curMap.ShadeBasedOnAlpha
							
							layr.layerColour.initValue	= [((curMap.layerColour.x) * 255) as integer,((curMap.layerColour.y) * 255) as integer,((curMap.layerColour.z) * 255) as integer,((curMap.layerColour.w) * 255) as integer]
							layr.mappingSource			= (curMap.mappingSource - 1)
							layr.alphaFlags 				= aflags
							
							layr.flags 						= bit.set layr.flags 3 curMap.U_Tile
							layr.flags						= bit.set layr.flags 4 curMap.V_Tile
							layr.flags						= bit.set layr.flags 5 curMap.inverseMap
							layr.flags						= bit.set layr.flags 6 curMap.uLF1
							layr.flags						= bit.set layr.flags 7 curMap.uLF2
							layr.flags						= bit.set layr.flags 8 curMap.uLF3
							layr.flags						= bit.set layr.flags 11 curMap.useColour
							
							layr.UVOffset.initValue 		= [curMap.U_Offset, curMap.V_Offset]
							layr.UVTiling.initValue 		= [curMap.U_Tiling, curMap.V_Tiling]
							layr.bright_mult.initValue 	= curMap.Bright_Mult
							layr.MidtoneOffset.initValue = curMap.MidtoneOffset
							
							
							
							layr.FalloffStrength			= curMap.FalloffStrength
							layr.FalloffStart				= curMap.FalloffStart
							layr.FalloffCutout				= curMap.FalloffCutout
							layr.FalloffUnk					= curMap.FalloffUnk
							layr.unkMapInt				= curMap.unkMapInt
							
							layr.FalloffFlags				= bit.set layr.FalloffFlags 1 curMap.FalloffUse
							layr.FalloffFlags				= bit.set layr.FalloffFlags 2 curMap.FalloffAlpha
							layr.FalloffFlags				= bit.set layr.FalloffFlags 3 curMap.FalloffF3
							layr.FalloffFlags				= bit.set layr.FalloffFlags 4 curMap.FalloffF4
							
							if (bool_CM) then
							(
								layr.flags = bit.set layr.flags 10 false
								layr.d1 = -1
								layr.d2 = 1
							)
							
							
							-- handling certain maps
							/*if (k > 2) then
							(
								layr.unkMapInt = 0
							)*/
							
							
							-- setup vert flags for decals
							if (k == 2) then
							(
								if (curMap.coords.mapping == 0) then 
								(
									if (curMap.mappingSource > 1) then
									(
										if (bit.get m3scene.vertFlags 19 == false) then m3scene.vertFlags = bit.set m3scene.vertFlags 19 true
									)
								)
							)
						--)
						--else layrBitmap = undefined
					)
					else if (mapClass == "Bitmaptexture") then
					(
						layrBitmap = curMap.bitmap.filename
					)
					else
					(
						--messageBox "Map textures must be of Bitmap or Starcraft 2 Bitmap type!"
						throw "Map textures must be of Bitmap or Starcraft 2 Bitmap type"
					)
				)
				
				/*
				if (g_UseInternalPath == false) then
				(
					fullPath = layrBitmap
				)
				else
				(
				*/
					if (layrBitmap != undefined) then
					(
						fullPath = (M3E_checkPath texPath) + (M3E_Name layrBitmap "\\")
					)
					else
					(
						fullPath = undefined
					)
				--)
					
				layr.name = fullPath
				append mtl.layrs #(layr)
			)
		)
		
		if (matType == 3) then
		(
			
			-- 2. Starcraft 2 Material
			if (matclass == "sc2material") then
			(
				-- Flag set
				local flags = 0
				--{
				flags = bit.set flags 3 mmat.Unfogged
				flags = bit.set flags 4 mmat.TwoSided
				flags = bit.set flags 5 mmat.Unshaded
				flags = bit.set flags 6 mmat.NoShadowsCast
				flags = bit.set flags 7 mmat.NoHitTest
				flags = bit.set flags 8 mmat.NoShadowsReceived
				flags = bit.set flags 9 mmat.DepthPrepass
				flags = bit.set flags 10 mmat.UseTerrainHDR
				flags = bit.set flags 12 mmat.SplatUVfix
				flags = bit.set flags 13 mmat.SoftBlending
				flags = bit.set flags 15 mmat.ForParticles
				--}
				
				
				mtl.globStr.initValue			= mmat.globStr
				mtl.flags							= flags
				mtl.priorityDis					= mmat.priorityDis
			)
				
			-- Map handling
			for k = 1 to 2 do
			(
				local layr = M3E_LAYR.init()
				local fullPath = ""
				local layrBitmap
				local curMap
				
				-- 2. Starcraft 2 Material
				if (matclass == "sc2material") then
				(
					case k of
					(
						1: curMap = mmat.normDisMap
						2: curMap = mmat.strenghtDisMap
					)
				)
				
				
				if (curMap != undefined) then
				(
					layr.maxBitmap = curMap
					local mapClass = classOf curMap as string
					
					if (mapClass == "sc2bitmap") then
					(
						local scBitmap
						local bool_CM = false
						
						try
						(
							if (curMap.scCustomMap.count > 0) then 
							(
								scBitmap = curMap.scCustomMap
								bool_CM = true
							)
							else if (curMap.scbitmap != undefined) then 
							(
								scBitmap = curMap.scbitmap.filename
								bool_CM = false
							)
						)
						catch
						(
							scBitmap = undefined
						)
						
						--if (scBitmap != undefined) then
						--(
							set time g_baseFrame
							layrBitmap = scBitmap
							
							local aflags = 0
							aflags = bit.set aflags 1 curMap.TeamColour
							aflags = bit.set aflags 2 curMap.TexAlphaOnly
							aflags = bit.set aflags 3 curMap.ShadeBasedOnAlpha
							
							layr.layerColour.initValue	= [((curMap.layerColour.x) * 255) as integer,((curMap.layerColour.y) * 255) as integer,((curMap.layerColour.z) * 255) as integer,((curMap.layerColour.w) * 255) as integer]
							layr.mappingSource			= (curMap.mappingSource - 1)
							layr.alphaFlags 				= aflags
							
							layr.flags 						= bit.set layr.flags 3 curMap.U_Tile
							layr.flags						= bit.set layr.flags 4 curMap.V_Tile
							layr.flags						= bit.set layr.flags 5 curMap.inverseMap
							layr.flags						= bit.set layr.flags 6 curMap.uLF1
							layr.flags						= bit.set layr.flags 7 curMap.uLF2
							layr.flags						= bit.set layr.flags 8 curMap.uLF3
							layr.flags						= bit.set layr.flags 11 curMap.useColour
							
							layr.UVOffset.initValue 		= [curMap.U_Offset, curMap.V_Offset]
							layr.UVTiling.initValue 		= [curMap.U_Tiling, curMap.V_Tiling]
							layr.bright_mult.initValue 	= curMap.Bright_Mult
							layr.MidtoneOffset.initValue = curMap.MidtoneOffset
							
							
							
							layr.FalloffStrength			= curMap.FalloffStrength
							layr.FalloffStart				= curMap.FalloffStart
							layr.FalloffCutout				= curMap.FalloffCutout
							layr.FalloffUnk					= curMap.FalloffUnk
							layr.unkMapInt				= curMap.unkMapInt
							
							layr.FalloffFlags				= bit.set layr.FalloffFlags 1 curMap.FalloffUse
							layr.FalloffFlags				= bit.set layr.FalloffFlags 2 curMap.FalloffAlpha
							layr.FalloffFlags				= bit.set layr.FalloffFlags 3 curMap.FalloffF3
							layr.FalloffFlags				= bit.set layr.FalloffFlags 4 curMap.FalloffF4
							
							if (bool_CM) then
							(
								layr.flags = bit.set layr.flags 10 false
								layr.d1 = -1
								layr.d2 = 1
							)
							
							--if (curMap.mapChannel > 1) then layr.uvChannel = bit.set layr.uvChannel 1 true
							
							
							-- handling certain maps
							/*if (k > 2) then
							(
								layr.unkMapInt = 0
							)*/
							
							
							-- setup vert flags for decals
							if (k == 2) then
							(
								if (curMap.coords.mapping == 0) then 
								(
									if (curMap.mappingSource > 1) then
									(
										format "ustawia vertFlags na true\n"
										if (bit.get m3scene.vertFlags 19 == false) then m3scene.vertFlags = bit.set m3scene.vertFlags 19 true
									)
								)
							)
						--)
						--else layrBitmap = undefined
					)
					else if (mapClass == "Bitmaptexture") then
					(
						layrBitmap = curMap.bitmap.filename
					)
					else
					(
						--messageBox "Map textures must be of Bitmap or Starcraft 2 Bitmap type!"
						throw "Map textures must be of Bitmap or Starcraft 2 Bitmap type"
					)
				)
				
				/*
				if (g_UseInternalPath == false) then
				(
					fullPath = layrBitmap
				)
				else
				(
				*/
					if (layrBitmap != undefined) then
					(
						fullPath = (M3E_checkPath texPath) + (M3E_Name layrBitmap "\\")
					)
					else
					(
						fullPath = undefined
					)
				--)
					
				layr.name = fullPath
				append mtl.layrs #(layr)
			)
		)
		
		if (matType == 4) then
		(
			
			-- 2. Starcraft 2 Material
			if (matclass == "sc2material") then
			(
				mtl.volDensity.initValue		= mmat.volDensity
				--mtl.volFloat1					= mmat.volFloat1
				--mtl.volFloat2					= mmat.volFloat2
				--mtl.volInt1						= mmat.volInt1
				--mtl.volInt2						= mmat.volInt2
			)
				
			-- Map handling
			for k = 1 to 3 do
			(
				local layr = M3E_LAYR.init()
				local fullPath = ""
				local layrBitmap
				local curMap

				-- 2. Starcraft 2 Material
				if (matclass == "sc2material") then
				(
					case k of
					(
						1: curMap = mmat.vDiffMap
						2: curMap = mmat.vUnkn01
						3: curMap = mmat.vUnkn02
					)					
				)
				
				
				if (curMap != undefined) then
				(
					layr.maxBitmap = curMap
					local mapClass = classOf curMap as string
					
					if (mapClass == "sc2bitmap") then
					(
						local scBitmap
						local bool_CM = false
						
						try
						(
							if (curMap.scCustomMap.count > 0) then 
							(
								scBitmap = curMap.scCustomMap
								bool_CM = true
							)
							else if (curMap.scbitmap != undefined) then 
							(
								scBitmap = curMap.scbitmap.filename
								bool_CM = false
							)
						)
						catch
						(
							scBitmap = undefined
						)
						
						--if (scBitmap != undefined) then
						--(
							set time g_baseFrame
							layrBitmap = scBitmap
							
							local aflags = 0
							aflags = bit.set aflags 1 curMap.TeamColour
							aflags = bit.set aflags 2 curMap.TexAlphaOnly
							aflags = bit.set aflags 3 curMap.ShadeBasedOnAlpha
							
							layr.layerColour.initValue	= [((curMap.layerColour.x) * 255) as integer,((curMap.layerColour.y) * 255) as integer,((curMap.layerColour.z) * 255) as integer,((curMap.layerColour.w) * 255) as integer]
							layr.mappingSource			= (curMap.mappingSource - 1)
							layr.alphaFlags 				= aflags
							
							layr.flags 						= bit.set layr.flags 3 curMap.U_Tile
							layr.flags						= bit.set layr.flags 4 curMap.V_Tile
							layr.flags						= bit.set layr.flags 5 curMap.inverseMap
							layr.flags						= bit.set layr.flags 6 curMap.uLF1
							layr.flags						= bit.set layr.flags 7 curMap.uLF2
							layr.flags						= bit.set layr.flags 8 curMap.uLF3
							layr.flags						= bit.set layr.flags 11 curMap.useColour
							
							layr.UVOffset.initValue 		= [curMap.U_Offset, curMap.V_Offset]
							layr.UVTiling.initValue 		= [curMap.U_Tiling, curMap.V_Tiling]
							layr.bright_mult.initValue 	= curMap.Bright_Mult
							layr.MidtoneOffset.initValue = curMap.MidtoneOffset
							
							
							layr.FalloffStrength			= curMap.FalloffStrength
							layr.FalloffStart				= curMap.FalloffStart
							layr.FalloffCutout				= curMap.FalloffCutout
							layr.FalloffUnk					= curMap.FalloffUnk
							layr.unkMapInt				= curMap.unkMapInt
							
							layr.FalloffFlags				= bit.set layr.FalloffFlags 1 curMap.FalloffUse
							layr.FalloffFlags				= bit.set layr.FalloffFlags 2 curMap.FalloffAlpha
							layr.FalloffFlags				= bit.set layr.FalloffFlags 3 curMap.FalloffF3
							layr.FalloffFlags				= bit.set layr.FalloffFlags 4 curMap.FalloffF4
							
							if (bool_CM) then
							(
								layr.flags = bit.set layr.flags 10 false
								layr.d1 = -1
								layr.d2 = 1
							)
							
							--if (curMap.mapChannel > 1) then layr.uvChannel = bit.set layr.uvChannel 1 true
							
							
							-- handling certain maps
							/*if (k > 2) then
							(
								layr.unkMapInt = 0
							)*/
							
							
							
							-- setup vert flags for decals
							if (k == 2) then
							(
								if (curMap.coords.mapping == 0) then 
								(
									if (curMap.mappingSource > 1) then
									(
										format "ustawia vertFlags na true\n"
										if (bit.get m3scene.vertFlags 19 == false) then m3scene.vertFlags = bit.set m3scene.vertFlags 19 true
									)
								)
							)
						--)
						--else layrBitmap = undefined
					)
					else if (mapClass == "Bitmaptexture") then
					(
						layrBitmap = curMap.bitmap.filename
					)
					else
					(
						--messageBox "Map textures must be of Bitmap or Starcraft 2 Bitmap type!"
						throw "Map textures must be of Bitmap or Starcraft 2 Bitmap type"
					)
				)
				
				/*
				if (g_UseInternalPath == false) then
				(
					fullPath = layrBitmap
				)
				else
				(
				*/
					if (layrBitmap != undefined) then
					(
						fullPath = (M3E_checkPath texPath) + (M3E_Name layrBitmap "\\")
					)
					else
					(
						fullPath = undefined
					)
				--)
					
				layr.name = fullPath
				append mtl.layrs #(layr)
			)
		)
		
		mtl.maxMat = mmat
		append mtlarray mtl
	)
	
	--echo ("nMaterials: "+mtlarray.count as string)
	return mtlarray
)

fn M3E_buildMaterials m3data m3materials =
(
	-- initialise containers
	m3data.matm = #()
	m3data.mat = #()
	m3data.dis = #()
	m3data.ter = #()
	m3data.vol = #()
	
	for i = 1 to m3materials.count do
	(
		local m3mat = m3materials[i]
		local matm
		matm = M3E_MATM.init()
		
		if ((isKindOf m3mat M3E_Mat) == true) then 
		(
			matm.matType = 1
			matm.matInd = m3data.mat.count
			
			append m3data.mat m3mat
		)
		if ((isKindOf m3mat M3E_Ter) == true) then 
		(
			matm.matType = 4
			matm.matInd = m3data.ter.count
			
			append m3data.ter m3mat
		)
		if((isKindOf m3mat M3E_Dis) == true) then
		(
			matm.matType = 2
			matm.matInd = m3data.dis.count
			
			append m3data.dis m3mat
		)
		if((isKindOf m3mat M3E_Vol) == true) then
		(
			matm.matType = 5
			matm.matInd = m3data.vol.count
			
			append m3data.vol m3mat
		)

		append m3data.matm matm
	)
	
	-- need to fix material flags for particle emitter materials
	/*for i = 1 to m3data.par.count do 
	(
		local par = m3data.par[i]
		local parRC = par.numberOfColumns + par.numberOfRows -- need to check to see if the material has rows and columns
		
		if (parRC > 0) then
		(
			local matm = m3data.matm[par.matmIndex + 1]
			for j = 1 to 13 do 
			(
				if (m3data.mat[matm.matInd + 1].layrs[j][1].maxBitmap != undefined) then
				(
					m3data.mat[matm.matInd + 1].layrs[j][1].uvChannel = 6
				)
			)
		)
	)*/
)


fn M3E_buildBitmapAnims m3mats =
(
	echo "Building material bitmap animation data..."
	
	for i = 1 to m3mats.count do
	(
		local mat = m3mats[i]
		
		for j = 1 to mat.layrs.count do
		(
			local layr = mat.layrs[j][1]
			local maxBitmap = layr.maxBitmap
			
			if (maxBitmap != undefined) then
			(
				local mb = maxBitmap
				M3E_buildArefData layr.layerColour maxBitmap #layerColour #COLOUR
				M3E_buildArefData layr.bright_mult maxBitmap #Bright_Mult #FLOAT
				M3E_buildArefData layr.MidtoneOffset maxBitmap #MidtoneOffset #FLOAT
				
				local UVoffsetVec2D = #(#U_Offset, #V_Offset)
				M3E_buildArefData layr.UVOffset maxBitmap UVoffsetVec2D #VEC2D
				
				local UVtilingVec2D = #(#U_Tiling, #V_Tiling)
				M3E_buildArefData layr.UVTiling maxBitmap UVtilingVec2D #VEC2D
				
				
			)
			
			-- update original array
			m3mats[i].layrs[j][1] = layr
		)
	)
)

fn M3E_buildMaterialAnims m3mats = 
(
	echo "Building Material animation data..."
	
	for i = 1 to m3mats.count do
	(
		local mat = m3mats[i]
		local maxMat = m3mats[i].maxMat
		
		case (classOf mat) of
		(
			M3E_Dis:
			(
				M3E_buildArefData mat.globStr maxMat #globStr #FLOAT
			)
			M3E_Vol:
			(
				M3E_buildArefData mat.volDensity maxMat #volDensity #FLOAT
			)
		)
	)
)

---------------------
-- M3 HELPERS  --
---------------------

fn M3E_WriteRef bitStream data reftype subtype:#none flags:0  =
(
	local ref = M3E_Ref()
	-- Make sure it isn't a null reference...
	-- Determine if data is in array or singular
	local dclass = (classOf data) as string
	
	local dcount
	if (dclass != "Array" and dclass != "String") then dcount = 1 else dcount = data.count
	if (dclass == "String") then dcount += 1 -- for null-terminated string increase string size by 1
	if (data != undefined and dcount > 0 and reftype != #null) then
	(
		-- MODL bookmark
		local bm = ftell bitStream
		
		-- head to end of file to write reference data
		fseek bitStream 0 #seek_end
		
		-- Calculate chunk size
		local byte_size = (M3E_getSize reftype) * dcount
		
		-- Append to Tags
		-- need special VERT clause
		case reftype of
		(
			#VERT: 
			(
				case subtype of
				(
					#v36: byte_size += (dcount * 4)
				)
				M3E_Tag.TagAppend bitStream #U8__ (byte_size) twrite flags:flags
				ref.entries = byte_size
			)
			default: M3E_Tag.TagAppend bitStream reftype (dcount) twrite flags:flags
		)
		M3E_WriteBlank bitStream byte_size flags:#none ret:true
		ref.refid = twrite.count - 1 -- convert to base 0
		
		-- ** REF DEBUG **
		/* 
		echo ("reftype: "+reftype as string)
		echo ("refid: "+ref.refid as string)
		echo ("data.count: "+data.count as string)
		echo ("byte_size: "+byte_size as string)
		echo ("ftell: "+(ftell bitstream) as string)
		*/
		
		if (reftype != #CHAR) then
		(
			for i = 1 to dcount do
			(
				if (dclass != "Array") then d = data else d = data[i]
				case reftype of
				(
					#SEQS: M3E_SEQS.Write bitStream d
					#STC_: M3E_STC.Write bitStream d
					#SDEV: M3E_AnimBlock.Write bitStream d #EVNT flags:1
					#EVNT: M3E_Event.Write bitStream d
					#SD2V: M3E_AnimBlock.Write bitStream d #VEC2
					#VEC2: M3E_WriteVec bitStream d #v2d
					#SD3V: M3E_AnimBlock.Write bitStream d #VEC3
					#VEC3: M3E_WriteVec bitStream d #v3d
					#SD4Q: M3E_AnimBlock.Write bitStream d #QUAT
					#QUAT: M3E_WriteVec bitStream d #v4d
					#SDR3: M3E_AnimBlock.Write bitStream d #REAL
					#REAL: M3E_WriteArray bitStream d flags:#float
					#SDS6: M3E_AnimBlock.Write bitStream d #I16_
					#STG_: M3E_STG.Write bitStream d
					#STS_: M3E_STS.Write bitStream d
					#BONE: M3E_Bone.Write bitStream d
					#VERT: M3E_Vertex.Write bitStream d
					#REGN: M3E_REGN.Write bitStream d
					#BAT_: M3E_BAT.Write bitStream d
					#MSEC: M3E_MSEC.Write bitStream d
					#PAR_: M3E_PAR.Write bitStream d
					#PARC: M3E_PARC.Write bitStream d
					#RIB_: M3E_Ribbon.Write bitStream d
					#SRIB: M3E_SRIB.Write bitStream d
					#ATT_: M3E_Attachment.Write bitStream d
					#ATVL: M3E_AttachVolume.Write bitStream d
					#MATM: M3E_MATM.Write bitStream d
					#MAT_: M3E_MAT.Write bitStream d
					#DIS_: M3E_DIS.Write bitStream d
					#TER_: M3E_TER.Write bitStream d
					#VOL_: M3E_VOL.Write bitStream d
					#LAYR: M3E_LAYR.Write bitStream d
					#DIV_: M3E_DIV.Write bitStream d
					#IREF: M3E_WriteVec bitStream d #matrix
					#I16_: M3E_WriteArray bitStream d flags:#uint16
					#U16_: M3E_WriteArray bitStream d flags:subtype
					#U32_: M3E_WriteArray bitStream d flags:subtype
					#I32_: M3E_WriteArray bitStream d flags:subtype
					#CAM_: M3E_Cam.Write bitStream d
					#LITE: M3E_Light.Write bitStream d
					#SSGS: M3E_SSGS.Write bitStream d
				)
			)
		)
		else
		(
			WriteString bitStream data
			-- fix tag entry
			twrite[twrite.count].nData = byte_size
			ref.entries = byte_size
		)
		
		if (ref.entries == undefined) then ref.entries = dcount
		-- return to MODL bookmark
		fseek bitStream bm #seek_set
	)
	else
	(
		-- setup blank reference
		ref.entries = 0
		ref.refid = 0
	)
	
	-- write reference 
	M3E_Ref.Write bitStream ref
)

fn M3E_WriteArray bitStream data flags:#uint32 reftype:#null =
(
	-- if not array...
	local dclass = (classOf data) as string
	if (dclass != "Array") then icount = 1 else icount = data.count
	for i = 1 to icount do
	(
		local d
		if (dclass != "Array") then d = data else d = data[i]
		case flags of
		(
			#Ref: M3E_WriteRef bitStream d reftype
			#uint32: writeLong bitStream d #unsigned
			#int32: WriteLong bitStream d #signed
			#uint16: writeShort bitStream d #unsigned
			#int16: writeShort bitStream d #signed
			#uint8: writeByte bitStream d #unsigned
			#int8: writeByte bitStream d #signed
			#float: writeFloat bitStream d
			#v3d: M3E_WriteVec bitStream d #v3d
			#animind: M3E_AnimOffs.Write bitStream d
			#uvshorts: for uv = 1 to 2 do (writeShort bitStream d[uv] #unsigned)
		)
	)
)

fn M3E_WriteTight bitStream d =
(
	M3E_Tight.Write bitStream d
)

fn M3E_GetMODL m3obj =
(
	m3 = M3E_MODL()
	
	-- grab bound sphere for SEQS and MODL structs
	set time g_baseFrame
	m3.bndSphere = M3E_GetBndSphere()
	
	m3.versID = 1576275
	if (awrite != undefined) then 
	(
		M3E_buildAnimations m3 awrite
	)
	else
	(
		m3.SEQS = #(M3E_SEQS.init())
		m3.STC = #(M3E_STC.init())
		m3.STG = #(M3E_STG.init())
		m3.STS = #(M3E_STS.initBlank m3)
	)
	m3.vec1 = [0,0,0]
	m3.cameras = m3obj.cameras;
	m3.bones = m3obj.bones

	local nSkinBones = 0
	for i = 1 to m3obj.skinBones.count do
	(
		local skinBoneInd = m3obj.skinBones[i] + 1
		if (skinBoneInd > nSkinBones) then nSkinBones = m3obj.skinBones[i] + 1
	)
	m3.nSkinBones = nSkinBones

	-- grab submeshes first
	m3.DIV = m3obj.submeshes
	
	-- build verts based on submesh list
	m3.verts = m3obj.verts
	m3.vflags = m3obj.vflags
	m3.bonelu = m3obj.skinBones
	
	if (m3obj.attachments != undefined) then
	(
		M3E_buildAttachments m3 m3obj.attachments
	)
	
	m3.PAR = m3obj.pemits
	m3.PARC = m3obj.parcs
	m3.sc2ribbon = m3obj.sc2ribbons
	m3.sc2force = m3obj.sc2forces
	m3.sc2lights = m3obj.sc2lights
	M3E_buildMaterials m3 m3obj.materials
	
	m3.iref = #()
	
	for i = 1 to m3.bones.count do
	(
		append m3.iref m3.bones[i].iref
	)
	
	m3.sc2tight = m3obj.sc2tight
	m3.sc2fuzzy = m3obj.sc2fuzzy
	
	return m3
)

fn M3E_getScene =
(
	step = "Gathering Scene Objects"
	echo "Gathering scene objects..."

	m3model = M3E_Model.init()
	m3scene = M3E_getSceneObjects()
		
	m3scene.skin = M3E_getSkin m3scene
	m3model.bones = M3E_getBones m3scene
	m3scene.materials = M3E_getUsedMats m3scene
	m3model.materials = M3E_getMaterials m3scene.materials
	
	cams = for cam in cameras where classof cam != TargetObject collect cam
	m3model.cameras = cams
	
	m3model.submeshes = M3E_getSubmesh m3scene
	m3model.verts = M3E_getVertices m3scene m3model
	m3model.vflags = m3scene.vertFlags
	
	M3E_buildDIV m3model 	-- Build REGN's for model DIV
	
	M3E_buildBones m3model -- Construct bone array
	M3E_buildCameras m3model;
	
	m3model.attachments = M3E_getObjects m3scene m3model #attachment -- Get attachments
	m3model.pemits = M3E_getObjects m3scene m3model #sc2pemitter2 -- Get particle emitters
	m3model.parcs = M3E_getObjects m3scene m3model #pinstance -- Get particle emitter instances
	m3model.sc2ribbons = M3E_getObjects m3scene m3model #sc2ribbon --Get ribbons
	m3model.sc2forces = M3E_getObjects m3scene m3model #sc2force --Get forces
	m3model.sc2lights = M3E_getObjects m3scene m3model #sc2light --Get lights
	m3model.sc2tight = M3E_getObjects m3scene m3model #sc2tight --Get tight hit test
	m3model.sc2fuzzy = M3E_getObjects m3scene m3model #sc2fuzzy --Get fuzzy hit tests
	
	if (g_exportAnimation == true) then awrite = M3E_getAnimations()
	
	if (awrite != undefined) then
	(
		M3E_buildBoneAnims m3model.bones
		M3E_buildPemitAnims m3model.pemits
		M3E_buildParcAnims m3model.parcs
		M3E_buildRibbonAnims m3model.sc2ribbons
		M3E_buildForcesAnims m3model.sc2forces
		M3E_buildBitmapAnims m3model.materials
		M3E_buildMaterialAnims m3model.materials
		M3E_buildLightAnims m3model.sc2lights
	)
	echo "Gathering scene data done!"
)

fn M3E_buildM3 m3obj =
(
	step = "Build M3"
	format "Building M3 structures..." to:listener
	head = M3E_Header.init()
	-- Gather MODL data
	-- assign global, will update this structure throughout export
	mwrite = M3E_GetMODL m3obj
	
	echo "done!"
)

fn M3E_Start_Write bitStream =
(
	step = "Start Model Write"

	-- write Header and blank bytes and tag reference
	echo "Writing buffers..."
	M3E_Tag.TagAppend bitStream #MD34 1 twrite flags:11
	M3E_WriteBlank bstream (M3E_getSize #MD34) flags:#none
	
	-- Create MODL tag, write MODL blank bytes
	-- using model version 23
	M3E_Tag.TagAppend bitStream #MODL 1 twrite flags:23	
	M3E_WriteBlank bstream (M3E_getSize #MODL flags:23) flags:#none
	
	echo "Writing model data..."
	-- Write Model Data first
	M3E_MODL.Write bitStream mwrite 23	

	echo "Writing tag data..."
	-- Write Tags second...
	M3E_Tag.Write bitStream twrite
	
	echo "Writing header data..."
	-- ... and write Header last
	M3E_Header.Write bitStream head
	echo "Model writing done!\n"
)

fn M3E_ScriptTime start end =
(
	local totTime = (end - start) / 1000.0
	local totMinutes = (totTime / 60) as integer
	if (totMinutes > 0) then 
	(
		local strMinutes
		if (totMinutes > 1) then strMinutes = " minutes" else strMinutes = " minute"
		local totRemainder = mod totTime 60.0
		totTime = (totMinutes as string) + strMinutes + " " + (totRemainder as string) + " seconds"
	)
	else
	(
		totTime = (totTime as string) + " seconds"
	)
	
	return totTime
)

fn M3E_Main file =
(
-- 	try 
-- 	(
		if (SC2PLUGIN_VERS == undefined) then
		(
			throw ("Starcraft 2 Object definition file not found\nMake sure sc2_objects.ms is loaded before trying to export")
		)
		
		echo "\n====== Export Starting ======\n"
		M3E_Reset_Globals()
		scrStart = TimeStamp() -- start tracking script processing time
		M3E_getScene()
		M3E_buildM3(m3model)
		local mname = (M3E_Name file "\\" ext:".m3") + ".max"
		mwrite.name = mname
	
		echo "\n==== Model Information ===="
		echo ("Mesh Name: "+mwrite.name)
		if (awrite != undefined) then (echo ("Animations: "+awrite.count as string))
		echo ("Vertices: "+mwrite.verts.count as string)
		--echo ("Faces: "+m3model.mesh.numfaces as string)
		echo ("Submeshes: "+m3model.submeshes.regn.count as string)
		echo ("Materials: "+m3model.materials.count as string)
		echo ("Bones: "+m3model.bones.count as string)
		echo ("Skinned bones: "+m3model.skinBones.count as string)
		echo "" -- spacer
		
		bstream = M3E_Open file "wb"
		M3E_Start_Write bstream
		M3E_Close bstream
		
		-- Calculate Script Execution time
		scrEnd = TimeStamp()
		local scrTime = M3E_ScriptTime scrStart scrEnd
			
		format "Export took %\n" scrTime
		echo "====== Export Successful ======\n"
		messageBox ("Export Successful\nExport took "+scrTime) title:"Export successful"
-- 	)
-- 	catch
-- 	(
-- 		echo ("****"+getCurrentException()+"****")
-- 		echo "====== Export Failed ======\n"
-- 		messageBox ("Export failed!\n"+getCurrentException() as string) title:"Export failed"
-- 	)
	
	clearSelection()
	setCommandPanelTaskMode #utility
	--M3E_Reset_Globals()
)

-- *********************************
--  USER INTERFACE FUNCTIONS
-- *********************************
-- Glob UI Funcs
fn M3E_GetFile ftypes =
(
	local ret = getSaveFileName types:ftypes
	if ret != undefined then return ret
)

utility M3E_export "M3 - Export"
(
	group "Scene Options"
	(
		checkbox chkExpHidden "Export Hidden Objects" checked:(g_exportHidden)
		checkbox chkExpFrozen "Export Frozen Objects" checked:(g_exportFrozen)
	)
	
	group "Texture Options"
	(
		label	lblTexPath "Internal Texture Path:" align:#left
		edittext tTexPath text:texPath
		--button bBrowsePath "Browse"
		--checkbox chkUseInternalPath "Use Internal Path" checked:(g_UseInternalPath)
		checkbox chkFlipUVY "Flip UV Y-coord" checked:(g_flipuvy)
	)
	
	group "Animation Options"
	(	
		label lblPoses "Pose Frames" align:#left
		spinner spnBindposeFrame "Bindpose:" range:[0,1e8,0] type:#integer
		spinner spnBaseFrame "Base:" range:[0,1e8,3] type:#integer
		checkbox chkExpAnim "Export Animation" checked:(g_exportAnimation)
		spinner spnUseFPS "FPS:" range:[0,1e8,1000] type:#integer
	)
	
	group "Scene check"
	(
		button bExpErrors "Scene check" height:21 width:100 offset:[0,2] tooltip:"Runs a series of scene tests to check for errors.\n\nTest will be run only on StarCraft II compatible objects."
		button addBones "Add bones"  height:21 width:100 offset:[0,2] tooltip:"If any of selected objects is missing a bone, adds one in exact position of an object.\n\nBones will be added only to StarCraft II compatible objects."
	)
	
	button bExport "Export" height:35 width:100 offset:[0,5]
	
	button bAbout "About" height:21 width:100 offset:[-1,0]
	
	on bExpErrors pressed do
	(
		M3E_SceneCheck false
	)
	
	on addBones pressed do
	(
		format "adding bones\n"
		M3E_AddBones()
	)
	
	on M3E_export open do
	(
		/*
		if (chkUseInternalPath.checked == true) then
		(
			tTexPath.enabled = true
			--bBrowsePath.enabled = true
		)
		else
		(
			tTexPath.enabled = false
			--bBrowsePath.enabled = false			
		)
		*/
		if (chkExpAnim.checked == true) then spnUseFPS.enabled = true else spnUseFPS.enabled = false
	)
	on bBrowsePath pressed do
	(
		local tPath = getSavePath caption:"Texture Path"
		if (tPath != undefined) then tTexPath.text = tPath
	)
	/*
	on chkUseInternalPath changed state do
	(
		if state == on then
		(
			tTexPath.enabled = true
			--bBrowsePath.enabled = true
		)
		else
		(
			tTexPath.enabled = false
			--bBrowsePath.enabled = false
		)
	)
	*/
	on chkExpAnim changed state do
	(
		if state == on then
		(
			spnUseFPS.enabled = true
		)
		else
		(
			spnUseFPS.enabled = false
		)
	)
	on bExport pressed do
	(
		--M3E_SceneCheck true
		local fname = getSaveFileName types:"M3 model (*.m3)|*.m3|All Files|*.*|"
		--local fname = "d:\_Projects\SCII_imp_exp\exp_tst\ribb.m3"
		format "Fname: %\n" fname
		if fname != undefined then
		(
			g_exportHidden = chkExpHidden.checked
			g_exportFrozen = chkExpFrozen.checked
			--g_UseInternalPath = chkUseInternalPath.checked
			g_flipuvy = chkFlipUVY.checked
			texPath = tTexPath.text
			g_bindposeFrame = spnBindposeFrame.value
			g_baseFrame = spnBaseFrame.value
			g_exportAnimation = chkExpAnim.checked
			g_useFPS = spnUseFPS.value
			
			M3E_Main(fname)
		)
	)
	on bAbout pressed do
	(
		messagebox "M3 Exporter v0.22 - 05-03-2011\n\xa9 2010, NiNtoxicated (madyavic@gmail.com)\nVisit www.sc2mapster.com for updates and more information\nHead to the forum for support. Feedback is appreciated!" title:"About"
	)
)

--clearListener()